/***********************************************************
* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
*----------------------------------------------------------*
* (c) 1995-2006 Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* conf_cus.h                                               *
***********************************************************/

/* ----- this file was automatically generated -- do not edit by hand ----- */

#ifndef CONF_CUS_H
#define CONF_CUS_H

/* values for elements configuration (custom elements) */

#define EL_CUSTOM_1			(EL_CUSTOM_START + 0)
#define EL_CUSTOM_2			(EL_CUSTOM_START + 1)
#define EL_CUSTOM_3			(EL_CUSTOM_START + 2)
#define EL_CUSTOM_4			(EL_CUSTOM_START + 3)
#define EL_CUSTOM_5			(EL_CUSTOM_START + 4)
#define EL_CUSTOM_6			(EL_CUSTOM_START + 5)
#define EL_CUSTOM_7			(EL_CUSTOM_START + 6)
#define EL_CUSTOM_8			(EL_CUSTOM_START + 7)
#define EL_CUSTOM_9			(EL_CUSTOM_START + 8)
#define EL_CUSTOM_10			(EL_CUSTOM_START + 9)
#define EL_CUSTOM_11			(EL_CUSTOM_START + 10)
#define EL_CUSTOM_12			(EL_CUSTOM_START + 11)
#define EL_CUSTOM_13			(EL_CUSTOM_START + 12)
#define EL_CUSTOM_14			(EL_CUSTOM_START + 13)
#define EL_CUSTOM_15			(EL_CUSTOM_START + 14)
#define EL_CUSTOM_16			(EL_CUSTOM_START + 15)
#define EL_CUSTOM_17			(EL_CUSTOM_START + 16)
#define EL_CUSTOM_18			(EL_CUSTOM_START + 17)
#define EL_CUSTOM_19			(EL_CUSTOM_START + 18)
#define EL_CUSTOM_20			(EL_CUSTOM_START + 19)
#define EL_CUSTOM_21			(EL_CUSTOM_START + 20)
#define EL_CUSTOM_22			(EL_CUSTOM_START + 21)
#define EL_CUSTOM_23			(EL_CUSTOM_START + 22)
#define EL_CUSTOM_24			(EL_CUSTOM_START + 23)
#define EL_CUSTOM_25			(EL_CUSTOM_START + 24)
#define EL_CUSTOM_26			(EL_CUSTOM_START + 25)
#define EL_CUSTOM_27			(EL_CUSTOM_START + 26)
#define EL_CUSTOM_28			(EL_CUSTOM_START + 27)
#define EL_CUSTOM_29			(EL_CUSTOM_START + 28)
#define EL_CUSTOM_30			(EL_CUSTOM_START + 29)
#define EL_CUSTOM_31			(EL_CUSTOM_START + 30)
#define EL_CUSTOM_32			(EL_CUSTOM_START + 31)
#define EL_CUSTOM_33			(EL_CUSTOM_START + 32)
#define EL_CUSTOM_34			(EL_CUSTOM_START + 33)
#define EL_CUSTOM_35			(EL_CUSTOM_START + 34)
#define EL_CUSTOM_36			(EL_CUSTOM_START + 35)
#define EL_CUSTOM_37			(EL_CUSTOM_START + 36)
#define EL_CUSTOM_38			(EL_CUSTOM_START + 37)
#define EL_CUSTOM_39			(EL_CUSTOM_START + 38)
#define EL_CUSTOM_40			(EL_CUSTOM_START + 39)
#define EL_CUSTOM_41			(EL_CUSTOM_START + 40)
#define EL_CUSTOM_42			(EL_CUSTOM_START + 41)
#define EL_CUSTOM_43			(EL_CUSTOM_START + 42)
#define EL_CUSTOM_44			(EL_CUSTOM_START + 43)
#define EL_CUSTOM_45			(EL_CUSTOM_START + 44)
#define EL_CUSTOM_46			(EL_CUSTOM_START + 45)
#define EL_CUSTOM_47			(EL_CUSTOM_START + 46)
#define EL_CUSTOM_48			(EL_CUSTOM_START + 47)
#define EL_CUSTOM_49			(EL_CUSTOM_START + 48)
#define EL_CUSTOM_50			(EL_CUSTOM_START + 49)
#define EL_CUSTOM_51			(EL_CUSTOM_START + 50)
#define EL_CUSTOM_52			(EL_CUSTOM_START + 51)
#define EL_CUSTOM_53			(EL_CUSTOM_START + 52)
#define EL_CUSTOM_54			(EL_CUSTOM_START + 53)
#define EL_CUSTOM_55			(EL_CUSTOM_START + 54)
#define EL_CUSTOM_56			(EL_CUSTOM_START + 55)
#define EL_CUSTOM_57			(EL_CUSTOM_START + 56)
#define EL_CUSTOM_58			(EL_CUSTOM_START + 57)
#define EL_CUSTOM_59			(EL_CUSTOM_START + 58)
#define EL_CUSTOM_60			(EL_CUSTOM_START + 59)
#define EL_CUSTOM_61			(EL_CUSTOM_START + 60)
#define EL_CUSTOM_62			(EL_CUSTOM_START + 61)
#define EL_CUSTOM_63			(EL_CUSTOM_START + 62)
#define EL_CUSTOM_64			(EL_CUSTOM_START + 63)
#define EL_CUSTOM_65			(EL_CUSTOM_START + 64)
#define EL_CUSTOM_66			(EL_CUSTOM_START + 65)
#define EL_CUSTOM_67			(EL_CUSTOM_START + 66)
#define EL_CUSTOM_68			(EL_CUSTOM_START + 67)
#define EL_CUSTOM_69			(EL_CUSTOM_START + 68)
#define EL_CUSTOM_70			(EL_CUSTOM_START + 69)
#define EL_CUSTOM_71			(EL_CUSTOM_START + 70)
#define EL_CUSTOM_72			(EL_CUSTOM_START + 71)
#define EL_CUSTOM_73			(EL_CUSTOM_START + 72)
#define EL_CUSTOM_74			(EL_CUSTOM_START + 73)
#define EL_CUSTOM_75			(EL_CUSTOM_START + 74)
#define EL_CUSTOM_76			(EL_CUSTOM_START + 75)
#define EL_CUSTOM_77			(EL_CUSTOM_START + 76)
#define EL_CUSTOM_78			(EL_CUSTOM_START + 77)
#define EL_CUSTOM_79			(EL_CUSTOM_START + 78)
#define EL_CUSTOM_80			(EL_CUSTOM_START + 79)
#define EL_CUSTOM_81			(EL_CUSTOM_START + 80)
#define EL_CUSTOM_82			(EL_CUSTOM_START + 81)
#define EL_CUSTOM_83			(EL_CUSTOM_START + 82)
#define EL_CUSTOM_84			(EL_CUSTOM_START + 83)
#define EL_CUSTOM_85			(EL_CUSTOM_START + 84)
#define EL_CUSTOM_86			(EL_CUSTOM_START + 85)
#define EL_CUSTOM_87			(EL_CUSTOM_START + 86)
#define EL_CUSTOM_88			(EL_CUSTOM_START + 87)
#define EL_CUSTOM_89			(EL_CUSTOM_START + 88)
#define EL_CUSTOM_90			(EL_CUSTOM_START + 89)
#define EL_CUSTOM_91			(EL_CUSTOM_START + 90)
#define EL_CUSTOM_92			(EL_CUSTOM_START + 91)
#define EL_CUSTOM_93			(EL_CUSTOM_START + 92)
#define EL_CUSTOM_94			(EL_CUSTOM_START + 93)
#define EL_CUSTOM_95			(EL_CUSTOM_START + 94)
#define EL_CUSTOM_96			(EL_CUSTOM_START + 95)
#define EL_CUSTOM_97			(EL_CUSTOM_START + 96)
#define EL_CUSTOM_98			(EL_CUSTOM_START + 97)
#define EL_CUSTOM_99			(EL_CUSTOM_START + 98)
#define EL_CUSTOM_100			(EL_CUSTOM_START + 99)
#define EL_CUSTOM_101			(EL_CUSTOM_START + 100)
#define EL_CUSTOM_102			(EL_CUSTOM_START + 101)
#define EL_CUSTOM_103			(EL_CUSTOM_START + 102)
#define EL_CUSTOM_104			(EL_CUSTOM_START + 103)
#define EL_CUSTOM_105			(EL_CUSTOM_START + 104)
#define EL_CUSTOM_106			(EL_CUSTOM_START + 105)
#define EL_CUSTOM_107			(EL_CUSTOM_START + 106)
#define EL_CUSTOM_108			(EL_CUSTOM_START + 107)
#define EL_CUSTOM_109			(EL_CUSTOM_START + 108)
#define EL_CUSTOM_110			(EL_CUSTOM_START + 109)
#define EL_CUSTOM_111			(EL_CUSTOM_START + 110)
#define EL_CUSTOM_112			(EL_CUSTOM_START + 111)
#define EL_CUSTOM_113			(EL_CUSTOM_START + 112)
#define EL_CUSTOM_114			(EL_CUSTOM_START + 113)
#define EL_CUSTOM_115			(EL_CUSTOM_START + 114)
#define EL_CUSTOM_116			(EL_CUSTOM_START + 115)
#define EL_CUSTOM_117			(EL_CUSTOM_START + 116)
#define EL_CUSTOM_118			(EL_CUSTOM_START + 117)
#define EL_CUSTOM_119			(EL_CUSTOM_START + 118)
#define EL_CUSTOM_120			(EL_CUSTOM_START + 119)
#define EL_CUSTOM_121			(EL_CUSTOM_START + 120)
#define EL_CUSTOM_122			(EL_CUSTOM_START + 121)
#define EL_CUSTOM_123			(EL_CUSTOM_START + 122)
#define EL_CUSTOM_124			(EL_CUSTOM_START + 123)
#define EL_CUSTOM_125			(EL_CUSTOM_START + 124)
#define EL_CUSTOM_126			(EL_CUSTOM_START + 125)
#define EL_CUSTOM_127			(EL_CUSTOM_START + 126)
#define EL_CUSTOM_128			(EL_CUSTOM_START + 127)
#define EL_CUSTOM_129			(EL_CUSTOM_START + 128)
#define EL_CUSTOM_130			(EL_CUSTOM_START + 129)
#define EL_CUSTOM_131			(EL_CUSTOM_START + 130)
#define EL_CUSTOM_132			(EL_CUSTOM_START + 131)
#define EL_CUSTOM_133			(EL_CUSTOM_START + 132)
#define EL_CUSTOM_134			(EL_CUSTOM_START + 133)
#define EL_CUSTOM_135			(EL_CUSTOM_START + 134)
#define EL_CUSTOM_136			(EL_CUSTOM_START + 135)
#define EL_CUSTOM_137			(EL_CUSTOM_START + 136)
#define EL_CUSTOM_138			(EL_CUSTOM_START + 137)
#define EL_CUSTOM_139			(EL_CUSTOM_START + 138)
#define EL_CUSTOM_140			(EL_CUSTOM_START + 139)
#define EL_CUSTOM_141			(EL_CUSTOM_START + 140)
#define EL_CUSTOM_142			(EL_CUSTOM_START + 141)
#define EL_CUSTOM_143			(EL_CUSTOM_START + 142)
#define EL_CUSTOM_144			(EL_CUSTOM_START + 143)
#define EL_CUSTOM_145			(EL_CUSTOM_START + 144)
#define EL_CUSTOM_146			(EL_CUSTOM_START + 145)
#define EL_CUSTOM_147			(EL_CUSTOM_START + 146)
#define EL_CUSTOM_148			(EL_CUSTOM_START + 147)
#define EL_CUSTOM_149			(EL_CUSTOM_START + 148)
#define EL_CUSTOM_150			(EL_CUSTOM_START + 149)
#define EL_CUSTOM_151			(EL_CUSTOM_START + 150)
#define EL_CUSTOM_152			(EL_CUSTOM_START + 151)
#define EL_CUSTOM_153			(EL_CUSTOM_START + 152)
#define EL_CUSTOM_154			(EL_CUSTOM_START + 153)
#define EL_CUSTOM_155			(EL_CUSTOM_START + 154)
#define EL_CUSTOM_156			(EL_CUSTOM_START + 155)
#define EL_CUSTOM_157			(EL_CUSTOM_START + 156)
#define EL_CUSTOM_158			(EL_CUSTOM_START + 157)
#define EL_CUSTOM_159			(EL_CUSTOM_START + 158)
#define EL_CUSTOM_160			(EL_CUSTOM_START + 159)
#define EL_CUSTOM_161			(EL_CUSTOM_START + 160)
#define EL_CUSTOM_162			(EL_CUSTOM_START + 161)
#define EL_CUSTOM_163			(EL_CUSTOM_START + 162)
#define EL_CUSTOM_164			(EL_CUSTOM_START + 163)
#define EL_CUSTOM_165			(EL_CUSTOM_START + 164)
#define EL_CUSTOM_166			(EL_CUSTOM_START + 165)
#define EL_CUSTOM_167			(EL_CUSTOM_START + 166)
#define EL_CUSTOM_168			(EL_CUSTOM_START + 167)
#define EL_CUSTOM_169			(EL_CUSTOM_START + 168)
#define EL_CUSTOM_170			(EL_CUSTOM_START + 169)
#define EL_CUSTOM_171			(EL_CUSTOM_START + 170)
#define EL_CUSTOM_172			(EL_CUSTOM_START + 171)
#define EL_CUSTOM_173			(EL_CUSTOM_START + 172)
#define EL_CUSTOM_174			(EL_CUSTOM_START + 173)
#define EL_CUSTOM_175			(EL_CUSTOM_START + 174)
#define EL_CUSTOM_176			(EL_CUSTOM_START + 175)
#define EL_CUSTOM_177			(EL_CUSTOM_START + 176)
#define EL_CUSTOM_178			(EL_CUSTOM_START + 177)
#define EL_CUSTOM_179			(EL_CUSTOM_START + 178)
#define EL_CUSTOM_180			(EL_CUSTOM_START + 179)
#define EL_CUSTOM_181			(EL_CUSTOM_START + 180)
#define EL_CUSTOM_182			(EL_CUSTOM_START + 181)
#define EL_CUSTOM_183			(EL_CUSTOM_START + 182)
#define EL_CUSTOM_184			(EL_CUSTOM_START + 183)
#define EL_CUSTOM_185			(EL_CUSTOM_START + 184)
#define EL_CUSTOM_186			(EL_CUSTOM_START + 185)
#define EL_CUSTOM_187			(EL_CUSTOM_START + 186)
#define EL_CUSTOM_188			(EL_CUSTOM_START + 187)
#define EL_CUSTOM_189			(EL_CUSTOM_START + 188)
#define EL_CUSTOM_190			(EL_CUSTOM_START + 189)
#define EL_CUSTOM_191			(EL_CUSTOM_START + 190)
#define EL_CUSTOM_192			(EL_CUSTOM_START + 191)
#define EL_CUSTOM_193			(EL_CUSTOM_START + 192)
#define EL_CUSTOM_194			(EL_CUSTOM_START + 193)
#define EL_CUSTOM_195			(EL_CUSTOM_START + 194)
#define EL_CUSTOM_196			(EL_CUSTOM_START + 195)
#define EL_CUSTOM_197			(EL_CUSTOM_START + 196)
#define EL_CUSTOM_198			(EL_CUSTOM_START + 197)
#define EL_CUSTOM_199			(EL_CUSTOM_START + 198)
#define EL_CUSTOM_200			(EL_CUSTOM_START + 199)
#define EL_CUSTOM_201			(EL_CUSTOM_START + 200)
#define EL_CUSTOM_202			(EL_CUSTOM_START + 201)
#define EL_CUSTOM_203			(EL_CUSTOM_START + 202)
#define EL_CUSTOM_204			(EL_CUSTOM_START + 203)
#define EL_CUSTOM_205			(EL_CUSTOM_START + 204)
#define EL_CUSTOM_206			(EL_CUSTOM_START + 205)
#define EL_CUSTOM_207			(EL_CUSTOM_START + 206)
#define EL_CUSTOM_208			(EL_CUSTOM_START + 207)
#define EL_CUSTOM_209			(EL_CUSTOM_START + 208)
#define EL_CUSTOM_210			(EL_CUSTOM_START + 209)
#define EL_CUSTOM_211			(EL_CUSTOM_START + 210)
#define EL_CUSTOM_212			(EL_CUSTOM_START + 211)
#define EL_CUSTOM_213			(EL_CUSTOM_START + 212)
#define EL_CUSTOM_214			(EL_CUSTOM_START + 213)
#define EL_CUSTOM_215			(EL_CUSTOM_START + 214)
#define EL_CUSTOM_216			(EL_CUSTOM_START + 215)
#define EL_CUSTOM_217			(EL_CUSTOM_START + 216)
#define EL_CUSTOM_218			(EL_CUSTOM_START + 217)
#define EL_CUSTOM_219			(EL_CUSTOM_START + 218)
#define EL_CUSTOM_220			(EL_CUSTOM_START + 219)
#define EL_CUSTOM_221			(EL_CUSTOM_START + 220)
#define EL_CUSTOM_222			(EL_CUSTOM_START + 221)
#define EL_CUSTOM_223			(EL_CUSTOM_START + 222)
#define EL_CUSTOM_224			(EL_CUSTOM_START + 223)
#define EL_CUSTOM_225			(EL_CUSTOM_START + 224)
#define EL_CUSTOM_226			(EL_CUSTOM_START + 225)
#define EL_CUSTOM_227			(EL_CUSTOM_START + 226)
#define EL_CUSTOM_228			(EL_CUSTOM_START + 227)
#define EL_CUSTOM_229			(EL_CUSTOM_START + 228)
#define EL_CUSTOM_230			(EL_CUSTOM_START + 229)
#define EL_CUSTOM_231			(EL_CUSTOM_START + 230)
#define EL_CUSTOM_232			(EL_CUSTOM_START + 231)
#define EL_CUSTOM_233			(EL_CUSTOM_START + 232)
#define EL_CUSTOM_234			(EL_CUSTOM_START + 233)
#define EL_CUSTOM_235			(EL_CUSTOM_START + 234)
#define EL_CUSTOM_236			(EL_CUSTOM_START + 235)
#define EL_CUSTOM_237			(EL_CUSTOM_START + 236)
#define EL_CUSTOM_238			(EL_CUSTOM_START + 237)
#define EL_CUSTOM_239			(EL_CUSTOM_START + 238)
#define EL_CUSTOM_240			(EL_CUSTOM_START + 239)
#define EL_CUSTOM_241			(EL_CUSTOM_START + 240)
#define EL_CUSTOM_242			(EL_CUSTOM_START + 241)
#define EL_CUSTOM_243			(EL_CUSTOM_START + 242)
#define EL_CUSTOM_244			(EL_CUSTOM_START + 243)
#define EL_CUSTOM_245			(EL_CUSTOM_START + 244)
#define EL_CUSTOM_246			(EL_CUSTOM_START + 245)
#define EL_CUSTOM_247			(EL_CUSTOM_START + 246)
#define EL_CUSTOM_248			(EL_CUSTOM_START + 247)
#define EL_CUSTOM_249			(EL_CUSTOM_START + 248)
#define EL_CUSTOM_250			(EL_CUSTOM_START + 249)
#define EL_CUSTOM_251			(EL_CUSTOM_START + 250)
#define EL_CUSTOM_252			(EL_CUSTOM_START + 251)
#define EL_CUSTOM_253			(EL_CUSTOM_START + 252)
#define EL_CUSTOM_254			(EL_CUSTOM_START + 253)
#define EL_CUSTOM_255			(EL_CUSTOM_START + 254)
#define EL_CUSTOM_256			(EL_CUSTOM_START + 255)

#endif	/* CONF_CUS_C */
