/***********************************************************
* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
*----------------------------------------------------------*
* (c) 1995-2006 Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* conf_snd.h                                               *
***********************************************************/

/* ----- this file was automatically generated -- do not edit by hand ----- */

#ifndef CONF_SND_H
#define CONF_SND_H

/* values for sounds configuration */

#define SND_CLASS_DEFAULT_DIGGING			0
#define SND_CLASS_DEFAULT_COLLECTING			1
#define SND_CLASS_DEFAULT_SNAPPING			2
#define SND_CLASS_DEFAULT_PUSHING			3
#define SND_CLASS_DEFAULT_IMPACT			4
#define SND_CLASS_DEFAULT_WALKING			5
#define SND_CLASS_DEFAULT_PASSING			6
#define SND_CLASS_DEFAULT_DYING				7
#define SND_CLASS_DEFAULT_EXPLODING			8
#define SND_CLASS_SP_DEFAULT_EXPLODING			9
#define SND_BD_DIAMOND_COLLECTING			10
#define SND_BD_DIAMOND_IMPACT				11
#define SND_BD_ROCK_PUSHING				12
#define SND_BD_ROCK_IMPACT				13
#define SND_BD_MAGIC_WALL_ACTIVATING			14
#define SND_BD_MAGIC_WALL_ACTIVE			15
#define SND_BD_MAGIC_WALL_FILLING			16
#define SND_BD_AMOEBA_WAITING				17
#define SND_BD_AMOEBA_GROWING				18
#define SND_BD_AMOEBA_TURNING_TO_GEM			19
#define SND_BD_AMOEBA_TURNING_TO_ROCK			20
#define SND_BD_BUTTERFLY_MOVING				21
#define SND_BD_BUTTERFLY_WAITING			22
#define SND_BD_FIREFLY_MOVING				23
#define SND_BD_FIREFLY_WAITING				24
#define SND_SP_BASE_DIGGING				25
#define SND_SP_BUGGY_BASE_DIGGING			26
#define SND_SP_BUGGY_BASE_ACTIVE			27
#define SND_SP_INFOTRON_COLLECTING			28
#define SND_SP_INFOTRON_IMPACT				29
#define SND_SP_ZONK_PUSHING				30
#define SND_SP_ZONK_IMPACT				31
#define SND_SP_DISK_RED_COLLECTING			32
#define SND_SP_DISK_ORANGE_PUSHING			33
#define SND_SP_DISK_YELLOW_PUSHING			34
#define SND_CLASS_SP_PORT_PASSING			35
#define SND_CLASS_SP_EXIT_PASSING			36
#define SND_CLASS_SP_EXIT_OPENING			37
#define SND_CLASS_SP_EXIT_CLOSING			38
#define SND_SP_SNIKSNAK_MOVING				39
#define SND_SP_SNIKSNAK_WAITING				40
#define SND_SP_ELECTRON_MOVING				41
#define SND_SP_ELECTRON_WAITING				42
#define SND_SP_TERMINAL_ACTIVATING			43
#define SND_SP_TERMINAL_ACTIVE				44
#define SND_CLASS_SOKOBAN_PUSHING			45
#define SND_CLASS_SOKOBAN_FILLING			46
#define SND_CLASS_SOKOBAN_EMPTYING			47
#define SND_CLASS_PLAYER_MOVING				48
#define SND_SAND_DIGGING				49
#define SND_CLASS_EMERALD_COLLECTING			50
#define SND_CLASS_EMERALD_IMPACT			51
#define SND_DIAMOND_COLLECTING				52
#define SND_DIAMOND_IMPACT				53
#define SND_DIAMOND_BREAKING				54
#define SND_ROCK_PUSHING				55
#define SND_ROCK_IMPACT					56
#define SND_BOMB_PUSHING				57
#define SND_NUT_PUSHING					58
#define SND_NUT_BREAKING				59
#define SND_NUT_IMPACT					60
#define SND_CLASS_DYNAMITE_COLLECTING			61
#define SND_CLASS_DYNAMITE_DROPPING			62
#define SND_CLASS_DYNAMITE_ACTIVE			63
#define SND_CLASS_KEY_COLLECTING			64
#define SND_CLASS_GATE_PASSING				65
#define SND_BUG_MOVING					66
#define SND_BUG_WAITING					67
#define SND_SPACESHIP_MOVING				68
#define SND_SPACESHIP_WAITING				69
#define SND_YAMYAM_MOVING				70
#define SND_YAMYAM_WAITING				71
#define SND_YAMYAM_DIGGING				72
#define SND_ROBOT_MOVING				73
#define SND_ROBOT_WAITING				74
#define SND_ROBOT_WHEEL_ACTIVATING			75
#define SND_ROBOT_WHEEL_ACTIVE				76
#define SND_MAGIC_WALL_ACTIVATING			77
#define SND_MAGIC_WALL_ACTIVE				78
#define SND_MAGIC_WALL_FILLING				79
#define SND_CLASS_AMOEBA_WAITING			80
#define SND_CLASS_AMOEBA_GROWING			81
#define SND_CLASS_AMOEBA_DROPPING			82
#define SND_ACID_SPLASHING				83
#define SND_CLASS_QUICKSAND_FILLING			84
#define SND_CLASS_QUICKSAND_EMPTYING			85
#define SND_CLASS_EXIT_OPENING				86
#define SND_CLASS_EXIT_CLOSING				87
#define SND_CLASS_EXIT_PASSING				88
#define SND_PENGUIN_PASSING				89
#define SND_BALLOON_MOVING				90
#define SND_BALLOON_WAITING				91
#define SND_BALLOON_PUSHING				92
#define SND_CLASS_BALLOON_SWITCH_ACTIVATING		93
#define SND_SPRING_MOVING				94
#define SND_SPRING_PUSHING				95
#define SND_SPRING_IMPACT				96
#define SND_CLASS_WALL_GROWING				97
#define SND_EMC_ANDROID_PUSHING				98
#define SND_EMC_ANDROID_MOVING				99
#define SND_EMC_ANDROID_DROPPING			100
#define SND_EMC_MAGIC_BALL_DROPPING			101
#define SND_PEARL_COLLECTING				102
#define SND_PEARL_BREAKING				103
#define SND_PEARL_IMPACT				104
#define SND_CRYSTAL_COLLECTING				105
#define SND_CRYSTAL_IMPACT				106
#define SND_CLASS_ENVELOPE_COLLECTING			107
#define SND_CLASS_ENVELOPE_OPENING			108
#define SND_CLASS_ENVELOPE_CLOSING			109
#define SND_INVISIBLE_SAND_DIGGING			110
#define SND_INVISIBLE_SAND_ACTIVE_DIGGING		111
#define SND_SHIELD_NORMAL_COLLECTING			112
#define SND_SHIELD_NORMAL_ACTIVE			113
#define SND_SHIELD_DEADLY_COLLECTING			114
#define SND_SHIELD_DEADLY_ACTIVE			115
#define SND_EXTRA_TIME_COLLECTING			116
#define SND_MOLE_MOVING					117
#define SND_MOLE_WAITING				118
#define SND_MOLE_DIGGING				119
#define SND_CLASS_SWITCHGATE_SWITCH_ACTIVATING		120
#define SND_CLASS_SWITCHGATE_OPENING			121
#define SND_CLASS_SWITCHGATE_CLOSING			122
#define SND_CLASS_SWITCHGATE_PASSING			123
#define SND_TIMEGATE_SWITCH_ACTIVATING			124
#define SND_TIMEGATE_SWITCH_ACTIVE			125
#define SND_TIMEGATE_SWITCH_DEACTIVATING		126
#define SND_TIMEGATE_OPENING				127
#define SND_CLASS_TIMEGATE_CLOSING			128
#define SND_CLASS_TIMEGATE_PASSING			129
#define SND_CLASS_CONVEYOR_BELT_SWITCH_ACTIVATING	130
#define SND_CLASS_CONVEYOR_BELT_ACTIVE			131
#define SND_CLASS_CONVEYOR_BELT_SWITCH_DEACTIVATING	132
#define SND_LIGHT_SWITCH_ACTIVATING			133
#define SND_LIGHT_SWITCH_DEACTIVATING			134
#define SND_DX_SUPABOMB_PUSHING				135
#define SND_TRAP_DIGGING				136
#define SND_TRAP_ACTIVATING				137
#define SND_CLASS_TUBE_WALKING				138
#define SND_AMOEBA_TURNING_TO_GEM			139
#define SND_AMOEBA_TURNING_TO_ROCK			140
#define SND_SPEED_PILL_COLLECTING			141
#define SND_DYNABOMB_INCREASE_NUMBER_COLLECTING		142
#define SND_DYNABOMB_INCREASE_SIZE_COLLECTING		143
#define SND_DYNABOMB_INCREASE_POWER_COLLECTING		144
#define SND_CLASS_DYNABOMB_DROPPING			145
#define SND_CLASS_DYNABOMB_ACTIVE			146
#define SND_SATELLITE_MOVING				147
#define SND_SATELLITE_WAITING				148
#define SND_SATELLITE_PUSHING				149
#define SND_LAMP_ACTIVATING				150
#define SND_LAMP_DEACTIVATING				151
#define SND_TIME_ORB_FULL_COLLECTING			152
#define SND_TIME_ORB_FULL_IMPACT			153
#define SND_TIME_ORB_EMPTY_PUSHING			154
#define SND_TIME_ORB_EMPTY_IMPACT			155
#define SND_GAME_OF_LIFE_WAITING			156
#define SND_GAME_OF_LIFE_GROWING			157
#define SND_BIOMAZE_WAITING				158
#define SND_BIOMAZE_GROWING				159
#define SND_PACMAN_MOVING				160
#define SND_PACMAN_WAITING				161
#define SND_PACMAN_DIGGING				162
#define SND_DARK_YAMYAM_MOVING				163
#define SND_DARK_YAMYAM_WAITING				164
#define SND_DARK_YAMYAM_DIGGING				165
#define SND_PENGUIN_MOVING				166
#define SND_PENGUIN_WAITING				167
#define SND_PIG_MOVING					168
#define SND_PIG_WAITING					169
#define SND_PIG_DIGGING					170
#define SND_DRAGON_MOVING				171
#define SND_DRAGON_WAITING				172
#define SND_DRAGON_ATTACKING				173
#define SND_GAME_STARTING				174
#define SND_GAME_RUNNING_OUT_OF_TIME			175
#define SND_GAME_LEVELTIME_BONUS			176
#define SND_GAME_LOSING					177
#define SND_GAME_WINNING				178
#define SND_GAME_SOKOBAN_SOLVING			179
#define SND_DOOR_OPENING				180
#define SND_DOOR_CLOSING				181
#define SND_MENU_ITEM_ACTIVATING			182
#define SND_MENU_ITEM_SELECTING				183
#define SND_BACKGROUND_TITLE				184
#define SND_BACKGROUND_MESSAGE				185
#define SND_BACKGROUND_MAIN				186
#define SND_BACKGROUND_LEVELS				187
#define SND_BACKGROUND_SCORES				188
#define SND_BACKGROUND_EDITOR				189
#define SND_BACKGROUND_INFO				190
#define SND_BACKGROUND_SETUP				191

#define NUM_SOUND_FILES					192

#endif	/* CONF_SND_H */
