/***********************************************************
* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
*----------------------------------------------------------*
* (c) 1995-2006 Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* conf_grp.c                                               *
***********************************************************/

/* ----- this file was automatically generated -- do not edit by hand ----- */

#ifndef CONF_GRP_C
#define CONF_GRP_C

/* values for graphics configuration (group elements) */

  { "group_1",					"RocksDC.pcx"		},
  { "group_1.xpos",				"4"			},
  { "group_1.ypos",				"15"			},
  { "group_1.frames",				"1"			},
  { "group_1.EDITOR",				"RocksDC.pcx"		},
  { "group_1.EDITOR.xpos",			"14"			},
  { "group_1.EDITOR.ypos",			"15"			},

  { "group_2",					"RocksDC.pcx"		},
  { "group_2.xpos",				"4"			},
  { "group_2.ypos",				"15"			},
  { "group_2.frames",				"1"			},
  { "group_2.EDITOR",				"RocksDC.pcx"		},
  { "group_2.EDITOR.xpos",			"14"			},
  { "group_2.EDITOR.ypos",			"15"			},

  { "group_3",					"RocksDC.pcx"		},
  { "group_3.xpos",				"4"			},
  { "group_3.ypos",				"15"			},
  { "group_3.frames",				"1"			},
  { "group_3.EDITOR",				"RocksDC.pcx"		},
  { "group_3.EDITOR.xpos",			"14"			},
  { "group_3.EDITOR.ypos",			"15"			},

  { "group_4",					"RocksDC.pcx"		},
  { "group_4.xpos",				"4"			},
  { "group_4.ypos",				"15"			},
  { "group_4.frames",				"1"			},
  { "group_4.EDITOR",				"RocksDC.pcx"		},
  { "group_4.EDITOR.xpos",			"14"			},
  { "group_4.EDITOR.ypos",			"15"			},

  { "group_5",					"RocksDC.pcx"		},
  { "group_5.xpos",				"4"			},
  { "group_5.ypos",				"15"			},
  { "group_5.frames",				"1"			},
  { "group_5.EDITOR",				"RocksDC.pcx"		},
  { "group_5.EDITOR.xpos",			"14"			},
  { "group_5.EDITOR.ypos",			"15"			},

  { "group_6",					"RocksDC.pcx"		},
  { "group_6.xpos",				"4"			},
  { "group_6.ypos",				"15"			},
  { "group_6.frames",				"1"			},
  { "group_6.EDITOR",				"RocksDC.pcx"		},
  { "group_6.EDITOR.xpos",			"14"			},
  { "group_6.EDITOR.ypos",			"15"			},

  { "group_7",					"RocksDC.pcx"		},
  { "group_7.xpos",				"4"			},
  { "group_7.ypos",				"15"			},
  { "group_7.frames",				"1"			},
  { "group_7.EDITOR",				"RocksDC.pcx"		},
  { "group_7.EDITOR.xpos",			"14"			},
  { "group_7.EDITOR.ypos",			"15"			},

  { "group_8",					"RocksDC.pcx"		},
  { "group_8.xpos",				"4"			},
  { "group_8.ypos",				"15"			},
  { "group_8.frames",				"1"			},
  { "group_8.EDITOR",				"RocksDC.pcx"		},
  { "group_8.EDITOR.xpos",			"14"			},
  { "group_8.EDITOR.ypos",			"15"			},

  { "group_9",					"RocksDC.pcx"		},
  { "group_9.xpos",				"4"			},
  { "group_9.ypos",				"15"			},
  { "group_9.frames",				"1"			},
  { "group_9.EDITOR",				"RocksDC.pcx"		},
  { "group_9.EDITOR.xpos",			"14"			},
  { "group_9.EDITOR.ypos",			"15"			},

  { "group_10",					"RocksDC.pcx"		},
  { "group_10.xpos",				"4"			},
  { "group_10.ypos",				"15"			},
  { "group_10.frames",				"1"			},
  { "group_10.EDITOR",				"RocksDC.pcx"		},
  { "group_10.EDITOR.xpos",			"14"			},
  { "group_10.EDITOR.ypos",			"15"			},

  { "group_11",					"RocksDC.pcx"		},
  { "group_11.xpos",				"4"			},
  { "group_11.ypos",				"15"			},
  { "group_11.frames",				"1"			},
  { "group_11.EDITOR",				"RocksDC.pcx"		},
  { "group_11.EDITOR.xpos",			"14"			},
  { "group_11.EDITOR.ypos",			"15"			},

  { "group_12",					"RocksDC.pcx"		},
  { "group_12.xpos",				"4"			},
  { "group_12.ypos",				"15"			},
  { "group_12.frames",				"1"			},
  { "group_12.EDITOR",				"RocksDC.pcx"		},
  { "group_12.EDITOR.xpos",			"14"			},
  { "group_12.EDITOR.ypos",			"15"			},

  { "group_13",					"RocksDC.pcx"		},
  { "group_13.xpos",				"4"			},
  { "group_13.ypos",				"15"			},
  { "group_13.frames",				"1"			},
  { "group_13.EDITOR",				"RocksDC.pcx"		},
  { "group_13.EDITOR.xpos",			"14"			},
  { "group_13.EDITOR.ypos",			"15"			},

  { "group_14",					"RocksDC.pcx"		},
  { "group_14.xpos",				"4"			},
  { "group_14.ypos",				"15"			},
  { "group_14.frames",				"1"			},
  { "group_14.EDITOR",				"RocksDC.pcx"		},
  { "group_14.EDITOR.xpos",			"14"			},
  { "group_14.EDITOR.ypos",			"15"			},

  { "group_15",					"RocksDC.pcx"		},
  { "group_15.xpos",				"4"			},
  { "group_15.ypos",				"15"			},
  { "group_15.frames",				"1"			},
  { "group_15.EDITOR",				"RocksDC.pcx"		},
  { "group_15.EDITOR.xpos",			"14"			},
  { "group_15.EDITOR.ypos",			"15"			},

  { "group_16",					"RocksDC.pcx"		},
  { "group_16.xpos",				"4"			},
  { "group_16.ypos",				"15"			},
  { "group_16.frames",				"1"			},
  { "group_16.EDITOR",				"RocksDC.pcx"		},
  { "group_16.EDITOR.xpos",			"14"			},
  { "group_16.EDITOR.ypos",			"15"			},

  { "group_17",					"RocksDC.pcx"		},
  { "group_17.xpos",				"4"			},
  { "group_17.ypos",				"15"			},
  { "group_17.frames",				"1"			},
  { "group_17.EDITOR",				"RocksDC.pcx"		},
  { "group_17.EDITOR.xpos",			"14"			},
  { "group_17.EDITOR.ypos",			"15"			},

  { "group_18",					"RocksDC.pcx"		},
  { "group_18.xpos",				"4"			},
  { "group_18.ypos",				"15"			},
  { "group_18.frames",				"1"			},
  { "group_18.EDITOR",				"RocksDC.pcx"		},
  { "group_18.EDITOR.xpos",			"14"			},
  { "group_18.EDITOR.ypos",			"15"			},

  { "group_19",					"RocksDC.pcx"		},
  { "group_19.xpos",				"4"			},
  { "group_19.ypos",				"15"			},
  { "group_19.frames",				"1"			},
  { "group_19.EDITOR",				"RocksDC.pcx"		},
  { "group_19.EDITOR.xpos",			"14"			},
  { "group_19.EDITOR.ypos",			"15"			},

  { "group_20",					"RocksDC.pcx"		},
  { "group_20.xpos",				"4"			},
  { "group_20.ypos",				"15"			},
  { "group_20.frames",				"1"			},
  { "group_20.EDITOR",				"RocksDC.pcx"		},
  { "group_20.EDITOR.xpos",			"14"			},
  { "group_20.EDITOR.ypos",			"15"			},

  { "group_21",					"RocksDC.pcx"		},
  { "group_21.xpos",				"4"			},
  { "group_21.ypos",				"15"			},
  { "group_21.frames",				"1"			},
  { "group_21.EDITOR",				"RocksDC.pcx"		},
  { "group_21.EDITOR.xpos",			"14"			},
  { "group_21.EDITOR.ypos",			"15"			},

  { "group_22",					"RocksDC.pcx"		},
  { "group_22.xpos",				"4"			},
  { "group_22.ypos",				"15"			},
  { "group_22.frames",				"1"			},
  { "group_22.EDITOR",				"RocksDC.pcx"		},
  { "group_22.EDITOR.xpos",			"14"			},
  { "group_22.EDITOR.ypos",			"15"			},

  { "group_23",					"RocksDC.pcx"		},
  { "group_23.xpos",				"4"			},
  { "group_23.ypos",				"15"			},
  { "group_23.frames",				"1"			},
  { "group_23.EDITOR",				"RocksDC.pcx"		},
  { "group_23.EDITOR.xpos",			"14"			},
  { "group_23.EDITOR.ypos",			"15"			},

  { "group_24",					"RocksDC.pcx"		},
  { "group_24.xpos",				"4"			},
  { "group_24.ypos",				"15"			},
  { "group_24.frames",				"1"			},
  { "group_24.EDITOR",				"RocksDC.pcx"		},
  { "group_24.EDITOR.xpos",			"14"			},
  { "group_24.EDITOR.ypos",			"15"			},

  { "group_25",					"RocksDC.pcx"		},
  { "group_25.xpos",				"4"			},
  { "group_25.ypos",				"15"			},
  { "group_25.frames",				"1"			},
  { "group_25.EDITOR",				"RocksDC.pcx"		},
  { "group_25.EDITOR.xpos",			"14"			},
  { "group_25.EDITOR.ypos",			"15"			},

  { "group_26",					"RocksDC.pcx"		},
  { "group_26.xpos",				"4"			},
  { "group_26.ypos",				"15"			},
  { "group_26.frames",				"1"			},
  { "group_26.EDITOR",				"RocksDC.pcx"		},
  { "group_26.EDITOR.xpos",			"14"			},
  { "group_26.EDITOR.ypos",			"15"			},

  { "group_27",					"RocksDC.pcx"		},
  { "group_27.xpos",				"4"			},
  { "group_27.ypos",				"15"			},
  { "group_27.frames",				"1"			},
  { "group_27.EDITOR",				"RocksDC.pcx"		},
  { "group_27.EDITOR.xpos",			"14"			},
  { "group_27.EDITOR.ypos",			"15"			},

  { "group_28",					"RocksDC.pcx"		},
  { "group_28.xpos",				"4"			},
  { "group_28.ypos",				"15"			},
  { "group_28.frames",				"1"			},
  { "group_28.EDITOR",				"RocksDC.pcx"		},
  { "group_28.EDITOR.xpos",			"14"			},
  { "group_28.EDITOR.ypos",			"15"			},

  { "group_29",					"RocksDC.pcx"		},
  { "group_29.xpos",				"4"			},
  { "group_29.ypos",				"15"			},
  { "group_29.frames",				"1"			},
  { "group_29.EDITOR",				"RocksDC.pcx"		},
  { "group_29.EDITOR.xpos",			"14"			},
  { "group_29.EDITOR.ypos",			"15"			},

  { "group_30",					"RocksDC.pcx"		},
  { "group_30.xpos",				"4"			},
  { "group_30.ypos",				"15"			},
  { "group_30.frames",				"1"			},
  { "group_30.EDITOR",				"RocksDC.pcx"		},
  { "group_30.EDITOR.xpos",			"14"			},
  { "group_30.EDITOR.ypos",			"15"			},

  { "group_31",					"RocksDC.pcx"		},
  { "group_31.xpos",				"4"			},
  { "group_31.ypos",				"15"			},
  { "group_31.frames",				"1"			},
  { "group_31.EDITOR",				"RocksDC.pcx"		},
  { "group_31.EDITOR.xpos",			"14"			},
  { "group_31.EDITOR.ypos",			"15"			},

  { "group_32",					"RocksDC.pcx"		},
  { "group_32.xpos",				"4"			},
  { "group_32.ypos",				"15"			},
  { "group_32.frames",				"1"			},
  { "group_32.EDITOR",				"RocksDC.pcx"		},
  { "group_32.EDITOR.xpos",			"14"			},
  { "group_32.EDITOR.ypos",			"15"			},


#endif	/* CONF_GRP_C */
