/***********************************************************
* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
*----------------------------------------------------------*
* (c) 1995-2006 Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* conf_gfx.h                                               *
***********************************************************/

/* ----- this file was automatically generated -- do not edit by hand ----- */

#ifndef CONF_GFX_H
#define CONF_GFX_H

/* values for graphics configuration (normal elements) */

#define IMG_BD_WALL					0
#define IMG_BD_WALL_EDITOR				1
#define IMG_BD_ROCK					2
#define IMG_BD_ROCK_EDITOR				3
#define IMG_BD_ROCK_MOVING_LEFT				4
#define IMG_BD_ROCK_MOVING_RIGHT			5
#define IMG_BD_ROCK_PUSHING_LEFT			6
#define IMG_BD_ROCK_PUSHING_RIGHT			7
#define IMG_BD_DIAMOND					8
#define IMG_BD_DIAMOND_MOVING				9
#define IMG_BD_DIAMOND_FALLING				10
#define IMG_BD_MAGIC_WALL				11
#define IMG_BD_MAGIC_WALL_ACTIVE			12
#define IMG_BD_MAGIC_WALL_FILLING			13
#define IMG_BD_MAGIC_WALL_FULL				14
#define IMG_BD_MAGIC_WALL_EMPTYING			15
#define IMG_BD_MAGIC_WALL_DEAD				16
#define IMG_BD_AMOEBA					17
#define IMG_BD_AMOEBA_EDITOR				18
#define IMG_BD_BUTTERFLY				19
#define IMG_BD_BUTTERFLY_RIGHT				20
#define IMG_BD_BUTTERFLY_RIGHT_EDITOR			21
#define IMG_BD_BUTTERFLY_UP				22
#define IMG_BD_BUTTERFLY_UP_EDITOR			23
#define IMG_BD_BUTTERFLY_LEFT				24
#define IMG_BD_BUTTERFLY_LEFT_EDITOR			25
#define IMG_BD_BUTTERFLY_DOWN				26
#define IMG_BD_BUTTERFLY_DOWN_EDITOR			27
#define IMG_BD_FIREFLY					28
#define IMG_BD_FIREFLY_RIGHT				29
#define IMG_BD_FIREFLY_RIGHT_EDITOR			30
#define IMG_BD_FIREFLY_UP				31
#define IMG_BD_FIREFLY_UP_EDITOR			32
#define IMG_BD_FIREFLY_LEFT				33
#define IMG_BD_FIREFLY_LEFT_EDITOR			34
#define IMG_BD_FIREFLY_DOWN				35
#define IMG_BD_FIREFLY_DOWN_EDITOR			36
#define IMG_SP_DEFAULT_EXPLODING			37
#define IMG_SP_ZONK					38
#define IMG_SP_ZONK_MOVING_LEFT				39
#define IMG_SP_ZONK_MOVING_RIGHT			40
#define IMG_SP_ZONK_PUSHING_LEFT			41
#define IMG_SP_ZONK_PUSHING_RIGHT			42
#define IMG_SP_BASE					43
#define IMG_SP_BASE_DIGGING				44
#define IMG_SP_BASE_SNAPPING				45
#define IMG_SP_MURPHY					46
#define IMG_SP_MURPHY_MOVING_LEFT			47
#define IMG_SP_MURPHY_MOVING_RIGHT			48
#define IMG_SP_MURPHY_DIGGING_LEFT			49
#define IMG_SP_MURPHY_DIGGING_RIGHT			50
#define IMG_SP_MURPHY_COLLECTING_LEFT			51
#define IMG_SP_MURPHY_COLLECTING_RIGHT			52
#define IMG_SP_MURPHY_PUSHING_LEFT			53
#define IMG_SP_MURPHY_PUSHING_RIGHT			54
#define IMG_SP_MURPHY_SNAPPING_LEFT			55
#define IMG_SP_MURPHY_SNAPPING_RIGHT			56
#define IMG_SP_MURPHY_SNAPPING_UP			57
#define IMG_SP_MURPHY_SNAPPING_DOWN			58
#define IMG_SP_MURPHY_BORING				59
#define IMG_SP_MURPHY_BORING_1				60
#define IMG_SP_MURPHY_SLEEPING_LEFT			61
#define IMG_SP_MURPHY_SLEEPING_RIGHT			62
#define IMG_SP_MURPHY_CLONE				63
#define IMG_SP_INFOTRON					64
#define IMG_SP_INFOTRON_EDITOR				65
#define IMG_SP_INFOTRON_COLLECTING			66
#define IMG_SP_CHIP_SINGLE				67
#define IMG_SP_CHIP_LEFT				68
#define IMG_SP_CHIP_RIGHT				69
#define IMG_SP_CHIP_TOP					70
#define IMG_SP_CHIP_BOTTOM				71
#define IMG_SP_HARDWARE_GRAY				72
#define IMG_SP_HARDWARE_GREEN				73
#define IMG_SP_HARDWARE_BLUE				74
#define IMG_SP_HARDWARE_RED				75
#define IMG_SP_HARDWARE_YELLOW				76
#define IMG_SP_EXIT_CLOSED				77
#define IMG_SP_EXIT_OPENING				78
#define IMG_SP_EXIT_OPEN				79
#define IMG_SP_EXIT_CLOSING				80
#define IMG_SP_DISK_ORANGE				81
#define IMG_SP_DISK_YELLOW				82
#define IMG_SP_DISK_RED					83
#define IMG_SP_DISK_RED_COLLECTING			84
#define IMG_SP_DISK_RED_ACTIVE				85
#define IMG_SP_PORT_RIGHT				86
#define IMG_SP_PORT_DOWN				87
#define IMG_SP_PORT_LEFT				88
#define IMG_SP_PORT_UP					89
#define IMG_SP_PORT_HORIZONTAL				90
#define IMG_SP_PORT_VERTICAL				91
#define IMG_SP_PORT_ANY					92
#define IMG_SP_GRAVITY_PORT_RIGHT			93
#define IMG_SP_GRAVITY_PORT_RIGHT_EDITOR		94
#define IMG_SP_GRAVITY_PORT_DOWN			95
#define IMG_SP_GRAVITY_PORT_DOWN_EDITOR			96
#define IMG_SP_GRAVITY_PORT_LEFT			97
#define IMG_SP_GRAVITY_PORT_LEFT_EDITOR			98
#define IMG_SP_GRAVITY_PORT_UP				99
#define IMG_SP_GRAVITY_PORT_UP_EDITOR			100
#define IMG_SP_GRAVITY_ON_PORT_RIGHT			101
#define IMG_SP_GRAVITY_ON_PORT_RIGHT_EDITOR		102
#define IMG_SP_GRAVITY_ON_PORT_DOWN			103
#define IMG_SP_GRAVITY_ON_PORT_DOWN_EDITOR		104
#define IMG_SP_GRAVITY_ON_PORT_LEFT			105
#define IMG_SP_GRAVITY_ON_PORT_LEFT_EDITOR		106
#define IMG_SP_GRAVITY_ON_PORT_UP			107
#define IMG_SP_GRAVITY_ON_PORT_UP_EDITOR		108
#define IMG_SP_GRAVITY_OFF_PORT_RIGHT			109
#define IMG_SP_GRAVITY_OFF_PORT_RIGHT_EDITOR		110
#define IMG_SP_GRAVITY_OFF_PORT_DOWN			111
#define IMG_SP_GRAVITY_OFF_PORT_DOWN_EDITOR		112
#define IMG_SP_GRAVITY_OFF_PORT_LEFT			113
#define IMG_SP_GRAVITY_OFF_PORT_LEFT_EDITOR		114
#define IMG_SP_GRAVITY_OFF_PORT_UP			115
#define IMG_SP_GRAVITY_OFF_PORT_UP_EDITOR		116
#define IMG_SP_SNIKSNAK					117
#define IMG_SP_SNIKSNAK_LEFT				118
#define IMG_SP_SNIKSNAK_RIGHT				119
#define IMG_SP_SNIKSNAK_UP				120
#define IMG_SP_SNIKSNAK_DOWN				121
#define IMG_SP_SNIKSNAK_TURNING_FROM_LEFT_UP		122
#define IMG_SP_SNIKSNAK_TURNING_FROM_LEFT_DOWN		123
#define IMG_SP_SNIKSNAK_TURNING_FROM_RIGHT_UP		124
#define IMG_SP_SNIKSNAK_TURNING_FROM_RIGHT_DOWN		125
#define IMG_SP_SNIKSNAK_TURNING_FROM_UP_LEFT		126
#define IMG_SP_SNIKSNAK_TURNING_FROM_UP_RIGHT		127
#define IMG_SP_SNIKSNAK_TURNING_FROM_DOWN_LEFT		128
#define IMG_SP_SNIKSNAK_TURNING_FROM_DOWN_RIGHT		129
#define IMG_SP_ELECTRON					130
#define IMG_SP_ELECTRON_EDITOR				131
#define IMG_SP_ELECTRON_EXPLODING			132
#define IMG_SP_TERMINAL					133
#define IMG_SP_TERMINAL_EDITOR				134
#define IMG_SP_TERMINAL_ACTIVE				135
#define IMG_SP_BUGGY_BASE				136
#define IMG_SP_BUGGY_BASE_EDITOR			137
#define IMG_SP_BUGGY_BASE_ACTIVATING			138
#define IMG_SP_BUGGY_BASE_ACTIVE			139
#define IMG_SP_HARDWARE_BASE_1				140
#define IMG_SP_HARDWARE_BASE_2				141
#define IMG_SP_HARDWARE_BASE_3				142
#define IMG_SP_HARDWARE_BASE_4				143
#define IMG_SP_HARDWARE_BASE_5				144
#define IMG_SP_HARDWARE_BASE_6				145
#define IMG_SOKOBAN_OBJECT				146
#define IMG_SOKOBAN_OBJECT_EDITOR			147
#define IMG_SOKOBAN_FIELD_EMPTY				148
#define IMG_SOKOBAN_FIELD_FULL				149
#define IMG_SOKOBAN_FIELD_PLAYER			150
#define IMG_SOKOBAN_FIELD_PLAYER_EDITOR			151
#define IMG_EMPTY_SPACE					152
#define IMG_SAND					153
#define IMG_SAND_CRUMBLED				154
#define IMG_SAND_DIGGING_LEFT				155
#define IMG_SAND_DIGGING_RIGHT				156
#define IMG_SAND_DIGGING_UP				157
#define IMG_SAND_DIGGING_DOWN				158
#define IMG_SAND_DIGGING_LEFT_CRUMBLED			159
#define IMG_SAND_DIGGING_RIGHT_CRUMBLED			160
#define IMG_SAND_DIGGING_UP_CRUMBLED			161
#define IMG_SAND_DIGGING_DOWN_CRUMBLED			162
#define IMG_WALL					163
#define IMG_WALL_SLIPPERY				164
#define IMG_STEELWALL					165
#define IMG_ROCK					166
#define IMG_ROCK_MOVING_LEFT				167
#define IMG_ROCK_MOVING_RIGHT				168
#define IMG_ROCK_PUSHING_LEFT				169
#define IMG_ROCK_PUSHING_RIGHT				170
#define IMG_EMERALD					171
#define IMG_EMERALD_MOVING				172
#define IMG_EMERALD_FALLING				173
#define IMG_EMERALD_COLLECTING				174
#define IMG_DIAMOND					175
#define IMG_DIAMOND_MOVING				176
#define IMG_DIAMOND_FALLING				177
#define IMG_DIAMOND_COLLECTING				178
#define IMG_BOMB					179
#define IMG_NUT						180
#define IMG_NUT_BREAKING				181
#define IMG_DYNAMITE					182
#define IMG_DYNAMITE_EDITOR				183
#define IMG_DYNAMITE_ACTIVE				184
#define IMG_DYNAMITE_ACTIVE_EDITOR			185
#define IMG_EM_DYNAMITE					186
#define IMG_EM_DYNAMITE_ACTIVE				187
#define IMG_EM_DYNAMITE_ACTIVE_EDITOR			188
#define IMG_WALL_EMERALD				189
#define IMG_WALL_DIAMOND				190
#define IMG_BUG						191
#define IMG_BUG_RIGHT					192
#define IMG_BUG_UP					193
#define IMG_BUG_LEFT					194
#define IMG_BUG_DOWN					195
#define IMG_BUG_MOVING_RIGHT				196
#define IMG_BUG_MOVING_UP				197
#define IMG_BUG_MOVING_LEFT				198
#define IMG_BUG_MOVING_DOWN				199
#define IMG_BUG_TURNING_FROM_RIGHT_UP			200
#define IMG_BUG_TURNING_FROM_UP_LEFT			201
#define IMG_BUG_TURNING_FROM_LEFT_DOWN			202
#define IMG_BUG_TURNING_FROM_DOWN_RIGHT			203
#define IMG_BUG_TURNING_FROM_RIGHT_DOWN			204
#define IMG_BUG_TURNING_FROM_UP_RIGHT			205
#define IMG_BUG_TURNING_FROM_LEFT_UP			206
#define IMG_BUG_TURNING_FROM_DOWN_LEFT			207
#define IMG_SPACESHIP					208
#define IMG_SPACESHIP_RIGHT				209
#define IMG_SPACESHIP_UP				210
#define IMG_SPACESHIP_LEFT				211
#define IMG_SPACESHIP_DOWN				212
#define IMG_SPACESHIP_MOVING_RIGHT			213
#define IMG_SPACESHIP_MOVING_UP				214
#define IMG_SPACESHIP_MOVING_LEFT			215
#define IMG_SPACESHIP_MOVING_DOWN			216
#define IMG_SPACESHIP_TURNING_FROM_RIGHT_UP		217
#define IMG_SPACESHIP_TURNING_FROM_UP_LEFT		218
#define IMG_SPACESHIP_TURNING_FROM_LEFT_DOWN		219
#define IMG_SPACESHIP_TURNING_FROM_DOWN_RIGHT		220
#define IMG_SPACESHIP_TURNING_FROM_RIGHT_DOWN		221
#define IMG_SPACESHIP_TURNING_FROM_UP_RIGHT		222
#define IMG_SPACESHIP_TURNING_FROM_LEFT_UP		223
#define IMG_SPACESHIP_TURNING_FROM_DOWN_LEFT		224
#define IMG_YAMYAM					225
#define IMG_YAMYAM_LEFT					226
#define IMG_YAMYAM_LEFT_EDITOR				227
#define IMG_YAMYAM_RIGHT				228
#define IMG_YAMYAM_RIGHT_EDITOR				229
#define IMG_YAMYAM_UP					230
#define IMG_YAMYAM_UP_EDITOR				231
#define IMG_YAMYAM_DOWN					232
#define IMG_YAMYAM_DOWN_EDITOR				233
#define IMG_YAMYAM_MOVING				234
#define IMG_ROBOT					235
#define IMG_ROBOT_MOVING				236
#define IMG_ROBOT_WHEEL					237
#define IMG_ROBOT_WHEEL_ACTIVE				238
#define IMG_MAGIC_WALL					239
#define IMG_MAGIC_WALL_ACTIVE				240
#define IMG_MAGIC_WALL_FILLING				241
#define IMG_MAGIC_WALL_FULL				242
#define IMG_MAGIC_WALL_EMPTYING				243
#define IMG_MAGIC_WALL_DEAD				244
#define IMG_QUICKSAND_EMPTY				245
#define IMG_QUICKSAND_FILLING				246
#define IMG_QUICKSAND_FULL				247
#define IMG_QUICKSAND_FULL_EDITOR			248
#define IMG_QUICKSAND_EMPTYING				249
#define IMG_ACID_POOL_TOPLEFT				250
#define IMG_ACID_POOL_TOPRIGHT				251
#define IMG_ACID_POOL_BOTTOMLEFT			252
#define IMG_ACID_POOL_BOTTOM				253
#define IMG_ACID_POOL_BOTTOMRIGHT			254
#define IMG_ACID					255
#define IMG_ACID_SPLASH_LEFT				256
#define IMG_ACID_SPLASH_RIGHT				257
#define IMG_AMOEBA_DROP					258
#define IMG_AMOEBA_GROWING				259
#define IMG_AMOEBA_SHRINKING				260
#define IMG_AMOEBA_WET					261
#define IMG_AMOEBA_WET_EDITOR				262
#define IMG_AMOEBA_DROPPING				263
#define IMG_AMOEBA_DRY					264
#define IMG_AMOEBA_FULL					265
#define IMG_AMOEBA_FULL_EDITOR				266
#define IMG_AMOEBA_DEAD					267
#define IMG_AMOEBA_DEAD_EDITOR				268
#define IMG_EM_KEY_1					269
#define IMG_EM_KEY_2					270
#define IMG_EM_KEY_3					271
#define IMG_EM_KEY_4					272
#define IMG_EM_GATE_1					273
#define IMG_EM_GATE_2					274
#define IMG_EM_GATE_3					275
#define IMG_EM_GATE_4					276
#define IMG_EM_GATE_1_GRAY				277
#define IMG_EM_GATE_1_GRAY_EDITOR			278
#define IMG_EM_GATE_1_GRAY_ACTIVE			279
#define IMG_EM_GATE_2_GRAY				280
#define IMG_EM_GATE_2_GRAY_EDITOR			281
#define IMG_EM_GATE_2_GRAY_ACTIVE			282
#define IMG_EM_GATE_3_GRAY				283
#define IMG_EM_GATE_3_GRAY_EDITOR			284
#define IMG_EM_GATE_3_GRAY_ACTIVE			285
#define IMG_EM_GATE_4_GRAY				286
#define IMG_EM_GATE_4_GRAY_EDITOR			287
#define IMG_EM_GATE_4_GRAY_ACTIVE			288
#define IMG_EXIT_CLOSED					289
#define IMG_EXIT_OPENING				290
#define IMG_EXIT_OPEN					291
#define IMG_EXIT_CLOSING				292
#define IMG_BALLOON					293
#define IMG_BALLOON_MOVING				294
#define IMG_BALLOON_PUSHING				295
#define IMG_BALLOON_SWITCH_LEFT				296
#define IMG_BALLOON_SWITCH_RIGHT			297
#define IMG_BALLOON_SWITCH_UP				298
#define IMG_BALLOON_SWITCH_DOWN				299
#define IMG_BALLOON_SWITCH_ANY				300
#define IMG_BALLOON_SWITCH_NONE				301
#define IMG_SPRING					302
#define IMG_EMC_STEELWALL_1				303
#define IMG_EMC_STEELWALL_2				304
#define IMG_EMC_STEELWALL_3				305
#define IMG_EMC_STEELWALL_4				306
#define IMG_EMC_WALL_1					307
#define IMG_EMC_WALL_2					308
#define IMG_EMC_WALL_3					309
#define IMG_EMC_WALL_4					310
#define IMG_EMC_WALL_5					311
#define IMG_EMC_WALL_6					312
#define IMG_EMC_WALL_7					313
#define IMG_EMC_WALL_8					314
#define IMG_INVISIBLE_STEELWALL				315
#define IMG_INVISIBLE_STEELWALL_EDITOR			316
#define IMG_INVISIBLE_STEELWALL_ACTIVE			317
#define IMG_INVISIBLE_WALL				318
#define IMG_INVISIBLE_WALL_EDITOR			319
#define IMG_INVISIBLE_WALL_ACTIVE			320
#define IMG_INVISIBLE_SAND				321
#define IMG_INVISIBLE_SAND_EDITOR			322
#define IMG_INVISIBLE_SAND_ACTIVE			323
#define IMG_INVISIBLE_SAND_ACTIVE_CRUMBLED		324
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_LEFT		325
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_RIGHT		326
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_UP		327
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_DOWN		328
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_LEFT_CRUMBLED	329
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_RIGHT_CRUMBLED	330
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_UP_CRUMBLED	331
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_DOWN_CRUMBLED	332
#define IMG_CONVEYOR_BELT_1_MIDDLE			333
#define IMG_CONVEYOR_BELT_1_MIDDLE_ACTIVE		334
#define IMG_CONVEYOR_BELT_1_LEFT			335
#define IMG_CONVEYOR_BELT_1_LEFT_ACTIVE			336
#define IMG_CONVEYOR_BELT_1_RIGHT			337
#define IMG_CONVEYOR_BELT_1_RIGHT_ACTIVE		338
#define IMG_CONVEYOR_BELT_1_SWITCH_LEFT			339
#define IMG_CONVEYOR_BELT_1_SWITCH_MIDDLE		340
#define IMG_CONVEYOR_BELT_1_SWITCH_RIGHT		341
#define IMG_CONVEYOR_BELT_2_MIDDLE			342
#define IMG_CONVEYOR_BELT_2_MIDDLE_ACTIVE		343
#define IMG_CONVEYOR_BELT_2_LEFT			344
#define IMG_CONVEYOR_BELT_2_LEFT_ACTIVE			345
#define IMG_CONVEYOR_BELT_2_RIGHT			346
#define IMG_CONVEYOR_BELT_2_RIGHT_ACTIVE		347
#define IMG_CONVEYOR_BELT_2_SWITCH_LEFT			348
#define IMG_CONVEYOR_BELT_2_SWITCH_MIDDLE		349
#define IMG_CONVEYOR_BELT_2_SWITCH_RIGHT		350
#define IMG_CONVEYOR_BELT_3_MIDDLE			351
#define IMG_CONVEYOR_BELT_3_MIDDLE_ACTIVE		352
#define IMG_CONVEYOR_BELT_3_LEFT			353
#define IMG_CONVEYOR_BELT_3_LEFT_ACTIVE			354
#define IMG_CONVEYOR_BELT_3_RIGHT			355
#define IMG_CONVEYOR_BELT_3_RIGHT_ACTIVE		356
#define IMG_CONVEYOR_BELT_3_SWITCH_LEFT			357
#define IMG_CONVEYOR_BELT_3_SWITCH_MIDDLE		358
#define IMG_CONVEYOR_BELT_3_SWITCH_RIGHT		359
#define IMG_CONVEYOR_BELT_4_MIDDLE			360
#define IMG_CONVEYOR_BELT_4_MIDDLE_ACTIVE		361
#define IMG_CONVEYOR_BELT_4_LEFT			362
#define IMG_CONVEYOR_BELT_4_LEFT_ACTIVE			363
#define IMG_CONVEYOR_BELT_4_RIGHT			364
#define IMG_CONVEYOR_BELT_4_RIGHT_ACTIVE		365
#define IMG_CONVEYOR_BELT_4_SWITCH_LEFT			366
#define IMG_CONVEYOR_BELT_4_SWITCH_MIDDLE		367
#define IMG_CONVEYOR_BELT_4_SWITCH_RIGHT		368
#define IMG_SWITCHGATE_SWITCH_UP			369
#define IMG_SWITCHGATE_SWITCH_DOWN			370
#define IMG_LIGHT_SWITCH				371
#define IMG_LIGHT_SWITCH_ACTIVE				372
#define IMG_TIMEGATE_SWITCH				373
#define IMG_TIMEGATE_SWITCH_ACTIVE			374
#define IMG_ENVELOPE_1					375
#define IMG_ENVELOPE_1_COLLECTING			376
#define IMG_ENVELOPE_2					377
#define IMG_ENVELOPE_2_COLLECTING			378
#define IMG_ENVELOPE_3					379
#define IMG_ENVELOPE_3_COLLECTING			380
#define IMG_ENVELOPE_4					381
#define IMG_ENVELOPE_4_COLLECTING			382
#define IMG_SIGN_EXCLAMATION				383
#define IMG_SIGN_STOP					384
#define IMG_LANDMINE					385
#define IMG_STEELWALL_SLIPPERY				386
#define IMG_EXTRA_TIME					387
#define IMG_SHIELD_NORMAL				388
#define IMG_SHIELD_NORMAL_ACTIVE			389
#define IMG_SHIELD_DEADLY				390
#define IMG_SHIELD_DEADLY_ACTIVE			391
#define IMG_SWITCHGATE_CLOSED				392
#define IMG_SWITCHGATE_OPENING				393
#define IMG_SWITCHGATE_OPEN				394
#define IMG_SWITCHGATE_CLOSING				395
#define IMG_TIMEGATE_CLOSED				396
#define IMG_TIMEGATE_OPENING				397
#define IMG_TIMEGATE_OPEN				398
#define IMG_TIMEGATE_CLOSING				399
#define IMG_PEARL					400
#define IMG_PEARL_BREAKING				401
#define IMG_CRYSTAL					402
#define IMG_WALL_PEARL					403
#define IMG_WALL_CRYSTAL				404
#define IMG_TUBE_RIGHT_DOWN				405
#define IMG_TUBE_HORIZONTAL_DOWN			406
#define IMG_TUBE_LEFT_DOWN				407
#define IMG_TUBE_HORIZONTAL				408
#define IMG_TUBE_VERTICAL_RIGHT				409
#define IMG_TUBE_ANY					410
#define IMG_TUBE_VERTICAL_LEFT				411
#define IMG_TUBE_VERTICAL				412
#define IMG_TUBE_RIGHT_UP				413
#define IMG_TUBE_HORIZONTAL_UP				414
#define IMG_TUBE_LEFT_UP				415
#define IMG_TRAP					416
#define IMG_TRAP_ACTIVE					417
#define IMG_DX_SUPABOMB					418
#define IMG_KEY_1					419
#define IMG_KEY_1_EDITOR				420
#define IMG_KEY_2					421
#define IMG_KEY_2_EDITOR				422
#define IMG_KEY_3					423
#define IMG_KEY_3_EDITOR				424
#define IMG_KEY_4					425
#define IMG_KEY_4_EDITOR				426
#define IMG_GATE_1					427
#define IMG_GATE_2					428
#define IMG_GATE_3					429
#define IMG_GATE_4					430
#define IMG_GATE_1_GRAY					431
#define IMG_GATE_1_GRAY_EDITOR				432
#define IMG_GATE_1_GRAY_ACTIVE				433
#define IMG_GATE_2_GRAY					434
#define IMG_GATE_2_GRAY_EDITOR				435
#define IMG_GATE_2_GRAY_ACTIVE				436
#define IMG_GATE_3_GRAY					437
#define IMG_GATE_3_GRAY_EDITOR				438
#define IMG_GATE_3_GRAY_ACTIVE				439
#define IMG_GATE_4_GRAY					440
#define IMG_GATE_4_GRAY_EDITOR				441
#define IMG_GATE_4_GRAY_ACTIVE				442
#define IMG_GAME_OF_LIFE				443
#define IMG_BIOMAZE					444
#define IMG_PACMAN					445
#define IMG_PACMAN_RIGHT				446
#define IMG_PACMAN_UP					447
#define IMG_PACMAN_LEFT					448
#define IMG_PACMAN_DOWN					449
#define IMG_PACMAN_TURNING_FROM_RIGHT			450
#define IMG_PACMAN_TURNING_FROM_UP			451
#define IMG_PACMAN_TURNING_FROM_LEFT			452
#define IMG_PACMAN_TURNING_FROM_DOWN			453
#define IMG_LAMP					454
#define IMG_LAMP_EDITOR					455
#define IMG_LAMP_ACTIVE					456
#define IMG_TIME_ORB_FULL				457
#define IMG_TIME_ORB_EMPTY				458
#define IMG_EMERALD_YELLOW				459
#define IMG_EMERALD_YELLOW_MOVING			460
#define IMG_EMERALD_YELLOW_FALLING			461
#define IMG_EMERALD_RED					462
#define IMG_EMERALD_RED_MOVING				463
#define IMG_EMERALD_RED_FALLING				464
#define IMG_EMERALD_PURPLE				465
#define IMG_EMERALD_PURPLE_MOVING			466
#define IMG_EMERALD_PURPLE_FALLING			467
#define IMG_WALL_EMERALD_YELLOW				468
#define IMG_WALL_EMERALD_RED				469
#define IMG_WALL_EMERALD_PURPLE				470
#define IMG_WALL_BD_DIAMOND				471
#define IMG_EXPANDABLE_WALL				472
#define IMG_EXPANDABLE_WALL_HORIZONTAL			473
#define IMG_EXPANDABLE_WALL_HORIZONTAL_EDITOR		474
#define IMG_EXPANDABLE_WALL_VERTICAL			475
#define IMG_EXPANDABLE_WALL_VERTICAL_EDITOR		476
#define IMG_EXPANDABLE_WALL_ANY				477
#define IMG_EXPANDABLE_WALL_ANY_EDITOR			478
#define IMG_BD_EXPANDABLE_WALL				479
#define IMG_BD_EXPANDABLE_WALL_EDITOR			480
#define IMG_EXPANDABLE_WALL_GROWING_LEFT		481
#define IMG_EXPANDABLE_WALL_GROWING_RIGHT		482
#define IMG_EXPANDABLE_WALL_GROWING_UP			483
#define IMG_EXPANDABLE_WALL_GROWING_DOWN		484
#define IMG_BLACK_ORB					485
#define IMG_SPEED_PILL					486
#define IMG_DARK_YAMYAM					487
#define IMG_DYNABOMB					488
#define IMG_DYNABOMB_ACTIVE				489
#define IMG_DYNABOMB_PLAYER_1				490
#define IMG_DYNABOMB_PLAYER_1_ACTIVE			491
#define IMG_DYNABOMB_PLAYER_2				492
#define IMG_DYNABOMB_PLAYER_2_ACTIVE			493
#define IMG_DYNABOMB_PLAYER_3				494
#define IMG_DYNABOMB_PLAYER_3_ACTIVE			495
#define IMG_DYNABOMB_PLAYER_4				496
#define IMG_DYNABOMB_PLAYER_4_ACTIVE			497
#define IMG_DYNABOMB_INCREASE_NUMBER			498
#define IMG_DYNABOMB_INCREASE_SIZE			499
#define IMG_DYNABOMB_INCREASE_POWER			500
#define IMG_PIG						501
#define IMG_PIG_DOWN					502
#define IMG_PIG_UP					503
#define IMG_PIG_LEFT					504
#define IMG_PIG_RIGHT					505
#define IMG_PIG_MOVING_DOWN				506
#define IMG_PIG_MOVING_UP				507
#define IMG_PIG_MOVING_LEFT				508
#define IMG_PIG_MOVING_RIGHT				509
#define IMG_PIG_DIGGING_DOWN				510
#define IMG_PIG_DIGGING_UP				511
#define IMG_PIG_DIGGING_LEFT				512
#define IMG_PIG_DIGGING_RIGHT				513
#define IMG_DRAGON					514
#define IMG_DRAGON_DOWN					515
#define IMG_DRAGON_UP					516
#define IMG_DRAGON_LEFT					517
#define IMG_DRAGON_RIGHT				518
#define IMG_DRAGON_MOVING_DOWN				519
#define IMG_DRAGON_MOVING_UP				520
#define IMG_DRAGON_MOVING_LEFT				521
#define IMG_DRAGON_MOVING_RIGHT				522
#define IMG_DRAGON_ATTACKING_DOWN			523
#define IMG_DRAGON_ATTACKING_UP				524
#define IMG_DRAGON_ATTACKING_LEFT			525
#define IMG_DRAGON_ATTACKING_RIGHT			526
#define IMG_MOLE					527
#define IMG_MOLE_DOWN					528
#define IMG_MOLE_UP					529
#define IMG_MOLE_LEFT					530
#define IMG_MOLE_RIGHT					531
#define IMG_MOLE_MOVING_DOWN				532
#define IMG_MOLE_MOVING_UP				533
#define IMG_MOLE_MOVING_LEFT				534
#define IMG_MOLE_MOVING_RIGHT				535
#define IMG_MOLE_DIGGING_DOWN				536
#define IMG_MOLE_DIGGING_UP				537
#define IMG_MOLE_DIGGING_LEFT				538
#define IMG_MOLE_DIGGING_RIGHT				539
#define IMG_PENGUIN					540
#define IMG_PENGUIN_EDITOR				541
#define IMG_PENGUIN_DOWN				542
#define IMG_PENGUIN_UP					543
#define IMG_PENGUIN_LEFT				544
#define IMG_PENGUIN_RIGHT				545
#define IMG_PENGUIN_MOVING_DOWN				546
#define IMG_PENGUIN_MOVING_UP				547
#define IMG_PENGUIN_MOVING_LEFT				548
#define IMG_PENGUIN_MOVING_RIGHT			549
#define IMG_SATELLITE					550
#define IMG_FLAMES_1_LEFT				551
#define IMG_FLAMES_2_LEFT				552
#define IMG_FLAMES_3_LEFT				553
#define IMG_FLAMES_1_RIGHT				554
#define IMG_FLAMES_2_RIGHT				555
#define IMG_FLAMES_3_RIGHT				556
#define IMG_FLAMES_1_UP					557
#define IMG_FLAMES_2_UP					558
#define IMG_FLAMES_3_UP					559
#define IMG_FLAMES_1_DOWN				560
#define IMG_FLAMES_2_DOWN				561
#define IMG_FLAMES_3_DOWN				562
#define IMG_STONEBLOCK					563
#define IMG_PLAYER_1					564
#define IMG_PLAYER_1_EDITOR				565
#define IMG_PLAYER_1_DOWN				566
#define IMG_PLAYER_1_UP					567
#define IMG_PLAYER_1_LEFT				568
#define IMG_PLAYER_1_RIGHT				569
#define IMG_PLAYER_1_MOVING_DOWN			570
#define IMG_PLAYER_1_MOVING_UP				571
#define IMG_PLAYER_1_MOVING_LEFT			572
#define IMG_PLAYER_1_MOVING_RIGHT			573
#define IMG_PLAYER_1_DIGGING_DOWN			574
#define IMG_PLAYER_1_DIGGING_UP				575
#define IMG_PLAYER_1_DIGGING_LEFT			576
#define IMG_PLAYER_1_DIGGING_RIGHT			577
#define IMG_PLAYER_1_COLLECTING_DOWN			578
#define IMG_PLAYER_1_COLLECTING_UP			579
#define IMG_PLAYER_1_COLLECTING_LEFT			580
#define IMG_PLAYER_1_COLLECTING_RIGHT			581
#define IMG_PLAYER_1_PUSHING_DOWN			582
#define IMG_PLAYER_1_PUSHING_UP				583
#define IMG_PLAYER_1_PUSHING_LEFT			584
#define IMG_PLAYER_1_PUSHING_RIGHT			585
#define IMG_PLAYER_1_SNAPPING_DOWN			586
#define IMG_PLAYER_1_SNAPPING_UP			587
#define IMG_PLAYER_1_SNAPPING_LEFT			588
#define IMG_PLAYER_1_SNAPPING_RIGHT			589
#define IMG_PLAYER_2					590
#define IMG_PLAYER_2_EDITOR				591
#define IMG_PLAYER_2_DOWN				592
#define IMG_PLAYER_2_UP					593
#define IMG_PLAYER_2_LEFT				594
#define IMG_PLAYER_2_RIGHT				595
#define IMG_PLAYER_2_MOVING_DOWN			596
#define IMG_PLAYER_2_MOVING_UP				597
#define IMG_PLAYER_2_MOVING_LEFT			598
#define IMG_PLAYER_2_MOVING_RIGHT			599
#define IMG_PLAYER_2_DIGGING_DOWN			600
#define IMG_PLAYER_2_DIGGING_UP				601
#define IMG_PLAYER_2_DIGGING_LEFT			602
#define IMG_PLAYER_2_DIGGING_RIGHT			603
#define IMG_PLAYER_2_COLLECTING_DOWN			604
#define IMG_PLAYER_2_COLLECTING_UP			605
#define IMG_PLAYER_2_COLLECTING_LEFT			606
#define IMG_PLAYER_2_COLLECTING_RIGHT			607
#define IMG_PLAYER_2_PUSHING_DOWN			608
#define IMG_PLAYER_2_PUSHING_UP				609
#define IMG_PLAYER_2_PUSHING_LEFT			610
#define IMG_PLAYER_2_PUSHING_RIGHT			611
#define IMG_PLAYER_2_SNAPPING_DOWN			612
#define IMG_PLAYER_2_SNAPPING_UP			613
#define IMG_PLAYER_2_SNAPPING_LEFT			614
#define IMG_PLAYER_2_SNAPPING_RIGHT			615
#define IMG_PLAYER_3					616
#define IMG_PLAYER_3_EDITOR				617
#define IMG_PLAYER_3_DOWN				618
#define IMG_PLAYER_3_UP					619
#define IMG_PLAYER_3_LEFT				620
#define IMG_PLAYER_3_RIGHT				621
#define IMG_PLAYER_3_MOVING_DOWN			622
#define IMG_PLAYER_3_MOVING_UP				623
#define IMG_PLAYER_3_MOVING_LEFT			624
#define IMG_PLAYER_3_MOVING_RIGHT			625
#define IMG_PLAYER_3_DIGGING_DOWN			626
#define IMG_PLAYER_3_DIGGING_UP				627
#define IMG_PLAYER_3_DIGGING_LEFT			628
#define IMG_PLAYER_3_DIGGING_RIGHT			629
#define IMG_PLAYER_3_COLLECTING_DOWN			630
#define IMG_PLAYER_3_COLLECTING_UP			631
#define IMG_PLAYER_3_COLLECTING_LEFT			632
#define IMG_PLAYER_3_COLLECTING_RIGHT			633
#define IMG_PLAYER_3_PUSHING_DOWN			634
#define IMG_PLAYER_3_PUSHING_UP				635
#define IMG_PLAYER_3_PUSHING_LEFT			636
#define IMG_PLAYER_3_PUSHING_RIGHT			637
#define IMG_PLAYER_3_SNAPPING_DOWN			638
#define IMG_PLAYER_3_SNAPPING_UP			639
#define IMG_PLAYER_3_SNAPPING_LEFT			640
#define IMG_PLAYER_3_SNAPPING_RIGHT			641
#define IMG_PLAYER_4					642
#define IMG_PLAYER_4_EDITOR				643
#define IMG_PLAYER_4_DOWN				644
#define IMG_PLAYER_4_UP					645
#define IMG_PLAYER_4_LEFT				646
#define IMG_PLAYER_4_RIGHT				647
#define IMG_PLAYER_4_MOVING_DOWN			648
#define IMG_PLAYER_4_MOVING_UP				649
#define IMG_PLAYER_4_MOVING_LEFT			650
#define IMG_PLAYER_4_MOVING_RIGHT			651
#define IMG_PLAYER_4_DIGGING_DOWN			652
#define IMG_PLAYER_4_DIGGING_UP				653
#define IMG_PLAYER_4_DIGGING_LEFT			654
#define IMG_PLAYER_4_DIGGING_RIGHT			655
#define IMG_PLAYER_4_COLLECTING_DOWN			656
#define IMG_PLAYER_4_COLLECTING_UP			657
#define IMG_PLAYER_4_COLLECTING_LEFT			658
#define IMG_PLAYER_4_COLLECTING_RIGHT			659
#define IMG_PLAYER_4_PUSHING_DOWN			660
#define IMG_PLAYER_4_PUSHING_UP				661
#define IMG_PLAYER_4_PUSHING_LEFT			662
#define IMG_PLAYER_4_PUSHING_RIGHT			663
#define IMG_PLAYER_4_SNAPPING_DOWN			664
#define IMG_PLAYER_4_SNAPPING_UP			665
#define IMG_PLAYER_4_SNAPPING_LEFT			666
#define IMG_PLAYER_4_SNAPPING_RIGHT			667
#define IMG_DEFAULT_EXPLODING				668
#define IMG_TWINKLE_BLUE				669
#define IMG_TWINKLE_WHITE				670
#define IMG_STEELWALL_TOPLEFT				671
#define IMG_STEELWALL_TOPRIGHT				672
#define IMG_STEELWALL_BOTTOMLEFT			673
#define IMG_STEELWALL_BOTTOMRIGHT			674
#define IMG_STEELWALL_HORIZONTAL			675
#define IMG_STEELWALL_VERTICAL				676
#define IMG_STEELWALL_TOPLEFT_EDITOR			677
#define IMG_STEELWALL_TOPRIGHT_EDITOR			678
#define IMG_STEELWALL_BOTTOMLEFT_EDITOR			679
#define IMG_STEELWALL_BOTTOMRIGHT_EDITOR		680
#define IMG_STEELWALL_HORIZONTAL_EDITOR			681
#define IMG_STEELWALL_VERTICAL_EDITOR			682
#define IMG_INVISIBLE_STEELWALL_TOPLEFT			683
#define IMG_INVISIBLE_STEELWALL_TOPRIGHT		684
#define IMG_INVISIBLE_STEELWALL_BOTTOMLEFT		685
#define IMG_INVISIBLE_STEELWALL_BOTTOMRIGHT		686
#define IMG_INVISIBLE_STEELWALL_HORIZONTAL		687
#define IMG_INVISIBLE_STEELWALL_VERTICAL		688
#define IMG_INVISIBLE_STEELWALL_TOPLEFT_EDITOR		689
#define IMG_INVISIBLE_STEELWALL_TOPRIGHT_EDITOR		690
#define IMG_INVISIBLE_STEELWALL_BOTTOMLEFT_EDITOR	691
#define IMG_INVISIBLE_STEELWALL_BOTTOMRIGHT_EDITOR	692
#define IMG_INVISIBLE_STEELWALL_HORIZONTAL_EDITOR	693
#define IMG_INVISIBLE_STEELWALL_VERTICAL_EDITOR		694
#define IMG_ARROW_LEFT					695
#define IMG_ARROW_RIGHT					696
#define IMG_ARROW_UP					697
#define IMG_ARROW_DOWN					698
#define IMG_UNKNOWN					699
#define IMG_TRIGGER_ELEMENT				700
#define IMG_TRIGGER_PLAYER				701
#define IMG_TRIGGER_CE_VALUE				702
#define IMG_TRIGGER_CE_SCORE				703
#define IMG_CURRENT_CE_VALUE				704
#define IMG_CURRENT_CE_SCORE				705
#define IMG_PREV_CE_1					706
#define IMG_PREV_CE_2					707
#define IMG_PREV_CE_3					708
#define IMG_PREV_CE_4					709
#define IMG_PREV_CE_5					710
#define IMG_PREV_CE_6					711
#define IMG_PREV_CE_7					712
#define IMG_PREV_CE_8					713
#define IMG_NEXT_CE_1					714
#define IMG_NEXT_CE_2					715
#define IMG_NEXT_CE_3					716
#define IMG_NEXT_CE_4					717
#define IMG_NEXT_CE_5					718
#define IMG_NEXT_CE_6					719
#define IMG_NEXT_CE_7					720
#define IMG_NEXT_CE_8					721
#define IMG_SELF					722
#define IMG_ANY_ELEMENT					723
#define IMG_EMC_KEY_5					724
#define IMG_EMC_KEY_6					725
#define IMG_EMC_KEY_7					726
#define IMG_EMC_KEY_8					727
#define IMG_EMC_GATE_5					728
#define IMG_EMC_GATE_6					729
#define IMG_EMC_GATE_7					730
#define IMG_EMC_GATE_8					731
#define IMG_EMC_GATE_5_GRAY				732
#define IMG_EMC_GATE_5_GRAY_EDITOR			733
#define IMG_EMC_GATE_5_GRAY_ACTIVE			734
#define IMG_EMC_GATE_6_GRAY				735
#define IMG_EMC_GATE_6_GRAY_EDITOR			736
#define IMG_EMC_GATE_6_GRAY_ACTIVE			737
#define IMG_EMC_GATE_7_GRAY				738
#define IMG_EMC_GATE_7_GRAY_EDITOR			739
#define IMG_EMC_GATE_7_GRAY_ACTIVE			740
#define IMG_EMC_GATE_8_GRAY				741
#define IMG_EMC_GATE_8_GRAY_EDITOR			742
#define IMG_EMC_GATE_8_GRAY_ACTIVE			743
#define IMG_EMC_ANDROID					744
#define IMG_EMC_ANDROID_SHRINKING_UPLEFT		745
#define IMG_EMC_ANDROID_GROWING_DOWNRIGHT		746
#define IMG_EMC_ANDROID_SHRINKING_DOWNLEFT		747
#define IMG_EMC_ANDROID_GROWING_UPRIGHT			748
#define IMG_EMC_ANDROID_SHRINKING_UPRIGHT		749
#define IMG_EMC_ANDROID_GROWING_DOWNLEFT		750
#define IMG_EMC_ANDROID_SHRINKING_DOWNRIGHT		751
#define IMG_EMC_ANDROID_GROWING_UPLEFT			752
#define IMG_EMC_GRASS					753
#define IMG_EMC_GRASS_CRUMBLED				754
#define IMG_EMC_GRASS_DIGGING_LEFT			755
#define IMG_EMC_GRASS_DIGGING_RIGHT			756
#define IMG_EMC_GRASS_DIGGING_UP			757
#define IMG_EMC_GRASS_DIGGING_DOWN			758
#define IMG_EMC_GRASS_DIGGING_LEFT_CRUMBLED		759
#define IMG_EMC_GRASS_DIGGING_RIGHT_CRUMBLED		760
#define IMG_EMC_GRASS_DIGGING_UP_CRUMBLED		761
#define IMG_EMC_GRASS_DIGGING_DOWN_CRUMBLED		762
#define IMG_EMC_MAGIC_BALL				763
#define IMG_EMC_MAGIC_BALL_ACTIVE			764
#define IMG_EMC_MAGIC_BALL_DROPPING			765
#define IMG_EMC_MAGIC_BALL_SWITCH			766
#define IMG_EMC_MAGIC_BALL_SWITCH_ACTIVE		767
#define IMG_EMC_SPRING_BUMPER				768
#define IMG_EMC_SPRING_BUMPER_ACTIVE			769
#define IMG_EMC_PLANT					770
#define IMG_EMC_PLANT_CRUMBLED				771
#define IMG_EMC_LENSES					772
#define IMG_EMC_MAGNIFIER				773
#define IMG_EMC_WALL_9					774
#define IMG_EMC_WALL_10					775
#define IMG_EMC_WALL_11					776
#define IMG_EMC_WALL_12					777
#define IMG_EMC_WALL_13					778
#define IMG_EMC_WALL_14					779
#define IMG_EMC_WALL_15					780
#define IMG_EMC_WALL_16					781
#define IMG_EMC_WALL_SLIPPERY_1				782
#define IMG_EMC_WALL_SLIPPERY_2				783
#define IMG_EMC_WALL_SLIPPERY_3				784
#define IMG_EMC_WALL_SLIPPERY_4				785
#define IMG_EMC_FAKE_GRASS				786
#define IMG_EMC_FAKE_GRASS_CRUMBLED			787
#define IMG_EMC_FAKE_GRASS_ACTIVE			788
#define IMG_EMC_FAKE_GRASS_ACTIVE_CRUMBLED		789
#define IMG_EMC_FAKE_GRASS_EDITOR			790
#define IMG_EMC_FAKE_ACID				791
#define IMG_EMC_DRIPPER					792
#define IMG_EMC_DRIPPER_EDITOR				793
#define IMG_EMC_DRIPPER_ACTIVE				794
#define IMG_CHAR_SPACE					795
#define IMG_CHAR_SPACE_EDITOR				796
#define IMG_CHAR_EXCLAM					797
#define IMG_CHAR_QUOTEDBL				798
#define IMG_CHAR_NUMBERSIGN				799
#define IMG_CHAR_DOLLAR					800
#define IMG_CHAR_PERCENT				801
#define IMG_CHAR_AMPERSAND				802
#define IMG_CHAR_APOSTROPHE				803
#define IMG_CHAR_PARENLEFT				804
#define IMG_CHAR_PARENRIGHT				805
#define IMG_CHAR_ASTERISK				806
#define IMG_CHAR_PLUS					807
#define IMG_CHAR_COMMA					808
#define IMG_CHAR_MINUS					809
#define IMG_CHAR_PERIOD					810
#define IMG_CHAR_SLASH					811
#define IMG_CHAR_0					812
#define IMG_CHAR_1					813
#define IMG_CHAR_2					814
#define IMG_CHAR_3					815
#define IMG_CHAR_4					816
#define IMG_CHAR_5					817
#define IMG_CHAR_6					818
#define IMG_CHAR_7					819
#define IMG_CHAR_8					820
#define IMG_CHAR_9					821
#define IMG_CHAR_COLON					822
#define IMG_CHAR_SEMICOLON				823
#define IMG_CHAR_LESS					824
#define IMG_CHAR_EQUAL					825
#define IMG_CHAR_GREATER				826
#define IMG_CHAR_QUESTION				827
#define IMG_CHAR_AT					828
#define IMG_CHAR_A					829
#define IMG_CHAR_B					830
#define IMG_CHAR_C					831
#define IMG_CHAR_D					832
#define IMG_CHAR_E					833
#define IMG_CHAR_F					834
#define IMG_CHAR_G					835
#define IMG_CHAR_H					836
#define IMG_CHAR_I					837
#define IMG_CHAR_J					838
#define IMG_CHAR_K					839
#define IMG_CHAR_L					840
#define IMG_CHAR_M					841
#define IMG_CHAR_N					842
#define IMG_CHAR_O					843
#define IMG_CHAR_P					844
#define IMG_CHAR_Q					845
#define IMG_CHAR_R					846
#define IMG_CHAR_S					847
#define IMG_CHAR_T					848
#define IMG_CHAR_U					849
#define IMG_CHAR_V					850
#define IMG_CHAR_W					851
#define IMG_CHAR_X					852
#define IMG_CHAR_Y					853
#define IMG_CHAR_Z					854
#define IMG_CHAR_BRACKETLEFT				855
#define IMG_CHAR_BACKSLASH				856
#define IMG_CHAR_BRACKETRIGHT				857
#define IMG_CHAR_ASCIICIRCUM				858
#define IMG_CHAR_UNDERSCORE				859
#define IMG_CHAR_COPYRIGHT				860
#define IMG_CHAR_AUMLAUT				861
#define IMG_CHAR_OUMLAUT				862
#define IMG_CHAR_UUMLAUT				863
#define IMG_CHAR_DEGREE					864
#define IMG_CHAR_TRADEMARK				865
#define IMG_CHAR_CURSOR					866
#define IMG_CUSTOM_1					867
#define IMG_CUSTOM_1_EDITOR				868
#define IMG_CUSTOM_2					869
#define IMG_CUSTOM_2_EDITOR				870
#define IMG_CUSTOM_3					871
#define IMG_CUSTOM_3_EDITOR				872
#define IMG_CUSTOM_4					873
#define IMG_CUSTOM_4_EDITOR				874
#define IMG_CUSTOM_5					875
#define IMG_CUSTOM_5_EDITOR				876
#define IMG_CUSTOM_6					877
#define IMG_CUSTOM_6_EDITOR				878
#define IMG_CUSTOM_7					879
#define IMG_CUSTOM_7_EDITOR				880
#define IMG_CUSTOM_8					881
#define IMG_CUSTOM_8_EDITOR				882
#define IMG_CUSTOM_9					883
#define IMG_CUSTOM_9_EDITOR				884
#define IMG_CUSTOM_10					885
#define IMG_CUSTOM_10_EDITOR				886
#define IMG_CUSTOM_11					887
#define IMG_CUSTOM_11_EDITOR				888
#define IMG_CUSTOM_12					889
#define IMG_CUSTOM_12_EDITOR				890
#define IMG_CUSTOM_13					891
#define IMG_CUSTOM_13_EDITOR				892
#define IMG_CUSTOM_14					893
#define IMG_CUSTOM_14_EDITOR				894
#define IMG_CUSTOM_15					895
#define IMG_CUSTOM_15_EDITOR				896
#define IMG_CUSTOM_16					897
#define IMG_CUSTOM_16_EDITOR				898
#define IMG_CUSTOM_17					899
#define IMG_CUSTOM_17_EDITOR				900
#define IMG_CUSTOM_18					901
#define IMG_CUSTOM_18_EDITOR				902
#define IMG_CUSTOM_19					903
#define IMG_CUSTOM_19_EDITOR				904
#define IMG_CUSTOM_20					905
#define IMG_CUSTOM_20_EDITOR				906
#define IMG_CUSTOM_21					907
#define IMG_CUSTOM_21_EDITOR				908
#define IMG_CUSTOM_22					909
#define IMG_CUSTOM_22_EDITOR				910
#define IMG_CUSTOM_23					911
#define IMG_CUSTOM_23_EDITOR				912
#define IMG_CUSTOM_24					913
#define IMG_CUSTOM_24_EDITOR				914
#define IMG_CUSTOM_25					915
#define IMG_CUSTOM_25_EDITOR				916
#define IMG_CUSTOM_26					917
#define IMG_CUSTOM_26_EDITOR				918
#define IMG_CUSTOM_27					919
#define IMG_CUSTOM_27_EDITOR				920
#define IMG_CUSTOM_28					921
#define IMG_CUSTOM_28_EDITOR				922
#define IMG_CUSTOM_29					923
#define IMG_CUSTOM_29_EDITOR				924
#define IMG_CUSTOM_30					925
#define IMG_CUSTOM_30_EDITOR				926
#define IMG_CUSTOM_31					927
#define IMG_CUSTOM_31_EDITOR				928
#define IMG_CUSTOM_32					929
#define IMG_CUSTOM_32_EDITOR				930
#define IMG_CUSTOM_33					931
#define IMG_CUSTOM_33_EDITOR				932
#define IMG_CUSTOM_34					933
#define IMG_CUSTOM_34_EDITOR				934
#define IMG_CUSTOM_35					935
#define IMG_CUSTOM_35_EDITOR				936
#define IMG_CUSTOM_36					937
#define IMG_CUSTOM_36_EDITOR				938
#define IMG_CUSTOM_37					939
#define IMG_CUSTOM_37_EDITOR				940
#define IMG_CUSTOM_38					941
#define IMG_CUSTOM_38_EDITOR				942
#define IMG_CUSTOM_39					943
#define IMG_CUSTOM_39_EDITOR				944
#define IMG_CUSTOM_40					945
#define IMG_CUSTOM_40_EDITOR				946
#define IMG_CUSTOM_41					947
#define IMG_CUSTOM_41_EDITOR				948
#define IMG_CUSTOM_42					949
#define IMG_CUSTOM_42_EDITOR				950
#define IMG_CUSTOM_43					951
#define IMG_CUSTOM_43_EDITOR				952
#define IMG_CUSTOM_44					953
#define IMG_CUSTOM_44_EDITOR				954
#define IMG_CUSTOM_45					955
#define IMG_CUSTOM_45_EDITOR				956
#define IMG_CUSTOM_46					957
#define IMG_CUSTOM_46_EDITOR				958
#define IMG_CUSTOM_47					959
#define IMG_CUSTOM_47_EDITOR				960
#define IMG_CUSTOM_48					961
#define IMG_CUSTOM_48_EDITOR				962
#define IMG_CUSTOM_49					963
#define IMG_CUSTOM_49_EDITOR				964
#define IMG_CUSTOM_50					965
#define IMG_CUSTOM_50_EDITOR				966
#define IMG_CUSTOM_51					967
#define IMG_CUSTOM_51_EDITOR				968
#define IMG_CUSTOM_52					969
#define IMG_CUSTOM_52_EDITOR				970
#define IMG_CUSTOM_53					971
#define IMG_CUSTOM_53_EDITOR				972
#define IMG_CUSTOM_54					973
#define IMG_CUSTOM_54_EDITOR				974
#define IMG_CUSTOM_55					975
#define IMG_CUSTOM_55_EDITOR				976
#define IMG_CUSTOM_56					977
#define IMG_CUSTOM_56_EDITOR				978
#define IMG_CUSTOM_57					979
#define IMG_CUSTOM_57_EDITOR				980
#define IMG_CUSTOM_58					981
#define IMG_CUSTOM_58_EDITOR				982
#define IMG_CUSTOM_59					983
#define IMG_CUSTOM_59_EDITOR				984
#define IMG_CUSTOM_60					985
#define IMG_CUSTOM_60_EDITOR				986
#define IMG_CUSTOM_61					987
#define IMG_CUSTOM_61_EDITOR				988
#define IMG_CUSTOM_62					989
#define IMG_CUSTOM_62_EDITOR				990
#define IMG_CUSTOM_63					991
#define IMG_CUSTOM_63_EDITOR				992
#define IMG_CUSTOM_64					993
#define IMG_CUSTOM_64_EDITOR				994
#define IMG_CUSTOM_65					995
#define IMG_CUSTOM_65_EDITOR				996
#define IMG_CUSTOM_66					997
#define IMG_CUSTOM_66_EDITOR				998
#define IMG_CUSTOM_67					999
#define IMG_CUSTOM_67_EDITOR				1000
#define IMG_CUSTOM_68					1001
#define IMG_CUSTOM_68_EDITOR				1002
#define IMG_CUSTOM_69					1003
#define IMG_CUSTOM_69_EDITOR				1004
#define IMG_CUSTOM_70					1005
#define IMG_CUSTOM_70_EDITOR				1006
#define IMG_CUSTOM_71					1007
#define IMG_CUSTOM_71_EDITOR				1008
#define IMG_CUSTOM_72					1009
#define IMG_CUSTOM_72_EDITOR				1010
#define IMG_CUSTOM_73					1011
#define IMG_CUSTOM_73_EDITOR				1012
#define IMG_CUSTOM_74					1013
#define IMG_CUSTOM_74_EDITOR				1014
#define IMG_CUSTOM_75					1015
#define IMG_CUSTOM_75_EDITOR				1016
#define IMG_CUSTOM_76					1017
#define IMG_CUSTOM_76_EDITOR				1018
#define IMG_CUSTOM_77					1019
#define IMG_CUSTOM_77_EDITOR				1020
#define IMG_CUSTOM_78					1021
#define IMG_CUSTOM_78_EDITOR				1022
#define IMG_CUSTOM_79					1023
#define IMG_CUSTOM_79_EDITOR				1024
#define IMG_CUSTOM_80					1025
#define IMG_CUSTOM_80_EDITOR				1026
#define IMG_CUSTOM_81					1027
#define IMG_CUSTOM_81_EDITOR				1028
#define IMG_CUSTOM_82					1029
#define IMG_CUSTOM_82_EDITOR				1030
#define IMG_CUSTOM_83					1031
#define IMG_CUSTOM_83_EDITOR				1032
#define IMG_CUSTOM_84					1033
#define IMG_CUSTOM_84_EDITOR				1034
#define IMG_CUSTOM_85					1035
#define IMG_CUSTOM_85_EDITOR				1036
#define IMG_CUSTOM_86					1037
#define IMG_CUSTOM_86_EDITOR				1038
#define IMG_CUSTOM_87					1039
#define IMG_CUSTOM_87_EDITOR				1040
#define IMG_CUSTOM_88					1041
#define IMG_CUSTOM_88_EDITOR				1042
#define IMG_CUSTOM_89					1043
#define IMG_CUSTOM_89_EDITOR				1044
#define IMG_CUSTOM_90					1045
#define IMG_CUSTOM_90_EDITOR				1046
#define IMG_CUSTOM_91					1047
#define IMG_CUSTOM_91_EDITOR				1048
#define IMG_CUSTOM_92					1049
#define IMG_CUSTOM_92_EDITOR				1050
#define IMG_CUSTOM_93					1051
#define IMG_CUSTOM_93_EDITOR				1052
#define IMG_CUSTOM_94					1053
#define IMG_CUSTOM_94_EDITOR				1054
#define IMG_CUSTOM_95					1055
#define IMG_CUSTOM_95_EDITOR				1056
#define IMG_CUSTOM_96					1057
#define IMG_CUSTOM_96_EDITOR				1058
#define IMG_CUSTOM_97					1059
#define IMG_CUSTOM_97_EDITOR				1060
#define IMG_CUSTOM_98					1061
#define IMG_CUSTOM_98_EDITOR				1062
#define IMG_CUSTOM_99					1063
#define IMG_CUSTOM_99_EDITOR				1064
#define IMG_CUSTOM_100					1065
#define IMG_CUSTOM_100_EDITOR				1066
#define IMG_CUSTOM_101					1067
#define IMG_CUSTOM_101_EDITOR				1068
#define IMG_CUSTOM_102					1069
#define IMG_CUSTOM_102_EDITOR				1070
#define IMG_CUSTOM_103					1071
#define IMG_CUSTOM_103_EDITOR				1072
#define IMG_CUSTOM_104					1073
#define IMG_CUSTOM_104_EDITOR				1074
#define IMG_CUSTOM_105					1075
#define IMG_CUSTOM_105_EDITOR				1076
#define IMG_CUSTOM_106					1077
#define IMG_CUSTOM_106_EDITOR				1078
#define IMG_CUSTOM_107					1079
#define IMG_CUSTOM_107_EDITOR				1080
#define IMG_CUSTOM_108					1081
#define IMG_CUSTOM_108_EDITOR				1082
#define IMG_CUSTOM_109					1083
#define IMG_CUSTOM_109_EDITOR				1084
#define IMG_CUSTOM_110					1085
#define IMG_CUSTOM_110_EDITOR				1086
#define IMG_CUSTOM_111					1087
#define IMG_CUSTOM_111_EDITOR				1088
#define IMG_CUSTOM_112					1089
#define IMG_CUSTOM_112_EDITOR				1090
#define IMG_CUSTOM_113					1091
#define IMG_CUSTOM_113_EDITOR				1092
#define IMG_CUSTOM_114					1093
#define IMG_CUSTOM_114_EDITOR				1094
#define IMG_CUSTOM_115					1095
#define IMG_CUSTOM_115_EDITOR				1096
#define IMG_CUSTOM_116					1097
#define IMG_CUSTOM_116_EDITOR				1098
#define IMG_CUSTOM_117					1099
#define IMG_CUSTOM_117_EDITOR				1100
#define IMG_CUSTOM_118					1101
#define IMG_CUSTOM_118_EDITOR				1102
#define IMG_CUSTOM_119					1103
#define IMG_CUSTOM_119_EDITOR				1104
#define IMG_CUSTOM_120					1105
#define IMG_CUSTOM_120_EDITOR				1106
#define IMG_CUSTOM_121					1107
#define IMG_CUSTOM_121_EDITOR				1108
#define IMG_CUSTOM_122					1109
#define IMG_CUSTOM_122_EDITOR				1110
#define IMG_CUSTOM_123					1111
#define IMG_CUSTOM_123_EDITOR				1112
#define IMG_CUSTOM_124					1113
#define IMG_CUSTOM_124_EDITOR				1114
#define IMG_CUSTOM_125					1115
#define IMG_CUSTOM_125_EDITOR				1116
#define IMG_CUSTOM_126					1117
#define IMG_CUSTOM_126_EDITOR				1118
#define IMG_CUSTOM_127					1119
#define IMG_CUSTOM_127_EDITOR				1120
#define IMG_CUSTOM_128					1121
#define IMG_CUSTOM_128_EDITOR				1122
#define IMG_CUSTOM_129					1123
#define IMG_CUSTOM_129_EDITOR				1124
#define IMG_CUSTOM_130					1125
#define IMG_CUSTOM_130_EDITOR				1126
#define IMG_CUSTOM_131					1127
#define IMG_CUSTOM_131_EDITOR				1128
#define IMG_CUSTOM_132					1129
#define IMG_CUSTOM_132_EDITOR				1130
#define IMG_CUSTOM_133					1131
#define IMG_CUSTOM_133_EDITOR				1132
#define IMG_CUSTOM_134					1133
#define IMG_CUSTOM_134_EDITOR				1134
#define IMG_CUSTOM_135					1135
#define IMG_CUSTOM_135_EDITOR				1136
#define IMG_CUSTOM_136					1137
#define IMG_CUSTOM_136_EDITOR				1138
#define IMG_CUSTOM_137					1139
#define IMG_CUSTOM_137_EDITOR				1140
#define IMG_CUSTOM_138					1141
#define IMG_CUSTOM_138_EDITOR				1142
#define IMG_CUSTOM_139					1143
#define IMG_CUSTOM_139_EDITOR				1144
#define IMG_CUSTOM_140					1145
#define IMG_CUSTOM_140_EDITOR				1146
#define IMG_CUSTOM_141					1147
#define IMG_CUSTOM_141_EDITOR				1148
#define IMG_CUSTOM_142					1149
#define IMG_CUSTOM_142_EDITOR				1150
#define IMG_CUSTOM_143					1151
#define IMG_CUSTOM_143_EDITOR				1152
#define IMG_CUSTOM_144					1153
#define IMG_CUSTOM_144_EDITOR				1154
#define IMG_CUSTOM_145					1155
#define IMG_CUSTOM_145_EDITOR				1156
#define IMG_CUSTOM_146					1157
#define IMG_CUSTOM_146_EDITOR				1158
#define IMG_CUSTOM_147					1159
#define IMG_CUSTOM_147_EDITOR				1160
#define IMG_CUSTOM_148					1161
#define IMG_CUSTOM_148_EDITOR				1162
#define IMG_CUSTOM_149					1163
#define IMG_CUSTOM_149_EDITOR				1164
#define IMG_CUSTOM_150					1165
#define IMG_CUSTOM_150_EDITOR				1166
#define IMG_CUSTOM_151					1167
#define IMG_CUSTOM_151_EDITOR				1168
#define IMG_CUSTOM_152					1169
#define IMG_CUSTOM_152_EDITOR				1170
#define IMG_CUSTOM_153					1171
#define IMG_CUSTOM_153_EDITOR				1172
#define IMG_CUSTOM_154					1173
#define IMG_CUSTOM_154_EDITOR				1174
#define IMG_CUSTOM_155					1175
#define IMG_CUSTOM_155_EDITOR				1176
#define IMG_CUSTOM_156					1177
#define IMG_CUSTOM_156_EDITOR				1178
#define IMG_CUSTOM_157					1179
#define IMG_CUSTOM_157_EDITOR				1180
#define IMG_CUSTOM_158					1181
#define IMG_CUSTOM_158_EDITOR				1182
#define IMG_CUSTOM_159					1183
#define IMG_CUSTOM_159_EDITOR				1184
#define IMG_CUSTOM_160					1185
#define IMG_CUSTOM_160_EDITOR				1186
#define IMG_CUSTOM_161					1187
#define IMG_CUSTOM_161_EDITOR				1188
#define IMG_CUSTOM_162					1189
#define IMG_CUSTOM_162_EDITOR				1190
#define IMG_CUSTOM_163					1191
#define IMG_CUSTOM_163_EDITOR				1192
#define IMG_CUSTOM_164					1193
#define IMG_CUSTOM_164_EDITOR				1194
#define IMG_CUSTOM_165					1195
#define IMG_CUSTOM_165_EDITOR				1196
#define IMG_CUSTOM_166					1197
#define IMG_CUSTOM_166_EDITOR				1198
#define IMG_CUSTOM_167					1199
#define IMG_CUSTOM_167_EDITOR				1200
#define IMG_CUSTOM_168					1201
#define IMG_CUSTOM_168_EDITOR				1202
#define IMG_CUSTOM_169					1203
#define IMG_CUSTOM_169_EDITOR				1204
#define IMG_CUSTOM_170					1205
#define IMG_CUSTOM_170_EDITOR				1206
#define IMG_CUSTOM_171					1207
#define IMG_CUSTOM_171_EDITOR				1208
#define IMG_CUSTOM_172					1209
#define IMG_CUSTOM_172_EDITOR				1210
#define IMG_CUSTOM_173					1211
#define IMG_CUSTOM_173_EDITOR				1212
#define IMG_CUSTOM_174					1213
#define IMG_CUSTOM_174_EDITOR				1214
#define IMG_CUSTOM_175					1215
#define IMG_CUSTOM_175_EDITOR				1216
#define IMG_CUSTOM_176					1217
#define IMG_CUSTOM_176_EDITOR				1218
#define IMG_CUSTOM_177					1219
#define IMG_CUSTOM_177_EDITOR				1220
#define IMG_CUSTOM_178					1221
#define IMG_CUSTOM_178_EDITOR				1222
#define IMG_CUSTOM_179					1223
#define IMG_CUSTOM_179_EDITOR				1224
#define IMG_CUSTOM_180					1225
#define IMG_CUSTOM_180_EDITOR				1226
#define IMG_CUSTOM_181					1227
#define IMG_CUSTOM_181_EDITOR				1228
#define IMG_CUSTOM_182					1229
#define IMG_CUSTOM_182_EDITOR				1230
#define IMG_CUSTOM_183					1231
#define IMG_CUSTOM_183_EDITOR				1232
#define IMG_CUSTOM_184					1233
#define IMG_CUSTOM_184_EDITOR				1234
#define IMG_CUSTOM_185					1235
#define IMG_CUSTOM_185_EDITOR				1236
#define IMG_CUSTOM_186					1237
#define IMG_CUSTOM_186_EDITOR				1238
#define IMG_CUSTOM_187					1239
#define IMG_CUSTOM_187_EDITOR				1240
#define IMG_CUSTOM_188					1241
#define IMG_CUSTOM_188_EDITOR				1242
#define IMG_CUSTOM_189					1243
#define IMG_CUSTOM_189_EDITOR				1244
#define IMG_CUSTOM_190					1245
#define IMG_CUSTOM_190_EDITOR				1246
#define IMG_CUSTOM_191					1247
#define IMG_CUSTOM_191_EDITOR				1248
#define IMG_CUSTOM_192					1249
#define IMG_CUSTOM_192_EDITOR				1250
#define IMG_CUSTOM_193					1251
#define IMG_CUSTOM_193_EDITOR				1252
#define IMG_CUSTOM_194					1253
#define IMG_CUSTOM_194_EDITOR				1254
#define IMG_CUSTOM_195					1255
#define IMG_CUSTOM_195_EDITOR				1256
#define IMG_CUSTOM_196					1257
#define IMG_CUSTOM_196_EDITOR				1258
#define IMG_CUSTOM_197					1259
#define IMG_CUSTOM_197_EDITOR				1260
#define IMG_CUSTOM_198					1261
#define IMG_CUSTOM_198_EDITOR				1262
#define IMG_CUSTOM_199					1263
#define IMG_CUSTOM_199_EDITOR				1264
#define IMG_CUSTOM_200					1265
#define IMG_CUSTOM_200_EDITOR				1266
#define IMG_CUSTOM_201					1267
#define IMG_CUSTOM_201_EDITOR				1268
#define IMG_CUSTOM_202					1269
#define IMG_CUSTOM_202_EDITOR				1270
#define IMG_CUSTOM_203					1271
#define IMG_CUSTOM_203_EDITOR				1272
#define IMG_CUSTOM_204					1273
#define IMG_CUSTOM_204_EDITOR				1274
#define IMG_CUSTOM_205					1275
#define IMG_CUSTOM_205_EDITOR				1276
#define IMG_CUSTOM_206					1277
#define IMG_CUSTOM_206_EDITOR				1278
#define IMG_CUSTOM_207					1279
#define IMG_CUSTOM_207_EDITOR				1280
#define IMG_CUSTOM_208					1281
#define IMG_CUSTOM_208_EDITOR				1282
#define IMG_CUSTOM_209					1283
#define IMG_CUSTOM_209_EDITOR				1284
#define IMG_CUSTOM_210					1285
#define IMG_CUSTOM_210_EDITOR				1286
#define IMG_CUSTOM_211					1287
#define IMG_CUSTOM_211_EDITOR				1288
#define IMG_CUSTOM_212					1289
#define IMG_CUSTOM_212_EDITOR				1290
#define IMG_CUSTOM_213					1291
#define IMG_CUSTOM_213_EDITOR				1292
#define IMG_CUSTOM_214					1293
#define IMG_CUSTOM_214_EDITOR				1294
#define IMG_CUSTOM_215					1295
#define IMG_CUSTOM_215_EDITOR				1296
#define IMG_CUSTOM_216					1297
#define IMG_CUSTOM_216_EDITOR				1298
#define IMG_CUSTOM_217					1299
#define IMG_CUSTOM_217_EDITOR				1300
#define IMG_CUSTOM_218					1301
#define IMG_CUSTOM_218_EDITOR				1302
#define IMG_CUSTOM_219					1303
#define IMG_CUSTOM_219_EDITOR				1304
#define IMG_CUSTOM_220					1305
#define IMG_CUSTOM_220_EDITOR				1306
#define IMG_CUSTOM_221					1307
#define IMG_CUSTOM_221_EDITOR				1308
#define IMG_CUSTOM_222					1309
#define IMG_CUSTOM_222_EDITOR				1310
#define IMG_CUSTOM_223					1311
#define IMG_CUSTOM_223_EDITOR				1312
#define IMG_CUSTOM_224					1313
#define IMG_CUSTOM_224_EDITOR				1314
#define IMG_CUSTOM_225					1315
#define IMG_CUSTOM_225_EDITOR				1316
#define IMG_CUSTOM_226					1317
#define IMG_CUSTOM_226_EDITOR				1318
#define IMG_CUSTOM_227					1319
#define IMG_CUSTOM_227_EDITOR				1320
#define IMG_CUSTOM_228					1321
#define IMG_CUSTOM_228_EDITOR				1322
#define IMG_CUSTOM_229					1323
#define IMG_CUSTOM_229_EDITOR				1324
#define IMG_CUSTOM_230					1325
#define IMG_CUSTOM_230_EDITOR				1326
#define IMG_CUSTOM_231					1327
#define IMG_CUSTOM_231_EDITOR				1328
#define IMG_CUSTOM_232					1329
#define IMG_CUSTOM_232_EDITOR				1330
#define IMG_CUSTOM_233					1331
#define IMG_CUSTOM_233_EDITOR				1332
#define IMG_CUSTOM_234					1333
#define IMG_CUSTOM_234_EDITOR				1334
#define IMG_CUSTOM_235					1335
#define IMG_CUSTOM_235_EDITOR				1336
#define IMG_CUSTOM_236					1337
#define IMG_CUSTOM_236_EDITOR				1338
#define IMG_CUSTOM_237					1339
#define IMG_CUSTOM_237_EDITOR				1340
#define IMG_CUSTOM_238					1341
#define IMG_CUSTOM_238_EDITOR				1342
#define IMG_CUSTOM_239					1343
#define IMG_CUSTOM_239_EDITOR				1344
#define IMG_CUSTOM_240					1345
#define IMG_CUSTOM_240_EDITOR				1346
#define IMG_CUSTOM_241					1347
#define IMG_CUSTOM_241_EDITOR				1348
#define IMG_CUSTOM_242					1349
#define IMG_CUSTOM_242_EDITOR				1350
#define IMG_CUSTOM_243					1351
#define IMG_CUSTOM_243_EDITOR				1352
#define IMG_CUSTOM_244					1353
#define IMG_CUSTOM_244_EDITOR				1354
#define IMG_CUSTOM_245					1355
#define IMG_CUSTOM_245_EDITOR				1356
#define IMG_CUSTOM_246					1357
#define IMG_CUSTOM_246_EDITOR				1358
#define IMG_CUSTOM_247					1359
#define IMG_CUSTOM_247_EDITOR				1360
#define IMG_CUSTOM_248					1361
#define IMG_CUSTOM_248_EDITOR				1362
#define IMG_CUSTOM_249					1363
#define IMG_CUSTOM_249_EDITOR				1364
#define IMG_CUSTOM_250					1365
#define IMG_CUSTOM_250_EDITOR				1366
#define IMG_CUSTOM_251					1367
#define IMG_CUSTOM_251_EDITOR				1368
#define IMG_CUSTOM_252					1369
#define IMG_CUSTOM_252_EDITOR				1370
#define IMG_CUSTOM_253					1371
#define IMG_CUSTOM_253_EDITOR				1372
#define IMG_CUSTOM_254					1373
#define IMG_CUSTOM_254_EDITOR				1374
#define IMG_CUSTOM_255					1375
#define IMG_CUSTOM_255_EDITOR				1376
#define IMG_CUSTOM_256					1377
#define IMG_CUSTOM_256_EDITOR				1378
#define IMG_GROUP_1					1379
#define IMG_GROUP_1_EDITOR				1380
#define IMG_GROUP_2					1381
#define IMG_GROUP_2_EDITOR				1382
#define IMG_GROUP_3					1383
#define IMG_GROUP_3_EDITOR				1384
#define IMG_GROUP_4					1385
#define IMG_GROUP_4_EDITOR				1386
#define IMG_GROUP_5					1387
#define IMG_GROUP_5_EDITOR				1388
#define IMG_GROUP_6					1389
#define IMG_GROUP_6_EDITOR				1390
#define IMG_GROUP_7					1391
#define IMG_GROUP_7_EDITOR				1392
#define IMG_GROUP_8					1393
#define IMG_GROUP_8_EDITOR				1394
#define IMG_GROUP_9					1395
#define IMG_GROUP_9_EDITOR				1396
#define IMG_GROUP_10					1397
#define IMG_GROUP_10_EDITOR				1398
#define IMG_GROUP_11					1399
#define IMG_GROUP_11_EDITOR				1400
#define IMG_GROUP_12					1401
#define IMG_GROUP_12_EDITOR				1402
#define IMG_GROUP_13					1403
#define IMG_GROUP_13_EDITOR				1404
#define IMG_GROUP_14					1405
#define IMG_GROUP_14_EDITOR				1406
#define IMG_GROUP_15					1407
#define IMG_GROUP_15_EDITOR				1408
#define IMG_GROUP_16					1409
#define IMG_GROUP_16_EDITOR				1410
#define IMG_GROUP_17					1411
#define IMG_GROUP_17_EDITOR				1412
#define IMG_GROUP_18					1413
#define IMG_GROUP_18_EDITOR				1414
#define IMG_GROUP_19					1415
#define IMG_GROUP_19_EDITOR				1416
#define IMG_GROUP_20					1417
#define IMG_GROUP_20_EDITOR				1418
#define IMG_GROUP_21					1419
#define IMG_GROUP_21_EDITOR				1420
#define IMG_GROUP_22					1421
#define IMG_GROUP_22_EDITOR				1422
#define IMG_GROUP_23					1423
#define IMG_GROUP_23_EDITOR				1424
#define IMG_GROUP_24					1425
#define IMG_GROUP_24_EDITOR				1426
#define IMG_GROUP_25					1427
#define IMG_GROUP_25_EDITOR				1428
#define IMG_GROUP_26					1429
#define IMG_GROUP_26_EDITOR				1430
#define IMG_GROUP_27					1431
#define IMG_GROUP_27_EDITOR				1432
#define IMG_GROUP_28					1433
#define IMG_GROUP_28_EDITOR				1434
#define IMG_GROUP_29					1435
#define IMG_GROUP_29_EDITOR				1436
#define IMG_GROUP_30					1437
#define IMG_GROUP_30_EDITOR				1438
#define IMG_GROUP_31					1439
#define IMG_GROUP_31_EDITOR				1440
#define IMG_GROUP_32					1441
#define IMG_GROUP_32_EDITOR				1442
#define IMG_EMC_OBJECT					1443
#define IMG_EMC_SPRITE					1444
#define IMG_TOON_1					1445
#define IMG_TOON_2					1446
#define IMG_TOON_3					1447
#define IMG_TOON_4					1448
#define IMG_TOON_5					1449
#define IMG_TOON_6					1450
#define IMG_TOON_7					1451
#define IMG_TOON_8					1452
#define IMG_TOON_9					1453
#define IMG_TOON_10					1454
#define IMG_TOON_11					1455
#define IMG_TOON_12					1456
#define IMG_TOON_13					1457
#define IMG_TOON_14					1458
#define IMG_TOON_15					1459
#define IMG_TOON_16					1460
#define IMG_TOON_17					1461
#define IMG_TOON_18					1462
#define IMG_TOON_19					1463
#define IMG_TOON_20					1464
#define IMG_MENU_CALIBRATE_RED				1465
#define IMG_MENU_CALIBRATE_BLUE				1466
#define IMG_MENU_CALIBRATE_YELLOW			1467
#define IMG_MENU_BUTTON					1468
#define IMG_MENU_BUTTON_ACTIVE				1469
#define IMG_MENU_BUTTON_LEFT				1470
#define IMG_MENU_BUTTON_LEFT_ACTIVE			1471
#define IMG_MENU_BUTTON_RIGHT				1472
#define IMG_MENU_BUTTON_RIGHT_ACTIVE			1473
#define IMG_MENU_BUTTON_UP				1474
#define IMG_MENU_BUTTON_UP_ACTIVE			1475
#define IMG_MENU_BUTTON_DOWN				1476
#define IMG_MENU_BUTTON_DOWN_ACTIVE			1477
#define IMG_MENU_BUTTON_ENTER_MENU			1478
#define IMG_MENU_BUTTON_ENTER_MENU_ACTIVE		1479
#define IMG_MENU_BUTTON_LEAVE_MENU			1480
#define IMG_MENU_BUTTON_LEAVE_MENU_ACTIVE		1481
#define IMG_MENU_BUTTON_NEXT_LEVEL			1482
#define IMG_MENU_BUTTON_NEXT_LEVEL_ACTIVE		1483
#define IMG_MENU_BUTTON_PREV_LEVEL			1484
#define IMG_MENU_BUTTON_PREV_LEVEL_ACTIVE		1485
#define IMG_MENU_SCROLLBAR				1486
#define IMG_MENU_SCROLLBAR_ACTIVE			1487
#define IMG_FONT_INITIAL_1				1488
#define IMG_FONT_INITIAL_2				1489
#define IMG_FONT_INITIAL_3				1490
#define IMG_FONT_INITIAL_4				1491
#define IMG_FONT_TITLE_1				1492
#define IMG_FONT_TITLE_2				1493
#define IMG_FONT_MENU_1					1494
#define IMG_FONT_MENU_1_ACTIVE				1495
#define IMG_FONT_MENU_2					1496
#define IMG_FONT_MENU_2_ACTIVE				1497
#define IMG_FONT_TEXT_1					1498
#define IMG_FONT_TEXT_1_LEVELS				1499
#define IMG_FONT_TEXT_1_PREVIEW				1500
#define IMG_FONT_TEXT_1_SCORES				1501
#define IMG_FONT_TEXT_1_ACTIVE_SCORES			1502
#define IMG_FONT_TEXT_2					1503
#define IMG_FONT_TEXT_2_LEVELS				1504
#define IMG_FONT_TEXT_2_PREVIEW				1505
#define IMG_FONT_TEXT_2_SCORES				1506
#define IMG_FONT_TEXT_2_ACTIVE_SCORES			1507
#define IMG_FONT_TEXT_3					1508
#define IMG_FONT_TEXT_3_LEVELS				1509
#define IMG_FONT_TEXT_3_PREVIEW				1510
#define IMG_FONT_TEXT_3_SCORES				1511
#define IMG_FONT_TEXT_3_ACTIVE_SCORES			1512
#define IMG_FONT_TEXT_4					1513
#define IMG_FONT_TEXT_4_LEVELS				1514
#define IMG_FONT_TEXT_4_SCORES				1515
#define IMG_FONT_TEXT_4_ACTIVE_SCORES			1516
#define IMG_FONT_ENVELOPE_1				1517
#define IMG_FONT_ENVELOPE_2				1518
#define IMG_FONT_ENVELOPE_3				1519
#define IMG_FONT_ENVELOPE_4				1520
#define IMG_FONT_INPUT_1				1521
#define IMG_FONT_INPUT_1_MAIN				1522
#define IMG_FONT_INPUT_1_ACTIVE				1523
#define IMG_FONT_INPUT_1_ACTIVE_MAIN			1524
#define IMG_FONT_INPUT_1_ACTIVE_SETUP			1525
#define IMG_FONT_INPUT_2				1526
#define IMG_FONT_INPUT_2_ACTIVE				1527
#define IMG_FONT_OPTION_OFF				1528
#define IMG_FONT_OPTION_ON				1529
#define IMG_FONT_VALUE_1				1530
#define IMG_FONT_VALUE_2				1531
#define IMG_FONT_VALUE_OLD				1532
#define IMG_FONT_LEVEL_NUMBER				1533
#define IMG_FONT_TAPE_RECORDER				1534
#define IMG_FONT_GAME_INFO				1535
#define IMG_GLOBAL_BORDER				1536
#define IMG_GLOBAL_DOOR					1537
#define IMG_EDITOR_ELEMENT_BORDER			1538
#define IMG_EDITOR_ELEMENT_BORDER_INPUT			1539
#define IMG_EDITOR_CASCADE_LIST				1540
#define IMG_EDITOR_CASCADE_LIST_ACTIVE			1541
#define IMG_BACKGROUND_ENVELOPE_1			1542
#define IMG_BACKGROUND_ENVELOPE_2			1543
#define IMG_BACKGROUND_ENVELOPE_3			1544
#define IMG_BACKGROUND_ENVELOPE_4			1545
#define IMG_BACKGROUND					1546
#define IMG_BACKGROUND_TITLE				1547
#define IMG_BACKGROUND_MESSAGE				1548
#define IMG_BACKGROUND_MAIN				1549
#define IMG_BACKGROUND_LEVELS				1550
#define IMG_BACKGROUND_SCORES				1551
#define IMG_BACKGROUND_EDITOR				1552
#define IMG_BACKGROUND_INFO				1553
#define IMG_BACKGROUND_INFO_ELEMENTS			1554
#define IMG_BACKGROUND_INFO_MUSIC			1555
#define IMG_BACKGROUND_INFO_CREDITS			1556
#define IMG_BACKGROUND_INFO_PROGRAM			1557
#define IMG_BACKGROUND_INFO_LEVELSET			1558
#define IMG_BACKGROUND_SETUP				1559
#define IMG_BACKGROUND_DOOR				1560
#define IMG_TITLESCREEN_INITIAL_1			1561
#define IMG_TITLESCREEN_INITIAL_2			1562
#define IMG_TITLESCREEN_INITIAL_3			1563
#define IMG_TITLESCREEN_INITIAL_4			1564
#define IMG_TITLESCREEN_INITIAL_5			1565
#define IMG_TITLESCREEN_1				1566
#define IMG_TITLESCREEN_2				1567
#define IMG_TITLESCREEN_3				1568
#define IMG_TITLESCREEN_4				1569
#define IMG_TITLESCREEN_5				1570

#define NUM_IMAGE_FILES					1571

#endif	/* CONF_GFX_H */
