/***********************************************************
* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
*----------------------------------------------------------*
* (c) 1995-2006 Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* conf_mus.h                                               *
***********************************************************/

/* ----- this file was automatically generated -- do not edit by hand ----- */

#ifndef CONF_MUS_H
#define CONF_MUS_H

/* values for music configuration */

#define MUS_BACKGROUND					0
#define MUS_BACKGROUND_TITLE				1
#define MUS_BACKGROUND_MESSAGE				2
#define MUS_BACKGROUND_MAIN				3
#define MUS_BACKGROUND_LEVELS				4
#define MUS_BACKGROUND_SCORES				5
#define MUS_BACKGROUND_EDITOR				6
#define MUS_BACKGROUND_INFO				7
#define MUS_BACKGROUND_SETUP				8

#define NUM_MUSIC_FILES					9

#endif	/* CONF_MUS_H */
