/***********************************************************
* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
*----------------------------------------------------------*
* (c) 1995-2006 Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* conf_cus.c                                               *
***********************************************************/

/* ----- this file was automatically generated -- do not edit by hand ----- */

#ifndef CONF_CUS_C
#define CONF_CUS_C

/* values for graphics configuration (custom elements) */

  { "custom_1",					"RocksElements.pcx"	},
  { "custom_1.xpos",				"7"			},
  { "custom_1.ypos",				"9"			},
  { "custom_1.frames",				"1"			},
  { "custom_1.EDITOR",				"RocksElements.pcx"	},
  { "custom_1.EDITOR.xpos",			"15"			},
  { "custom_1.EDITOR.ypos",			"13"			},

  { "custom_2",					"RocksElements.pcx"	},
  { "custom_2.xpos",				"7"			},
  { "custom_2.ypos",				"9"			},
  { "custom_2.frames",				"1"			},
  { "custom_2.EDITOR",				"RocksElements.pcx"	},
  { "custom_2.EDITOR.xpos",			"15"			},
  { "custom_2.EDITOR.ypos",			"13"			},

  { "custom_3",					"RocksElements.pcx"	},
  { "custom_3.xpos",				"7"			},
  { "custom_3.ypos",				"9"			},
  { "custom_3.frames",				"1"			},
  { "custom_3.EDITOR",				"RocksElements.pcx"	},
  { "custom_3.EDITOR.xpos",			"15"			},
  { "custom_3.EDITOR.ypos",			"13"			},

  { "custom_4",					"RocksElements.pcx"	},
  { "custom_4.xpos",				"7"			},
  { "custom_4.ypos",				"9"			},
  { "custom_4.frames",				"1"			},
  { "custom_4.EDITOR",				"RocksElements.pcx"	},
  { "custom_4.EDITOR.xpos",			"15"			},
  { "custom_4.EDITOR.ypos",			"13"			},

  { "custom_5",					"RocksElements.pcx"	},
  { "custom_5.xpos",				"7"			},
  { "custom_5.ypos",				"9"			},
  { "custom_5.frames",				"1"			},
  { "custom_5.EDITOR",				"RocksElements.pcx"	},
  { "custom_5.EDITOR.xpos",			"15"			},
  { "custom_5.EDITOR.ypos",			"13"			},

  { "custom_6",					"RocksElements.pcx"	},
  { "custom_6.xpos",				"7"			},
  { "custom_6.ypos",				"9"			},
  { "custom_6.frames",				"1"			},
  { "custom_6.EDITOR",				"RocksElements.pcx"	},
  { "custom_6.EDITOR.xpos",			"15"			},
  { "custom_6.EDITOR.ypos",			"13"			},

  { "custom_7",					"RocksElements.pcx"	},
  { "custom_7.xpos",				"7"			},
  { "custom_7.ypos",				"9"			},
  { "custom_7.frames",				"1"			},
  { "custom_7.EDITOR",				"RocksElements.pcx"	},
  { "custom_7.EDITOR.xpos",			"15"			},
  { "custom_7.EDITOR.ypos",			"13"			},

  { "custom_8",					"RocksElements.pcx"	},
  { "custom_8.xpos",				"7"			},
  { "custom_8.ypos",				"9"			},
  { "custom_8.frames",				"1"			},
  { "custom_8.EDITOR",				"RocksElements.pcx"	},
  { "custom_8.EDITOR.xpos",			"15"			},
  { "custom_8.EDITOR.ypos",			"13"			},

  { "custom_9",					"RocksElements.pcx"	},
  { "custom_9.xpos",				"7"			},
  { "custom_9.ypos",				"9"			},
  { "custom_9.frames",				"1"			},
  { "custom_9.EDITOR",				"RocksElements.pcx"	},
  { "custom_9.EDITOR.xpos",			"15"			},
  { "custom_9.EDITOR.ypos",			"13"			},

  { "custom_10",				"RocksElements.pcx"	},
  { "custom_10.xpos",				"7"			},
  { "custom_10.ypos",				"9"			},
  { "custom_10.frames",				"1"			},
  { "custom_10.EDITOR",				"RocksElements.pcx"	},
  { "custom_10.EDITOR.xpos",			"15"			},
  { "custom_10.EDITOR.ypos",			"13"			},

  { "custom_11",				"RocksElements.pcx"	},
  { "custom_11.xpos",				"7"			},
  { "custom_11.ypos",				"9"			},
  { "custom_11.frames",				"1"			},
  { "custom_11.EDITOR",				"RocksElements.pcx"	},
  { "custom_11.EDITOR.xpos",			"15"			},
  { "custom_11.EDITOR.ypos",			"13"			},

  { "custom_12",				"RocksElements.pcx"	},
  { "custom_12.xpos",				"7"			},
  { "custom_12.ypos",				"9"			},
  { "custom_12.frames",				"1"			},
  { "custom_12.EDITOR",				"RocksElements.pcx"	},
  { "custom_12.EDITOR.xpos",			"15"			},
  { "custom_12.EDITOR.ypos",			"13"			},

  { "custom_13",				"RocksElements.pcx"	},
  { "custom_13.xpos",				"7"			},
  { "custom_13.ypos",				"9"			},
  { "custom_13.frames",				"1"			},
  { "custom_13.EDITOR",				"RocksElements.pcx"	},
  { "custom_13.EDITOR.xpos",			"15"			},
  { "custom_13.EDITOR.ypos",			"13"			},

  { "custom_14",				"RocksElements.pcx"	},
  { "custom_14.xpos",				"7"			},
  { "custom_14.ypos",				"9"			},
  { "custom_14.frames",				"1"			},
  { "custom_14.EDITOR",				"RocksElements.pcx"	},
  { "custom_14.EDITOR.xpos",			"15"			},
  { "custom_14.EDITOR.ypos",			"13"			},

  { "custom_15",				"RocksElements.pcx"	},
  { "custom_15.xpos",				"7"			},
  { "custom_15.ypos",				"9"			},
  { "custom_15.frames",				"1"			},
  { "custom_15.EDITOR",				"RocksElements.pcx"	},
  { "custom_15.EDITOR.xpos",			"15"			},
  { "custom_15.EDITOR.ypos",			"13"			},

  { "custom_16",				"RocksElements.pcx"	},
  { "custom_16.xpos",				"7"			},
  { "custom_16.ypos",				"9"			},
  { "custom_16.frames",				"1"			},
  { "custom_16.EDITOR",				"RocksElements.pcx"	},
  { "custom_16.EDITOR.xpos",			"15"			},
  { "custom_16.EDITOR.ypos",			"13"			},

  { "custom_17",				"RocksElements.pcx"	},
  { "custom_17.xpos",				"7"			},
  { "custom_17.ypos",				"9"			},
  { "custom_17.frames",				"1"			},
  { "custom_17.EDITOR",				"RocksElements.pcx"	},
  { "custom_17.EDITOR.xpos",			"15"			},
  { "custom_17.EDITOR.ypos",			"13"			},

  { "custom_18",				"RocksElements.pcx"	},
  { "custom_18.xpos",				"7"			},
  { "custom_18.ypos",				"9"			},
  { "custom_18.frames",				"1"			},
  { "custom_18.EDITOR",				"RocksElements.pcx"	},
  { "custom_18.EDITOR.xpos",			"15"			},
  { "custom_18.EDITOR.ypos",			"13"			},

  { "custom_19",				"RocksElements.pcx"	},
  { "custom_19.xpos",				"7"			},
  { "custom_19.ypos",				"9"			},
  { "custom_19.frames",				"1"			},
  { "custom_19.EDITOR",				"RocksElements.pcx"	},
  { "custom_19.EDITOR.xpos",			"15"			},
  { "custom_19.EDITOR.ypos",			"13"			},

  { "custom_20",				"RocksElements.pcx"	},
  { "custom_20.xpos",				"7"			},
  { "custom_20.ypos",				"9"			},
  { "custom_20.frames",				"1"			},
  { "custom_20.EDITOR",				"RocksElements.pcx"	},
  { "custom_20.EDITOR.xpos",			"15"			},
  { "custom_20.EDITOR.ypos",			"13"			},

  { "custom_21",				"RocksElements.pcx"	},
  { "custom_21.xpos",				"7"			},
  { "custom_21.ypos",				"9"			},
  { "custom_21.frames",				"1"			},
  { "custom_21.EDITOR",				"RocksElements.pcx"	},
  { "custom_21.EDITOR.xpos",			"15"			},
  { "custom_21.EDITOR.ypos",			"13"			},

  { "custom_22",				"RocksElements.pcx"	},
  { "custom_22.xpos",				"7"			},
  { "custom_22.ypos",				"9"			},
  { "custom_22.frames",				"1"			},
  { "custom_22.EDITOR",				"RocksElements.pcx"	},
  { "custom_22.EDITOR.xpos",			"15"			},
  { "custom_22.EDITOR.ypos",			"13"			},

  { "custom_23",				"RocksElements.pcx"	},
  { "custom_23.xpos",				"7"			},
  { "custom_23.ypos",				"9"			},
  { "custom_23.frames",				"1"			},
  { "custom_23.EDITOR",				"RocksElements.pcx"	},
  { "custom_23.EDITOR.xpos",			"15"			},
  { "custom_23.EDITOR.ypos",			"13"			},

  { "custom_24",				"RocksElements.pcx"	},
  { "custom_24.xpos",				"7"			},
  { "custom_24.ypos",				"9"			},
  { "custom_24.frames",				"1"			},
  { "custom_24.EDITOR",				"RocksElements.pcx"	},
  { "custom_24.EDITOR.xpos",			"15"			},
  { "custom_24.EDITOR.ypos",			"13"			},

  { "custom_25",				"RocksElements.pcx"	},
  { "custom_25.xpos",				"7"			},
  { "custom_25.ypos",				"9"			},
  { "custom_25.frames",				"1"			},
  { "custom_25.EDITOR",				"RocksElements.pcx"	},
  { "custom_25.EDITOR.xpos",			"15"			},
  { "custom_25.EDITOR.ypos",			"13"			},

  { "custom_26",				"RocksElements.pcx"	},
  { "custom_26.xpos",				"7"			},
  { "custom_26.ypos",				"9"			},
  { "custom_26.frames",				"1"			},
  { "custom_26.EDITOR",				"RocksElements.pcx"	},
  { "custom_26.EDITOR.xpos",			"15"			},
  { "custom_26.EDITOR.ypos",			"13"			},

  { "custom_27",				"RocksElements.pcx"	},
  { "custom_27.xpos",				"7"			},
  { "custom_27.ypos",				"9"			},
  { "custom_27.frames",				"1"			},
  { "custom_27.EDITOR",				"RocksElements.pcx"	},
  { "custom_27.EDITOR.xpos",			"15"			},
  { "custom_27.EDITOR.ypos",			"13"			},

  { "custom_28",				"RocksElements.pcx"	},
  { "custom_28.xpos",				"7"			},
  { "custom_28.ypos",				"9"			},
  { "custom_28.frames",				"1"			},
  { "custom_28.EDITOR",				"RocksElements.pcx"	},
  { "custom_28.EDITOR.xpos",			"15"			},
  { "custom_28.EDITOR.ypos",			"13"			},

  { "custom_29",				"RocksElements.pcx"	},
  { "custom_29.xpos",				"7"			},
  { "custom_29.ypos",				"9"			},
  { "custom_29.frames",				"1"			},
  { "custom_29.EDITOR",				"RocksElements.pcx"	},
  { "custom_29.EDITOR.xpos",			"15"			},
  { "custom_29.EDITOR.ypos",			"13"			},

  { "custom_30",				"RocksElements.pcx"	},
  { "custom_30.xpos",				"7"			},
  { "custom_30.ypos",				"9"			},
  { "custom_30.frames",				"1"			},
  { "custom_30.EDITOR",				"RocksElements.pcx"	},
  { "custom_30.EDITOR.xpos",			"15"			},
  { "custom_30.EDITOR.ypos",			"13"			},

  { "custom_31",				"RocksElements.pcx"	},
  { "custom_31.xpos",				"7"			},
  { "custom_31.ypos",				"9"			},
  { "custom_31.frames",				"1"			},
  { "custom_31.EDITOR",				"RocksElements.pcx"	},
  { "custom_31.EDITOR.xpos",			"15"			},
  { "custom_31.EDITOR.ypos",			"13"			},

  { "custom_32",				"RocksElements.pcx"	},
  { "custom_32.xpos",				"7"			},
  { "custom_32.ypos",				"9"			},
  { "custom_32.frames",				"1"			},
  { "custom_32.EDITOR",				"RocksElements.pcx"	},
  { "custom_32.EDITOR.xpos",			"15"			},
  { "custom_32.EDITOR.ypos",			"13"			},

  { "custom_33",				"RocksElements.pcx"	},
  { "custom_33.xpos",				"7"			},
  { "custom_33.ypos",				"9"			},
  { "custom_33.frames",				"1"			},
  { "custom_33.EDITOR",				"RocksElements.pcx"	},
  { "custom_33.EDITOR.xpos",			"15"			},
  { "custom_33.EDITOR.ypos",			"13"			},

  { "custom_34",				"RocksElements.pcx"	},
  { "custom_34.xpos",				"7"			},
  { "custom_34.ypos",				"9"			},
  { "custom_34.frames",				"1"			},
  { "custom_34.EDITOR",				"RocksElements.pcx"	},
  { "custom_34.EDITOR.xpos",			"15"			},
  { "custom_34.EDITOR.ypos",			"13"			},

  { "custom_35",				"RocksElements.pcx"	},
  { "custom_35.xpos",				"7"			},
  { "custom_35.ypos",				"9"			},
  { "custom_35.frames",				"1"			},
  { "custom_35.EDITOR",				"RocksElements.pcx"	},
  { "custom_35.EDITOR.xpos",			"15"			},
  { "custom_35.EDITOR.ypos",			"13"			},

  { "custom_36",				"RocksElements.pcx"	},
  { "custom_36.xpos",				"7"			},
  { "custom_36.ypos",				"9"			},
  { "custom_36.frames",				"1"			},
  { "custom_36.EDITOR",				"RocksElements.pcx"	},
  { "custom_36.EDITOR.xpos",			"15"			},
  { "custom_36.EDITOR.ypos",			"13"			},

  { "custom_37",				"RocksElements.pcx"	},
  { "custom_37.xpos",				"7"			},
  { "custom_37.ypos",				"9"			},
  { "custom_37.frames",				"1"			},
  { "custom_37.EDITOR",				"RocksElements.pcx"	},
  { "custom_37.EDITOR.xpos",			"15"			},
  { "custom_37.EDITOR.ypos",			"13"			},

  { "custom_38",				"RocksElements.pcx"	},
  { "custom_38.xpos",				"7"			},
  { "custom_38.ypos",				"9"			},
  { "custom_38.frames",				"1"			},
  { "custom_38.EDITOR",				"RocksElements.pcx"	},
  { "custom_38.EDITOR.xpos",			"15"			},
  { "custom_38.EDITOR.ypos",			"13"			},

  { "custom_39",				"RocksElements.pcx"	},
  { "custom_39.xpos",				"7"			},
  { "custom_39.ypos",				"9"			},
  { "custom_39.frames",				"1"			},
  { "custom_39.EDITOR",				"RocksElements.pcx"	},
  { "custom_39.EDITOR.xpos",			"15"			},
  { "custom_39.EDITOR.ypos",			"13"			},

  { "custom_40",				"RocksElements.pcx"	},
  { "custom_40.xpos",				"7"			},
  { "custom_40.ypos",				"9"			},
  { "custom_40.frames",				"1"			},
  { "custom_40.EDITOR",				"RocksElements.pcx"	},
  { "custom_40.EDITOR.xpos",			"15"			},
  { "custom_40.EDITOR.ypos",			"13"			},

  { "custom_41",				"RocksElements.pcx"	},
  { "custom_41.xpos",				"7"			},
  { "custom_41.ypos",				"9"			},
  { "custom_41.frames",				"1"			},
  { "custom_41.EDITOR",				"RocksElements.pcx"	},
  { "custom_41.EDITOR.xpos",			"15"			},
  { "custom_41.EDITOR.ypos",			"13"			},

  { "custom_42",				"RocksElements.pcx"	},
  { "custom_42.xpos",				"7"			},
  { "custom_42.ypos",				"9"			},
  { "custom_42.frames",				"1"			},
  { "custom_42.EDITOR",				"RocksElements.pcx"	},
  { "custom_42.EDITOR.xpos",			"15"			},
  { "custom_42.EDITOR.ypos",			"13"			},

  { "custom_43",				"RocksElements.pcx"	},
  { "custom_43.xpos",				"7"			},
  { "custom_43.ypos",				"9"			},
  { "custom_43.frames",				"1"			},
  { "custom_43.EDITOR",				"RocksElements.pcx"	},
  { "custom_43.EDITOR.xpos",			"15"			},
  { "custom_43.EDITOR.ypos",			"13"			},

  { "custom_44",				"RocksElements.pcx"	},
  { "custom_44.xpos",				"7"			},
  { "custom_44.ypos",				"9"			},
  { "custom_44.frames",				"1"			},
  { "custom_44.EDITOR",				"RocksElements.pcx"	},
  { "custom_44.EDITOR.xpos",			"15"			},
  { "custom_44.EDITOR.ypos",			"13"			},

  { "custom_45",				"RocksElements.pcx"	},
  { "custom_45.xpos",				"7"			},
  { "custom_45.ypos",				"9"			},
  { "custom_45.frames",				"1"			},
  { "custom_45.EDITOR",				"RocksElements.pcx"	},
  { "custom_45.EDITOR.xpos",			"15"			},
  { "custom_45.EDITOR.ypos",			"13"			},

  { "custom_46",				"RocksElements.pcx"	},
  { "custom_46.xpos",				"7"			},
  { "custom_46.ypos",				"9"			},
  { "custom_46.frames",				"1"			},
  { "custom_46.EDITOR",				"RocksElements.pcx"	},
  { "custom_46.EDITOR.xpos",			"15"			},
  { "custom_46.EDITOR.ypos",			"13"			},

  { "custom_47",				"RocksElements.pcx"	},
  { "custom_47.xpos",				"7"			},
  { "custom_47.ypos",				"9"			},
  { "custom_47.frames",				"1"			},
  { "custom_47.EDITOR",				"RocksElements.pcx"	},
  { "custom_47.EDITOR.xpos",			"15"			},
  { "custom_47.EDITOR.ypos",			"13"			},

  { "custom_48",				"RocksElements.pcx"	},
  { "custom_48.xpos",				"7"			},
  { "custom_48.ypos",				"9"			},
  { "custom_48.frames",				"1"			},
  { "custom_48.EDITOR",				"RocksElements.pcx"	},
  { "custom_48.EDITOR.xpos",			"15"			},
  { "custom_48.EDITOR.ypos",			"13"			},

  { "custom_49",				"RocksElements.pcx"	},
  { "custom_49.xpos",				"7"			},
  { "custom_49.ypos",				"9"			},
  { "custom_49.frames",				"1"			},
  { "custom_49.EDITOR",				"RocksElements.pcx"	},
  { "custom_49.EDITOR.xpos",			"15"			},
  { "custom_49.EDITOR.ypos",			"13"			},

  { "custom_50",				"RocksElements.pcx"	},
  { "custom_50.xpos",				"7"			},
  { "custom_50.ypos",				"9"			},
  { "custom_50.frames",				"1"			},
  { "custom_50.EDITOR",				"RocksElements.pcx"	},
  { "custom_50.EDITOR.xpos",			"15"			},
  { "custom_50.EDITOR.ypos",			"13"			},

  { "custom_51",				"RocksElements.pcx"	},
  { "custom_51.xpos",				"7"			},
  { "custom_51.ypos",				"9"			},
  { "custom_51.frames",				"1"			},
  { "custom_51.EDITOR",				"RocksElements.pcx"	},
  { "custom_51.EDITOR.xpos",			"15"			},
  { "custom_51.EDITOR.ypos",			"13"			},

  { "custom_52",				"RocksElements.pcx"	},
  { "custom_52.xpos",				"7"			},
  { "custom_52.ypos",				"9"			},
  { "custom_52.frames",				"1"			},
  { "custom_52.EDITOR",				"RocksElements.pcx"	},
  { "custom_52.EDITOR.xpos",			"15"			},
  { "custom_52.EDITOR.ypos",			"13"			},

  { "custom_53",				"RocksElements.pcx"	},
  { "custom_53.xpos",				"7"			},
  { "custom_53.ypos",				"9"			},
  { "custom_53.frames",				"1"			},
  { "custom_53.EDITOR",				"RocksElements.pcx"	},
  { "custom_53.EDITOR.xpos",			"15"			},
  { "custom_53.EDITOR.ypos",			"13"			},

  { "custom_54",				"RocksElements.pcx"	},
  { "custom_54.xpos",				"7"			},
  { "custom_54.ypos",				"9"			},
  { "custom_54.frames",				"1"			},
  { "custom_54.EDITOR",				"RocksElements.pcx"	},
  { "custom_54.EDITOR.xpos",			"15"			},
  { "custom_54.EDITOR.ypos",			"13"			},

  { "custom_55",				"RocksElements.pcx"	},
  { "custom_55.xpos",				"7"			},
  { "custom_55.ypos",				"9"			},
  { "custom_55.frames",				"1"			},
  { "custom_55.EDITOR",				"RocksElements.pcx"	},
  { "custom_55.EDITOR.xpos",			"15"			},
  { "custom_55.EDITOR.ypos",			"13"			},

  { "custom_56",				"RocksElements.pcx"	},
  { "custom_56.xpos",				"7"			},
  { "custom_56.ypos",				"9"			},
  { "custom_56.frames",				"1"			},
  { "custom_56.EDITOR",				"RocksElements.pcx"	},
  { "custom_56.EDITOR.xpos",			"15"			},
  { "custom_56.EDITOR.ypos",			"13"			},

  { "custom_57",				"RocksElements.pcx"	},
  { "custom_57.xpos",				"7"			},
  { "custom_57.ypos",				"9"			},
  { "custom_57.frames",				"1"			},
  { "custom_57.EDITOR",				"RocksElements.pcx"	},
  { "custom_57.EDITOR.xpos",			"15"			},
  { "custom_57.EDITOR.ypos",			"13"			},

  { "custom_58",				"RocksElements.pcx"	},
  { "custom_58.xpos",				"7"			},
  { "custom_58.ypos",				"9"			},
  { "custom_58.frames",				"1"			},
  { "custom_58.EDITOR",				"RocksElements.pcx"	},
  { "custom_58.EDITOR.xpos",			"15"			},
  { "custom_58.EDITOR.ypos",			"13"			},

  { "custom_59",				"RocksElements.pcx"	},
  { "custom_59.xpos",				"7"			},
  { "custom_59.ypos",				"9"			},
  { "custom_59.frames",				"1"			},
  { "custom_59.EDITOR",				"RocksElements.pcx"	},
  { "custom_59.EDITOR.xpos",			"15"			},
  { "custom_59.EDITOR.ypos",			"13"			},

  { "custom_60",				"RocksElements.pcx"	},
  { "custom_60.xpos",				"7"			},
  { "custom_60.ypos",				"9"			},
  { "custom_60.frames",				"1"			},
  { "custom_60.EDITOR",				"RocksElements.pcx"	},
  { "custom_60.EDITOR.xpos",			"15"			},
  { "custom_60.EDITOR.ypos",			"13"			},

  { "custom_61",				"RocksElements.pcx"	},
  { "custom_61.xpos",				"7"			},
  { "custom_61.ypos",				"9"			},
  { "custom_61.frames",				"1"			},
  { "custom_61.EDITOR",				"RocksElements.pcx"	},
  { "custom_61.EDITOR.xpos",			"15"			},
  { "custom_61.EDITOR.ypos",			"13"			},

  { "custom_62",				"RocksElements.pcx"	},
  { "custom_62.xpos",				"7"			},
  { "custom_62.ypos",				"9"			},
  { "custom_62.frames",				"1"			},
  { "custom_62.EDITOR",				"RocksElements.pcx"	},
  { "custom_62.EDITOR.xpos",			"15"			},
  { "custom_62.EDITOR.ypos",			"13"			},

  { "custom_63",				"RocksElements.pcx"	},
  { "custom_63.xpos",				"7"			},
  { "custom_63.ypos",				"9"			},
  { "custom_63.frames",				"1"			},
  { "custom_63.EDITOR",				"RocksElements.pcx"	},
  { "custom_63.EDITOR.xpos",			"15"			},
  { "custom_63.EDITOR.ypos",			"13"			},

  { "custom_64",				"RocksElements.pcx"	},
  { "custom_64.xpos",				"7"			},
  { "custom_64.ypos",				"9"			},
  { "custom_64.frames",				"1"			},
  { "custom_64.EDITOR",				"RocksElements.pcx"	},
  { "custom_64.EDITOR.xpos",			"15"			},
  { "custom_64.EDITOR.ypos",			"13"			},

  { "custom_65",				"RocksElements.pcx"	},
  { "custom_65.xpos",				"7"			},
  { "custom_65.ypos",				"9"			},
  { "custom_65.frames",				"1"			},
  { "custom_65.EDITOR",				"RocksElements.pcx"	},
  { "custom_65.EDITOR.xpos",			"15"			},
  { "custom_65.EDITOR.ypos",			"13"			},

  { "custom_66",				"RocksElements.pcx"	},
  { "custom_66.xpos",				"7"			},
  { "custom_66.ypos",				"9"			},
  { "custom_66.frames",				"1"			},
  { "custom_66.EDITOR",				"RocksElements.pcx"	},
  { "custom_66.EDITOR.xpos",			"15"			},
  { "custom_66.EDITOR.ypos",			"13"			},

  { "custom_67",				"RocksElements.pcx"	},
  { "custom_67.xpos",				"7"			},
  { "custom_67.ypos",				"9"			},
  { "custom_67.frames",				"1"			},
  { "custom_67.EDITOR",				"RocksElements.pcx"	},
  { "custom_67.EDITOR.xpos",			"15"			},
  { "custom_67.EDITOR.ypos",			"13"			},

  { "custom_68",				"RocksElements.pcx"	},
  { "custom_68.xpos",				"7"			},
  { "custom_68.ypos",				"9"			},
  { "custom_68.frames",				"1"			},
  { "custom_68.EDITOR",				"RocksElements.pcx"	},
  { "custom_68.EDITOR.xpos",			"15"			},
  { "custom_68.EDITOR.ypos",			"13"			},

  { "custom_69",				"RocksElements.pcx"	},
  { "custom_69.xpos",				"7"			},
  { "custom_69.ypos",				"9"			},
  { "custom_69.frames",				"1"			},
  { "custom_69.EDITOR",				"RocksElements.pcx"	},
  { "custom_69.EDITOR.xpos",			"15"			},
  { "custom_69.EDITOR.ypos",			"13"			},

  { "custom_70",				"RocksElements.pcx"	},
  { "custom_70.xpos",				"7"			},
  { "custom_70.ypos",				"9"			},
  { "custom_70.frames",				"1"			},
  { "custom_70.EDITOR",				"RocksElements.pcx"	},
  { "custom_70.EDITOR.xpos",			"15"			},
  { "custom_70.EDITOR.ypos",			"13"			},

  { "custom_71",				"RocksElements.pcx"	},
  { "custom_71.xpos",				"7"			},
  { "custom_71.ypos",				"9"			},
  { "custom_71.frames",				"1"			},
  { "custom_71.EDITOR",				"RocksElements.pcx"	},
  { "custom_71.EDITOR.xpos",			"15"			},
  { "custom_71.EDITOR.ypos",			"13"			},

  { "custom_72",				"RocksElements.pcx"	},
  { "custom_72.xpos",				"7"			},
  { "custom_72.ypos",				"9"			},
  { "custom_72.frames",				"1"			},
  { "custom_72.EDITOR",				"RocksElements.pcx"	},
  { "custom_72.EDITOR.xpos",			"15"			},
  { "custom_72.EDITOR.ypos",			"13"			},

  { "custom_73",				"RocksElements.pcx"	},
  { "custom_73.xpos",				"7"			},
  { "custom_73.ypos",				"9"			},
  { "custom_73.frames",				"1"			},
  { "custom_73.EDITOR",				"RocksElements.pcx"	},
  { "custom_73.EDITOR.xpos",			"15"			},
  { "custom_73.EDITOR.ypos",			"13"			},

  { "custom_74",				"RocksElements.pcx"	},
  { "custom_74.xpos",				"7"			},
  { "custom_74.ypos",				"9"			},
  { "custom_74.frames",				"1"			},
  { "custom_74.EDITOR",				"RocksElements.pcx"	},
  { "custom_74.EDITOR.xpos",			"15"			},
  { "custom_74.EDITOR.ypos",			"13"			},

  { "custom_75",				"RocksElements.pcx"	},
  { "custom_75.xpos",				"7"			},
  { "custom_75.ypos",				"9"			},
  { "custom_75.frames",				"1"			},
  { "custom_75.EDITOR",				"RocksElements.pcx"	},
  { "custom_75.EDITOR.xpos",			"15"			},
  { "custom_75.EDITOR.ypos",			"13"			},

  { "custom_76",				"RocksElements.pcx"	},
  { "custom_76.xpos",				"7"			},
  { "custom_76.ypos",				"9"			},
  { "custom_76.frames",				"1"			},
  { "custom_76.EDITOR",				"RocksElements.pcx"	},
  { "custom_76.EDITOR.xpos",			"15"			},
  { "custom_76.EDITOR.ypos",			"13"			},

  { "custom_77",				"RocksElements.pcx"	},
  { "custom_77.xpos",				"7"			},
  { "custom_77.ypos",				"9"			},
  { "custom_77.frames",				"1"			},
  { "custom_77.EDITOR",				"RocksElements.pcx"	},
  { "custom_77.EDITOR.xpos",			"15"			},
  { "custom_77.EDITOR.ypos",			"13"			},

  { "custom_78",				"RocksElements.pcx"	},
  { "custom_78.xpos",				"7"			},
  { "custom_78.ypos",				"9"			},
  { "custom_78.frames",				"1"			},
  { "custom_78.EDITOR",				"RocksElements.pcx"	},
  { "custom_78.EDITOR.xpos",			"15"			},
  { "custom_78.EDITOR.ypos",			"13"			},

  { "custom_79",				"RocksElements.pcx"	},
  { "custom_79.xpos",				"7"			},
  { "custom_79.ypos",				"9"			},
  { "custom_79.frames",				"1"			},
  { "custom_79.EDITOR",				"RocksElements.pcx"	},
  { "custom_79.EDITOR.xpos",			"15"			},
  { "custom_79.EDITOR.ypos",			"13"			},

  { "custom_80",				"RocksElements.pcx"	},
  { "custom_80.xpos",				"7"			},
  { "custom_80.ypos",				"9"			},
  { "custom_80.frames",				"1"			},
  { "custom_80.EDITOR",				"RocksElements.pcx"	},
  { "custom_80.EDITOR.xpos",			"15"			},
  { "custom_80.EDITOR.ypos",			"13"			},

  { "custom_81",				"RocksElements.pcx"	},
  { "custom_81.xpos",				"7"			},
  { "custom_81.ypos",				"9"			},
  { "custom_81.frames",				"1"			},
  { "custom_81.EDITOR",				"RocksElements.pcx"	},
  { "custom_81.EDITOR.xpos",			"15"			},
  { "custom_81.EDITOR.ypos",			"13"			},

  { "custom_82",				"RocksElements.pcx"	},
  { "custom_82.xpos",				"7"			},
  { "custom_82.ypos",				"9"			},
  { "custom_82.frames",				"1"			},
  { "custom_82.EDITOR",				"RocksElements.pcx"	},
  { "custom_82.EDITOR.xpos",			"15"			},
  { "custom_82.EDITOR.ypos",			"13"			},

  { "custom_83",				"RocksElements.pcx"	},
  { "custom_83.xpos",				"7"			},
  { "custom_83.ypos",				"9"			},
  { "custom_83.frames",				"1"			},
  { "custom_83.EDITOR",				"RocksElements.pcx"	},
  { "custom_83.EDITOR.xpos",			"15"			},
  { "custom_83.EDITOR.ypos",			"13"			},

  { "custom_84",				"RocksElements.pcx"	},
  { "custom_84.xpos",				"7"			},
  { "custom_84.ypos",				"9"			},
  { "custom_84.frames",				"1"			},
  { "custom_84.EDITOR",				"RocksElements.pcx"	},
  { "custom_84.EDITOR.xpos",			"15"			},
  { "custom_84.EDITOR.ypos",			"13"			},

  { "custom_85",				"RocksElements.pcx"	},
  { "custom_85.xpos",				"7"			},
  { "custom_85.ypos",				"9"			},
  { "custom_85.frames",				"1"			},
  { "custom_85.EDITOR",				"RocksElements.pcx"	},
  { "custom_85.EDITOR.xpos",			"15"			},
  { "custom_85.EDITOR.ypos",			"13"			},

  { "custom_86",				"RocksElements.pcx"	},
  { "custom_86.xpos",				"7"			},
  { "custom_86.ypos",				"9"			},
  { "custom_86.frames",				"1"			},
  { "custom_86.EDITOR",				"RocksElements.pcx"	},
  { "custom_86.EDITOR.xpos",			"15"			},
  { "custom_86.EDITOR.ypos",			"13"			},

  { "custom_87",				"RocksElements.pcx"	},
  { "custom_87.xpos",				"7"			},
  { "custom_87.ypos",				"9"			},
  { "custom_87.frames",				"1"			},
  { "custom_87.EDITOR",				"RocksElements.pcx"	},
  { "custom_87.EDITOR.xpos",			"15"			},
  { "custom_87.EDITOR.ypos",			"13"			},

  { "custom_88",				"RocksElements.pcx"	},
  { "custom_88.xpos",				"7"			},
  { "custom_88.ypos",				"9"			},
  { "custom_88.frames",				"1"			},
  { "custom_88.EDITOR",				"RocksElements.pcx"	},
  { "custom_88.EDITOR.xpos",			"15"			},
  { "custom_88.EDITOR.ypos",			"13"			},

  { "custom_89",				"RocksElements.pcx"	},
  { "custom_89.xpos",				"7"			},
  { "custom_89.ypos",				"9"			},
  { "custom_89.frames",				"1"			},
  { "custom_89.EDITOR",				"RocksElements.pcx"	},
  { "custom_89.EDITOR.xpos",			"15"			},
  { "custom_89.EDITOR.ypos",			"13"			},

  { "custom_90",				"RocksElements.pcx"	},
  { "custom_90.xpos",				"7"			},
  { "custom_90.ypos",				"9"			},
  { "custom_90.frames",				"1"			},
  { "custom_90.EDITOR",				"RocksElements.pcx"	},
  { "custom_90.EDITOR.xpos",			"15"			},
  { "custom_90.EDITOR.ypos",			"13"			},

  { "custom_91",				"RocksElements.pcx"	},
  { "custom_91.xpos",				"7"			},
  { "custom_91.ypos",				"9"			},
  { "custom_91.frames",				"1"			},
  { "custom_91.EDITOR",				"RocksElements.pcx"	},
  { "custom_91.EDITOR.xpos",			"15"			},
  { "custom_91.EDITOR.ypos",			"13"			},

  { "custom_92",				"RocksElements.pcx"	},
  { "custom_92.xpos",				"7"			},
  { "custom_92.ypos",				"9"			},
  { "custom_92.frames",				"1"			},
  { "custom_92.EDITOR",				"RocksElements.pcx"	},
  { "custom_92.EDITOR.xpos",			"15"			},
  { "custom_92.EDITOR.ypos",			"13"			},

  { "custom_93",				"RocksElements.pcx"	},
  { "custom_93.xpos",				"7"			},
  { "custom_93.ypos",				"9"			},
  { "custom_93.frames",				"1"			},
  { "custom_93.EDITOR",				"RocksElements.pcx"	},
  { "custom_93.EDITOR.xpos",			"15"			},
  { "custom_93.EDITOR.ypos",			"13"			},

  { "custom_94",				"RocksElements.pcx"	},
  { "custom_94.xpos",				"7"			},
  { "custom_94.ypos",				"9"			},
  { "custom_94.frames",				"1"			},
  { "custom_94.EDITOR",				"RocksElements.pcx"	},
  { "custom_94.EDITOR.xpos",			"15"			},
  { "custom_94.EDITOR.ypos",			"13"			},

  { "custom_95",				"RocksElements.pcx"	},
  { "custom_95.xpos",				"7"			},
  { "custom_95.ypos",				"9"			},
  { "custom_95.frames",				"1"			},
  { "custom_95.EDITOR",				"RocksElements.pcx"	},
  { "custom_95.EDITOR.xpos",			"15"			},
  { "custom_95.EDITOR.ypos",			"13"			},

  { "custom_96",				"RocksElements.pcx"	},
  { "custom_96.xpos",				"7"			},
  { "custom_96.ypos",				"9"			},
  { "custom_96.frames",				"1"			},
  { "custom_96.EDITOR",				"RocksElements.pcx"	},
  { "custom_96.EDITOR.xpos",			"15"			},
  { "custom_96.EDITOR.ypos",			"13"			},

  { "custom_97",				"RocksElements.pcx"	},
  { "custom_97.xpos",				"7"			},
  { "custom_97.ypos",				"9"			},
  { "custom_97.frames",				"1"			},
  { "custom_97.EDITOR",				"RocksElements.pcx"	},
  { "custom_97.EDITOR.xpos",			"15"			},
  { "custom_97.EDITOR.ypos",			"13"			},

  { "custom_98",				"RocksElements.pcx"	},
  { "custom_98.xpos",				"7"			},
  { "custom_98.ypos",				"9"			},
  { "custom_98.frames",				"1"			},
  { "custom_98.EDITOR",				"RocksElements.pcx"	},
  { "custom_98.EDITOR.xpos",			"15"			},
  { "custom_98.EDITOR.ypos",			"13"			},

  { "custom_99",				"RocksElements.pcx"	},
  { "custom_99.xpos",				"7"			},
  { "custom_99.ypos",				"9"			},
  { "custom_99.frames",				"1"			},
  { "custom_99.EDITOR",				"RocksElements.pcx"	},
  { "custom_99.EDITOR.xpos",			"15"			},
  { "custom_99.EDITOR.ypos",			"13"			},

  { "custom_100",				"RocksElements.pcx"	},
  { "custom_100.xpos",				"7"			},
  { "custom_100.ypos",				"9"			},
  { "custom_100.frames",			"1"			},
  { "custom_100.EDITOR",			"RocksElements.pcx"	},
  { "custom_100.EDITOR.xpos",			"15"			},
  { "custom_100.EDITOR.ypos",			"13"			},

  { "custom_101",				"RocksElements.pcx"	},
  { "custom_101.xpos",				"7"			},
  { "custom_101.ypos",				"9"			},
  { "custom_101.frames",			"1"			},
  { "custom_101.EDITOR",			"RocksElements.pcx"	},
  { "custom_101.EDITOR.xpos",			"15"			},
  { "custom_101.EDITOR.ypos",			"13"			},

  { "custom_102",				"RocksElements.pcx"	},
  { "custom_102.xpos",				"7"			},
  { "custom_102.ypos",				"9"			},
  { "custom_102.frames",			"1"			},
  { "custom_102.EDITOR",			"RocksElements.pcx"	},
  { "custom_102.EDITOR.xpos",			"15"			},
  { "custom_102.EDITOR.ypos",			"13"			},

  { "custom_103",				"RocksElements.pcx"	},
  { "custom_103.xpos",				"7"			},
  { "custom_103.ypos",				"9"			},
  { "custom_103.frames",			"1"			},
  { "custom_103.EDITOR",			"RocksElements.pcx"	},
  { "custom_103.EDITOR.xpos",			"15"			},
  { "custom_103.EDITOR.ypos",			"13"			},

  { "custom_104",				"RocksElements.pcx"	},
  { "custom_104.xpos",				"7"			},
  { "custom_104.ypos",				"9"			},
  { "custom_104.frames",			"1"			},
  { "custom_104.EDITOR",			"RocksElements.pcx"	},
  { "custom_104.EDITOR.xpos",			"15"			},
  { "custom_104.EDITOR.ypos",			"13"			},

  { "custom_105",				"RocksElements.pcx"	},
  { "custom_105.xpos",				"7"			},
  { "custom_105.ypos",				"9"			},
  { "custom_105.frames",			"1"			},
  { "custom_105.EDITOR",			"RocksElements.pcx"	},
  { "custom_105.EDITOR.xpos",			"15"			},
  { "custom_105.EDITOR.ypos",			"13"			},

  { "custom_106",				"RocksElements.pcx"	},
  { "custom_106.xpos",				"7"			},
  { "custom_106.ypos",				"9"			},
  { "custom_106.frames",			"1"			},
  { "custom_106.EDITOR",			"RocksElements.pcx"	},
  { "custom_106.EDITOR.xpos",			"15"			},
  { "custom_106.EDITOR.ypos",			"13"			},

  { "custom_107",				"RocksElements.pcx"	},
  { "custom_107.xpos",				"7"			},
  { "custom_107.ypos",				"9"			},
  { "custom_107.frames",			"1"			},
  { "custom_107.EDITOR",			"RocksElements.pcx"	},
  { "custom_107.EDITOR.xpos",			"15"			},
  { "custom_107.EDITOR.ypos",			"13"			},

  { "custom_108",				"RocksElements.pcx"	},
  { "custom_108.xpos",				"7"			},
  { "custom_108.ypos",				"9"			},
  { "custom_108.frames",			"1"			},
  { "custom_108.EDITOR",			"RocksElements.pcx"	},
  { "custom_108.EDITOR.xpos",			"15"			},
  { "custom_108.EDITOR.ypos",			"13"			},

  { "custom_109",				"RocksElements.pcx"	},
  { "custom_109.xpos",				"7"			},
  { "custom_109.ypos",				"9"			},
  { "custom_109.frames",			"1"			},
  { "custom_109.EDITOR",			"RocksElements.pcx"	},
  { "custom_109.EDITOR.xpos",			"15"			},
  { "custom_109.EDITOR.ypos",			"13"			},

  { "custom_110",				"RocksElements.pcx"	},
  { "custom_110.xpos",				"7"			},
  { "custom_110.ypos",				"9"			},
  { "custom_110.frames",			"1"			},
  { "custom_110.EDITOR",			"RocksElements.pcx"	},
  { "custom_110.EDITOR.xpos",			"15"			},
  { "custom_110.EDITOR.ypos",			"13"			},

  { "custom_111",				"RocksElements.pcx"	},
  { "custom_111.xpos",				"7"			},
  { "custom_111.ypos",				"9"			},
  { "custom_111.frames",			"1"			},
  { "custom_111.EDITOR",			"RocksElements.pcx"	},
  { "custom_111.EDITOR.xpos",			"15"			},
  { "custom_111.EDITOR.ypos",			"13"			},

  { "custom_112",				"RocksElements.pcx"	},
  { "custom_112.xpos",				"7"			},
  { "custom_112.ypos",				"9"			},
  { "custom_112.frames",			"1"			},
  { "custom_112.EDITOR",			"RocksElements.pcx"	},
  { "custom_112.EDITOR.xpos",			"15"			},
  { "custom_112.EDITOR.ypos",			"13"			},

  { "custom_113",				"RocksElements.pcx"	},
  { "custom_113.xpos",				"7"			},
  { "custom_113.ypos",				"9"			},
  { "custom_113.frames",			"1"			},
  { "custom_113.EDITOR",			"RocksElements.pcx"	},
  { "custom_113.EDITOR.xpos",			"15"			},
  { "custom_113.EDITOR.ypos",			"13"			},

  { "custom_114",				"RocksElements.pcx"	},
  { "custom_114.xpos",				"7"			},
  { "custom_114.ypos",				"9"			},
  { "custom_114.frames",			"1"			},
  { "custom_114.EDITOR",			"RocksElements.pcx"	},
  { "custom_114.EDITOR.xpos",			"15"			},
  { "custom_114.EDITOR.ypos",			"13"			},

  { "custom_115",				"RocksElements.pcx"	},
  { "custom_115.xpos",				"7"			},
  { "custom_115.ypos",				"9"			},
  { "custom_115.frames",			"1"			},
  { "custom_115.EDITOR",			"RocksElements.pcx"	},
  { "custom_115.EDITOR.xpos",			"15"			},
  { "custom_115.EDITOR.ypos",			"13"			},

  { "custom_116",				"RocksElements.pcx"	},
  { "custom_116.xpos",				"7"			},
  { "custom_116.ypos",				"9"			},
  { "custom_116.frames",			"1"			},
  { "custom_116.EDITOR",			"RocksElements.pcx"	},
  { "custom_116.EDITOR.xpos",			"15"			},
  { "custom_116.EDITOR.ypos",			"13"			},

  { "custom_117",				"RocksElements.pcx"	},
  { "custom_117.xpos",				"7"			},
  { "custom_117.ypos",				"9"			},
  { "custom_117.frames",			"1"			},
  { "custom_117.EDITOR",			"RocksElements.pcx"	},
  { "custom_117.EDITOR.xpos",			"15"			},
  { "custom_117.EDITOR.ypos",			"13"			},

  { "custom_118",				"RocksElements.pcx"	},
  { "custom_118.xpos",				"7"			},
  { "custom_118.ypos",				"9"			},
  { "custom_118.frames",			"1"			},
  { "custom_118.EDITOR",			"RocksElements.pcx"	},
  { "custom_118.EDITOR.xpos",			"15"			},
  { "custom_118.EDITOR.ypos",			"13"			},

  { "custom_119",				"RocksElements.pcx"	},
  { "custom_119.xpos",				"7"			},
  { "custom_119.ypos",				"9"			},
  { "custom_119.frames",			"1"			},
  { "custom_119.EDITOR",			"RocksElements.pcx"	},
  { "custom_119.EDITOR.xpos",			"15"			},
  { "custom_119.EDITOR.ypos",			"13"			},

  { "custom_120",				"RocksElements.pcx"	},
  { "custom_120.xpos",				"7"			},
  { "custom_120.ypos",				"9"			},
  { "custom_120.frames",			"1"			},
  { "custom_120.EDITOR",			"RocksElements.pcx"	},
  { "custom_120.EDITOR.xpos",			"15"			},
  { "custom_120.EDITOR.ypos",			"13"			},

  { "custom_121",				"RocksElements.pcx"	},
  { "custom_121.xpos",				"7"			},
  { "custom_121.ypos",				"9"			},
  { "custom_121.frames",			"1"			},
  { "custom_121.EDITOR",			"RocksElements.pcx"	},
  { "custom_121.EDITOR.xpos",			"15"			},
  { "custom_121.EDITOR.ypos",			"13"			},

  { "custom_122",				"RocksElements.pcx"	},
  { "custom_122.xpos",				"7"			},
  { "custom_122.ypos",				"9"			},
  { "custom_122.frames",			"1"			},
  { "custom_122.EDITOR",			"RocksElements.pcx"	},
  { "custom_122.EDITOR.xpos",			"15"			},
  { "custom_122.EDITOR.ypos",			"13"			},

  { "custom_123",				"RocksElements.pcx"	},
  { "custom_123.xpos",				"7"			},
  { "custom_123.ypos",				"9"			},
  { "custom_123.frames",			"1"			},
  { "custom_123.EDITOR",			"RocksElements.pcx"	},
  { "custom_123.EDITOR.xpos",			"15"			},
  { "custom_123.EDITOR.ypos",			"13"			},

  { "custom_124",				"RocksElements.pcx"	},
  { "custom_124.xpos",				"7"			},
  { "custom_124.ypos",				"9"			},
  { "custom_124.frames",			"1"			},
  { "custom_124.EDITOR",			"RocksElements.pcx"	},
  { "custom_124.EDITOR.xpos",			"15"			},
  { "custom_124.EDITOR.ypos",			"13"			},

  { "custom_125",				"RocksElements.pcx"	},
  { "custom_125.xpos",				"7"			},
  { "custom_125.ypos",				"9"			},
  { "custom_125.frames",			"1"			},
  { "custom_125.EDITOR",			"RocksElements.pcx"	},
  { "custom_125.EDITOR.xpos",			"15"			},
  { "custom_125.EDITOR.ypos",			"13"			},

  { "custom_126",				"RocksElements.pcx"	},
  { "custom_126.xpos",				"7"			},
  { "custom_126.ypos",				"9"			},
  { "custom_126.frames",			"1"			},
  { "custom_126.EDITOR",			"RocksElements.pcx"	},
  { "custom_126.EDITOR.xpos",			"15"			},
  { "custom_126.EDITOR.ypos",			"13"			},

  { "custom_127",				"RocksElements.pcx"	},
  { "custom_127.xpos",				"7"			},
  { "custom_127.ypos",				"9"			},
  { "custom_127.frames",			"1"			},
  { "custom_127.EDITOR",			"RocksElements.pcx"	},
  { "custom_127.EDITOR.xpos",			"15"			},
  { "custom_127.EDITOR.ypos",			"13"			},

  { "custom_128",				"RocksElements.pcx"	},
  { "custom_128.xpos",				"7"			},
  { "custom_128.ypos",				"9"			},
  { "custom_128.frames",			"1"			},
  { "custom_128.EDITOR",			"RocksElements.pcx"	},
  { "custom_128.EDITOR.xpos",			"15"			},
  { "custom_128.EDITOR.ypos",			"13"			},

  { "custom_129",				"RocksElements.pcx"	},
  { "custom_129.xpos",				"7"			},
  { "custom_129.ypos",				"9"			},
  { "custom_129.frames",			"1"			},
  { "custom_129.EDITOR",			"RocksElements.pcx"	},
  { "custom_129.EDITOR.xpos",			"15"			},
  { "custom_129.EDITOR.ypos",			"13"			},

  { "custom_130",				"RocksElements.pcx"	},
  { "custom_130.xpos",				"7"			},
  { "custom_130.ypos",				"9"			},
  { "custom_130.frames",			"1"			},
  { "custom_130.EDITOR",			"RocksElements.pcx"	},
  { "custom_130.EDITOR.xpos",			"15"			},
  { "custom_130.EDITOR.ypos",			"13"			},

  { "custom_131",				"RocksElements.pcx"	},
  { "custom_131.xpos",				"7"			},
  { "custom_131.ypos",				"9"			},
  { "custom_131.frames",			"1"			},
  { "custom_131.EDITOR",			"RocksElements.pcx"	},
  { "custom_131.EDITOR.xpos",			"15"			},
  { "custom_131.EDITOR.ypos",			"13"			},

  { "custom_132",				"RocksElements.pcx"	},
  { "custom_132.xpos",				"7"			},
  { "custom_132.ypos",				"9"			},
  { "custom_132.frames",			"1"			},
  { "custom_132.EDITOR",			"RocksElements.pcx"	},
  { "custom_132.EDITOR.xpos",			"15"			},
  { "custom_132.EDITOR.ypos",			"13"			},

  { "custom_133",				"RocksElements.pcx"	},
  { "custom_133.xpos",				"7"			},
  { "custom_133.ypos",				"9"			},
  { "custom_133.frames",			"1"			},
  { "custom_133.EDITOR",			"RocksElements.pcx"	},
  { "custom_133.EDITOR.xpos",			"15"			},
  { "custom_133.EDITOR.ypos",			"13"			},

  { "custom_134",				"RocksElements.pcx"	},
  { "custom_134.xpos",				"7"			},
  { "custom_134.ypos",				"9"			},
  { "custom_134.frames",			"1"			},
  { "custom_134.EDITOR",			"RocksElements.pcx"	},
  { "custom_134.EDITOR.xpos",			"15"			},
  { "custom_134.EDITOR.ypos",			"13"			},

  { "custom_135",				"RocksElements.pcx"	},
  { "custom_135.xpos",				"7"			},
  { "custom_135.ypos",				"9"			},
  { "custom_135.frames",			"1"			},
  { "custom_135.EDITOR",			"RocksElements.pcx"	},
  { "custom_135.EDITOR.xpos",			"15"			},
  { "custom_135.EDITOR.ypos",			"13"			},

  { "custom_136",				"RocksElements.pcx"	},
  { "custom_136.xpos",				"7"			},
  { "custom_136.ypos",				"9"			},
  { "custom_136.frames",			"1"			},
  { "custom_136.EDITOR",			"RocksElements.pcx"	},
  { "custom_136.EDITOR.xpos",			"15"			},
  { "custom_136.EDITOR.ypos",			"13"			},

  { "custom_137",				"RocksElements.pcx"	},
  { "custom_137.xpos",				"7"			},
  { "custom_137.ypos",				"9"			},
  { "custom_137.frames",			"1"			},
  { "custom_137.EDITOR",			"RocksElements.pcx"	},
  { "custom_137.EDITOR.xpos",			"15"			},
  { "custom_137.EDITOR.ypos",			"13"			},

  { "custom_138",				"RocksElements.pcx"	},
  { "custom_138.xpos",				"7"			},
  { "custom_138.ypos",				"9"			},
  { "custom_138.frames",			"1"			},
  { "custom_138.EDITOR",			"RocksElements.pcx"	},
  { "custom_138.EDITOR.xpos",			"15"			},
  { "custom_138.EDITOR.ypos",			"13"			},

  { "custom_139",				"RocksElements.pcx"	},
  { "custom_139.xpos",				"7"			},
  { "custom_139.ypos",				"9"			},
  { "custom_139.frames",			"1"			},
  { "custom_139.EDITOR",			"RocksElements.pcx"	},
  { "custom_139.EDITOR.xpos",			"15"			},
  { "custom_139.EDITOR.ypos",			"13"			},

  { "custom_140",				"RocksElements.pcx"	},
  { "custom_140.xpos",				"7"			},
  { "custom_140.ypos",				"9"			},
  { "custom_140.frames",			"1"			},
  { "custom_140.EDITOR",			"RocksElements.pcx"	},
  { "custom_140.EDITOR.xpos",			"15"			},
  { "custom_140.EDITOR.ypos",			"13"			},

  { "custom_141",				"RocksElements.pcx"	},
  { "custom_141.xpos",				"7"			},
  { "custom_141.ypos",				"9"			},
  { "custom_141.frames",			"1"			},
  { "custom_141.EDITOR",			"RocksElements.pcx"	},
  { "custom_141.EDITOR.xpos",			"15"			},
  { "custom_141.EDITOR.ypos",			"13"			},

  { "custom_142",				"RocksElements.pcx"	},
  { "custom_142.xpos",				"7"			},
  { "custom_142.ypos",				"9"			},
  { "custom_142.frames",			"1"			},
  { "custom_142.EDITOR",			"RocksElements.pcx"	},
  { "custom_142.EDITOR.xpos",			"15"			},
  { "custom_142.EDITOR.ypos",			"13"			},

  { "custom_143",				"RocksElements.pcx"	},
  { "custom_143.xpos",				"7"			},
  { "custom_143.ypos",				"9"			},
  { "custom_143.frames",			"1"			},
  { "custom_143.EDITOR",			"RocksElements.pcx"	},
  { "custom_143.EDITOR.xpos",			"15"			},
  { "custom_143.EDITOR.ypos",			"13"			},

  { "custom_144",				"RocksElements.pcx"	},
  { "custom_144.xpos",				"7"			},
  { "custom_144.ypos",				"9"			},
  { "custom_144.frames",			"1"			},
  { "custom_144.EDITOR",			"RocksElements.pcx"	},
  { "custom_144.EDITOR.xpos",			"15"			},
  { "custom_144.EDITOR.ypos",			"13"			},

  { "custom_145",				"RocksElements.pcx"	},
  { "custom_145.xpos",				"7"			},
  { "custom_145.ypos",				"9"			},
  { "custom_145.frames",			"1"			},
  { "custom_145.EDITOR",			"RocksElements.pcx"	},
  { "custom_145.EDITOR.xpos",			"15"			},
  { "custom_145.EDITOR.ypos",			"13"			},

  { "custom_146",				"RocksElements.pcx"	},
  { "custom_146.xpos",				"7"			},
  { "custom_146.ypos",				"9"			},
  { "custom_146.frames",			"1"			},
  { "custom_146.EDITOR",			"RocksElements.pcx"	},
  { "custom_146.EDITOR.xpos",			"15"			},
  { "custom_146.EDITOR.ypos",			"13"			},

  { "custom_147",				"RocksElements.pcx"	},
  { "custom_147.xpos",				"7"			},
  { "custom_147.ypos",				"9"			},
  { "custom_147.frames",			"1"			},
  { "custom_147.EDITOR",			"RocksElements.pcx"	},
  { "custom_147.EDITOR.xpos",			"15"			},
  { "custom_147.EDITOR.ypos",			"13"			},

  { "custom_148",				"RocksElements.pcx"	},
  { "custom_148.xpos",				"7"			},
  { "custom_148.ypos",				"9"			},
  { "custom_148.frames",			"1"			},
  { "custom_148.EDITOR",			"RocksElements.pcx"	},
  { "custom_148.EDITOR.xpos",			"15"			},
  { "custom_148.EDITOR.ypos",			"13"			},

  { "custom_149",				"RocksElements.pcx"	},
  { "custom_149.xpos",				"7"			},
  { "custom_149.ypos",				"9"			},
  { "custom_149.frames",			"1"			},
  { "custom_149.EDITOR",			"RocksElements.pcx"	},
  { "custom_149.EDITOR.xpos",			"15"			},
  { "custom_149.EDITOR.ypos",			"13"			},

  { "custom_150",				"RocksElements.pcx"	},
  { "custom_150.xpos",				"7"			},
  { "custom_150.ypos",				"9"			},
  { "custom_150.frames",			"1"			},
  { "custom_150.EDITOR",			"RocksElements.pcx"	},
  { "custom_150.EDITOR.xpos",			"15"			},
  { "custom_150.EDITOR.ypos",			"13"			},

  { "custom_151",				"RocksElements.pcx"	},
  { "custom_151.xpos",				"7"			},
  { "custom_151.ypos",				"9"			},
  { "custom_151.frames",			"1"			},
  { "custom_151.EDITOR",			"RocksElements.pcx"	},
  { "custom_151.EDITOR.xpos",			"15"			},
  { "custom_151.EDITOR.ypos",			"13"			},

  { "custom_152",				"RocksElements.pcx"	},
  { "custom_152.xpos",				"7"			},
  { "custom_152.ypos",				"9"			},
  { "custom_152.frames",			"1"			},
  { "custom_152.EDITOR",			"RocksElements.pcx"	},
  { "custom_152.EDITOR.xpos",			"15"			},
  { "custom_152.EDITOR.ypos",			"13"			},

  { "custom_153",				"RocksElements.pcx"	},
  { "custom_153.xpos",				"7"			},
  { "custom_153.ypos",				"9"			},
  { "custom_153.frames",			"1"			},
  { "custom_153.EDITOR",			"RocksElements.pcx"	},
  { "custom_153.EDITOR.xpos",			"15"			},
  { "custom_153.EDITOR.ypos",			"13"			},

  { "custom_154",				"RocksElements.pcx"	},
  { "custom_154.xpos",				"7"			},
  { "custom_154.ypos",				"9"			},
  { "custom_154.frames",			"1"			},
  { "custom_154.EDITOR",			"RocksElements.pcx"	},
  { "custom_154.EDITOR.xpos",			"15"			},
  { "custom_154.EDITOR.ypos",			"13"			},

  { "custom_155",				"RocksElements.pcx"	},
  { "custom_155.xpos",				"7"			},
  { "custom_155.ypos",				"9"			},
  { "custom_155.frames",			"1"			},
  { "custom_155.EDITOR",			"RocksElements.pcx"	},
  { "custom_155.EDITOR.xpos",			"15"			},
  { "custom_155.EDITOR.ypos",			"13"			},

  { "custom_156",				"RocksElements.pcx"	},
  { "custom_156.xpos",				"7"			},
  { "custom_156.ypos",				"9"			},
  { "custom_156.frames",			"1"			},
  { "custom_156.EDITOR",			"RocksElements.pcx"	},
  { "custom_156.EDITOR.xpos",			"15"			},
  { "custom_156.EDITOR.ypos",			"13"			},

  { "custom_157",				"RocksElements.pcx"	},
  { "custom_157.xpos",				"7"			},
  { "custom_157.ypos",				"9"			},
  { "custom_157.frames",			"1"			},
  { "custom_157.EDITOR",			"RocksElements.pcx"	},
  { "custom_157.EDITOR.xpos",			"15"			},
  { "custom_157.EDITOR.ypos",			"13"			},

  { "custom_158",				"RocksElements.pcx"	},
  { "custom_158.xpos",				"7"			},
  { "custom_158.ypos",				"9"			},
  { "custom_158.frames",			"1"			},
  { "custom_158.EDITOR",			"RocksElements.pcx"	},
  { "custom_158.EDITOR.xpos",			"15"			},
  { "custom_158.EDITOR.ypos",			"13"			},

  { "custom_159",				"RocksElements.pcx"	},
  { "custom_159.xpos",				"7"			},
  { "custom_159.ypos",				"9"			},
  { "custom_159.frames",			"1"			},
  { "custom_159.EDITOR",			"RocksElements.pcx"	},
  { "custom_159.EDITOR.xpos",			"15"			},
  { "custom_159.EDITOR.ypos",			"13"			},

  { "custom_160",				"RocksElements.pcx"	},
  { "custom_160.xpos",				"7"			},
  { "custom_160.ypos",				"9"			},
  { "custom_160.frames",			"1"			},
  { "custom_160.EDITOR",			"RocksElements.pcx"	},
  { "custom_160.EDITOR.xpos",			"15"			},
  { "custom_160.EDITOR.ypos",			"13"			},

  { "custom_161",				"RocksElements.pcx"	},
  { "custom_161.xpos",				"7"			},
  { "custom_161.ypos",				"9"			},
  { "custom_161.frames",			"1"			},
  { "custom_161.EDITOR",			"RocksElements.pcx"	},
  { "custom_161.EDITOR.xpos",			"15"			},
  { "custom_161.EDITOR.ypos",			"13"			},

  { "custom_162",				"RocksElements.pcx"	},
  { "custom_162.xpos",				"7"			},
  { "custom_162.ypos",				"9"			},
  { "custom_162.frames",			"1"			},
  { "custom_162.EDITOR",			"RocksElements.pcx"	},
  { "custom_162.EDITOR.xpos",			"15"			},
  { "custom_162.EDITOR.ypos",			"13"			},

  { "custom_163",				"RocksElements.pcx"	},
  { "custom_163.xpos",				"7"			},
  { "custom_163.ypos",				"9"			},
  { "custom_163.frames",			"1"			},
  { "custom_163.EDITOR",			"RocksElements.pcx"	},
  { "custom_163.EDITOR.xpos",			"15"			},
  { "custom_163.EDITOR.ypos",			"13"			},

  { "custom_164",				"RocksElements.pcx"	},
  { "custom_164.xpos",				"7"			},
  { "custom_164.ypos",				"9"			},
  { "custom_164.frames",			"1"			},
  { "custom_164.EDITOR",			"RocksElements.pcx"	},
  { "custom_164.EDITOR.xpos",			"15"			},
  { "custom_164.EDITOR.ypos",			"13"			},

  { "custom_165",				"RocksElements.pcx"	},
  { "custom_165.xpos",				"7"			},
  { "custom_165.ypos",				"9"			},
  { "custom_165.frames",			"1"			},
  { "custom_165.EDITOR",			"RocksElements.pcx"	},
  { "custom_165.EDITOR.xpos",			"15"			},
  { "custom_165.EDITOR.ypos",			"13"			},

  { "custom_166",				"RocksElements.pcx"	},
  { "custom_166.xpos",				"7"			},
  { "custom_166.ypos",				"9"			},
  { "custom_166.frames",			"1"			},
  { "custom_166.EDITOR",			"RocksElements.pcx"	},
  { "custom_166.EDITOR.xpos",			"15"			},
  { "custom_166.EDITOR.ypos",			"13"			},

  { "custom_167",				"RocksElements.pcx"	},
  { "custom_167.xpos",				"7"			},
  { "custom_167.ypos",				"9"			},
  { "custom_167.frames",			"1"			},
  { "custom_167.EDITOR",			"RocksElements.pcx"	},
  { "custom_167.EDITOR.xpos",			"15"			},
  { "custom_167.EDITOR.ypos",			"13"			},

  { "custom_168",				"RocksElements.pcx"	},
  { "custom_168.xpos",				"7"			},
  { "custom_168.ypos",				"9"			},
  { "custom_168.frames",			"1"			},
  { "custom_168.EDITOR",			"RocksElements.pcx"	},
  { "custom_168.EDITOR.xpos",			"15"			},
  { "custom_168.EDITOR.ypos",			"13"			},

  { "custom_169",				"RocksElements.pcx"	},
  { "custom_169.xpos",				"7"			},
  { "custom_169.ypos",				"9"			},
  { "custom_169.frames",			"1"			},
  { "custom_169.EDITOR",			"RocksElements.pcx"	},
  { "custom_169.EDITOR.xpos",			"15"			},
  { "custom_169.EDITOR.ypos",			"13"			},

  { "custom_170",				"RocksElements.pcx"	},
  { "custom_170.xpos",				"7"			},
  { "custom_170.ypos",				"9"			},
  { "custom_170.frames",			"1"			},
  { "custom_170.EDITOR",			"RocksElements.pcx"	},
  { "custom_170.EDITOR.xpos",			"15"			},
  { "custom_170.EDITOR.ypos",			"13"			},

  { "custom_171",				"RocksElements.pcx"	},
  { "custom_171.xpos",				"7"			},
  { "custom_171.ypos",				"9"			},
  { "custom_171.frames",			"1"			},
  { "custom_171.EDITOR",			"RocksElements.pcx"	},
  { "custom_171.EDITOR.xpos",			"15"			},
  { "custom_171.EDITOR.ypos",			"13"			},

  { "custom_172",				"RocksElements.pcx"	},
  { "custom_172.xpos",				"7"			},
  { "custom_172.ypos",				"9"			},
  { "custom_172.frames",			"1"			},
  { "custom_172.EDITOR",			"RocksElements.pcx"	},
  { "custom_172.EDITOR.xpos",			"15"			},
  { "custom_172.EDITOR.ypos",			"13"			},

  { "custom_173",				"RocksElements.pcx"	},
  { "custom_173.xpos",				"7"			},
  { "custom_173.ypos",				"9"			},
  { "custom_173.frames",			"1"			},
  { "custom_173.EDITOR",			"RocksElements.pcx"	},
  { "custom_173.EDITOR.xpos",			"15"			},
  { "custom_173.EDITOR.ypos",			"13"			},

  { "custom_174",				"RocksElements.pcx"	},
  { "custom_174.xpos",				"7"			},
  { "custom_174.ypos",				"9"			},
  { "custom_174.frames",			"1"			},
  { "custom_174.EDITOR",			"RocksElements.pcx"	},
  { "custom_174.EDITOR.xpos",			"15"			},
  { "custom_174.EDITOR.ypos",			"13"			},

  { "custom_175",				"RocksElements.pcx"	},
  { "custom_175.xpos",				"7"			},
  { "custom_175.ypos",				"9"			},
  { "custom_175.frames",			"1"			},
  { "custom_175.EDITOR",			"RocksElements.pcx"	},
  { "custom_175.EDITOR.xpos",			"15"			},
  { "custom_175.EDITOR.ypos",			"13"			},

  { "custom_176",				"RocksElements.pcx"	},
  { "custom_176.xpos",				"7"			},
  { "custom_176.ypos",				"9"			},
  { "custom_176.frames",			"1"			},
  { "custom_176.EDITOR",			"RocksElements.pcx"	},
  { "custom_176.EDITOR.xpos",			"15"			},
  { "custom_176.EDITOR.ypos",			"13"			},

  { "custom_177",				"RocksElements.pcx"	},
  { "custom_177.xpos",				"7"			},
  { "custom_177.ypos",				"9"			},
  { "custom_177.frames",			"1"			},
  { "custom_177.EDITOR",			"RocksElements.pcx"	},
  { "custom_177.EDITOR.xpos",			"15"			},
  { "custom_177.EDITOR.ypos",			"13"			},

  { "custom_178",				"RocksElements.pcx"	},
  { "custom_178.xpos",				"7"			},
  { "custom_178.ypos",				"9"			},
  { "custom_178.frames",			"1"			},
  { "custom_178.EDITOR",			"RocksElements.pcx"	},
  { "custom_178.EDITOR.xpos",			"15"			},
  { "custom_178.EDITOR.ypos",			"13"			},

  { "custom_179",				"RocksElements.pcx"	},
  { "custom_179.xpos",				"7"			},
  { "custom_179.ypos",				"9"			},
  { "custom_179.frames",			"1"			},
  { "custom_179.EDITOR",			"RocksElements.pcx"	},
  { "custom_179.EDITOR.xpos",			"15"			},
  { "custom_179.EDITOR.ypos",			"13"			},

  { "custom_180",				"RocksElements.pcx"	},
  { "custom_180.xpos",				"7"			},
  { "custom_180.ypos",				"9"			},
  { "custom_180.frames",			"1"			},
  { "custom_180.EDITOR",			"RocksElements.pcx"	},
  { "custom_180.EDITOR.xpos",			"15"			},
  { "custom_180.EDITOR.ypos",			"13"			},

  { "custom_181",				"RocksElements.pcx"	},
  { "custom_181.xpos",				"7"			},
  { "custom_181.ypos",				"9"			},
  { "custom_181.frames",			"1"			},
  { "custom_181.EDITOR",			"RocksElements.pcx"	},
  { "custom_181.EDITOR.xpos",			"15"			},
  { "custom_181.EDITOR.ypos",			"13"			},

  { "custom_182",				"RocksElements.pcx"	},
  { "custom_182.xpos",				"7"			},
  { "custom_182.ypos",				"9"			},
  { "custom_182.frames",			"1"			},
  { "custom_182.EDITOR",			"RocksElements.pcx"	},
  { "custom_182.EDITOR.xpos",			"15"			},
  { "custom_182.EDITOR.ypos",			"13"			},

  { "custom_183",				"RocksElements.pcx"	},
  { "custom_183.xpos",				"7"			},
  { "custom_183.ypos",				"9"			},
  { "custom_183.frames",			"1"			},
  { "custom_183.EDITOR",			"RocksElements.pcx"	},
  { "custom_183.EDITOR.xpos",			"15"			},
  { "custom_183.EDITOR.ypos",			"13"			},

  { "custom_184",				"RocksElements.pcx"	},
  { "custom_184.xpos",				"7"			},
  { "custom_184.ypos",				"9"			},
  { "custom_184.frames",			"1"			},
  { "custom_184.EDITOR",			"RocksElements.pcx"	},
  { "custom_184.EDITOR.xpos",			"15"			},
  { "custom_184.EDITOR.ypos",			"13"			},

  { "custom_185",				"RocksElements.pcx"	},
  { "custom_185.xpos",				"7"			},
  { "custom_185.ypos",				"9"			},
  { "custom_185.frames",			"1"			},
  { "custom_185.EDITOR",			"RocksElements.pcx"	},
  { "custom_185.EDITOR.xpos",			"15"			},
  { "custom_185.EDITOR.ypos",			"13"			},

  { "custom_186",				"RocksElements.pcx"	},
  { "custom_186.xpos",				"7"			},
  { "custom_186.ypos",				"9"			},
  { "custom_186.frames",			"1"			},
  { "custom_186.EDITOR",			"RocksElements.pcx"	},
  { "custom_186.EDITOR.xpos",			"15"			},
  { "custom_186.EDITOR.ypos",			"13"			},

  { "custom_187",				"RocksElements.pcx"	},
  { "custom_187.xpos",				"7"			},
  { "custom_187.ypos",				"9"			},
  { "custom_187.frames",			"1"			},
  { "custom_187.EDITOR",			"RocksElements.pcx"	},
  { "custom_187.EDITOR.xpos",			"15"			},
  { "custom_187.EDITOR.ypos",			"13"			},

  { "custom_188",				"RocksElements.pcx"	},
  { "custom_188.xpos",				"7"			},
  { "custom_188.ypos",				"9"			},
  { "custom_188.frames",			"1"			},
  { "custom_188.EDITOR",			"RocksElements.pcx"	},
  { "custom_188.EDITOR.xpos",			"15"			},
  { "custom_188.EDITOR.ypos",			"13"			},

  { "custom_189",				"RocksElements.pcx"	},
  { "custom_189.xpos",				"7"			},
  { "custom_189.ypos",				"9"			},
  { "custom_189.frames",			"1"			},
  { "custom_189.EDITOR",			"RocksElements.pcx"	},
  { "custom_189.EDITOR.xpos",			"15"			},
  { "custom_189.EDITOR.ypos",			"13"			},

  { "custom_190",				"RocksElements.pcx"	},
  { "custom_190.xpos",				"7"			},
  { "custom_190.ypos",				"9"			},
  { "custom_190.frames",			"1"			},
  { "custom_190.EDITOR",			"RocksElements.pcx"	},
  { "custom_190.EDITOR.xpos",			"15"			},
  { "custom_190.EDITOR.ypos",			"13"			},

  { "custom_191",				"RocksElements.pcx"	},
  { "custom_191.xpos",				"7"			},
  { "custom_191.ypos",				"9"			},
  { "custom_191.frames",			"1"			},
  { "custom_191.EDITOR",			"RocksElements.pcx"	},
  { "custom_191.EDITOR.xpos",			"15"			},
  { "custom_191.EDITOR.ypos",			"13"			},

  { "custom_192",				"RocksElements.pcx"	},
  { "custom_192.xpos",				"7"			},
  { "custom_192.ypos",				"9"			},
  { "custom_192.frames",			"1"			},
  { "custom_192.EDITOR",			"RocksElements.pcx"	},
  { "custom_192.EDITOR.xpos",			"15"			},
  { "custom_192.EDITOR.ypos",			"13"			},

  { "custom_193",				"RocksElements.pcx"	},
  { "custom_193.xpos",				"7"			},
  { "custom_193.ypos",				"9"			},
  { "custom_193.frames",			"1"			},
  { "custom_193.EDITOR",			"RocksElements.pcx"	},
  { "custom_193.EDITOR.xpos",			"15"			},
  { "custom_193.EDITOR.ypos",			"13"			},

  { "custom_194",				"RocksElements.pcx"	},
  { "custom_194.xpos",				"7"			},
  { "custom_194.ypos",				"9"			},
  { "custom_194.frames",			"1"			},
  { "custom_194.EDITOR",			"RocksElements.pcx"	},
  { "custom_194.EDITOR.xpos",			"15"			},
  { "custom_194.EDITOR.ypos",			"13"			},

  { "custom_195",				"RocksElements.pcx"	},
  { "custom_195.xpos",				"7"			},
  { "custom_195.ypos",				"9"			},
  { "custom_195.frames",			"1"			},
  { "custom_195.EDITOR",			"RocksElements.pcx"	},
  { "custom_195.EDITOR.xpos",			"15"			},
  { "custom_195.EDITOR.ypos",			"13"			},

  { "custom_196",				"RocksElements.pcx"	},
  { "custom_196.xpos",				"7"			},
  { "custom_196.ypos",				"9"			},
  { "custom_196.frames",			"1"			},
  { "custom_196.EDITOR",			"RocksElements.pcx"	},
  { "custom_196.EDITOR.xpos",			"15"			},
  { "custom_196.EDITOR.ypos",			"13"			},

  { "custom_197",				"RocksElements.pcx"	},
  { "custom_197.xpos",				"7"			},
  { "custom_197.ypos",				"9"			},
  { "custom_197.frames",			"1"			},
  { "custom_197.EDITOR",			"RocksElements.pcx"	},
  { "custom_197.EDITOR.xpos",			"15"			},
  { "custom_197.EDITOR.ypos",			"13"			},

  { "custom_198",				"RocksElements.pcx"	},
  { "custom_198.xpos",				"7"			},
  { "custom_198.ypos",				"9"			},
  { "custom_198.frames",			"1"			},
  { "custom_198.EDITOR",			"RocksElements.pcx"	},
  { "custom_198.EDITOR.xpos",			"15"			},
  { "custom_198.EDITOR.ypos",			"13"			},

  { "custom_199",				"RocksElements.pcx"	},
  { "custom_199.xpos",				"7"			},
  { "custom_199.ypos",				"9"			},
  { "custom_199.frames",			"1"			},
  { "custom_199.EDITOR",			"RocksElements.pcx"	},
  { "custom_199.EDITOR.xpos",			"15"			},
  { "custom_199.EDITOR.ypos",			"13"			},

  { "custom_200",				"RocksElements.pcx"	},
  { "custom_200.xpos",				"7"			},
  { "custom_200.ypos",				"9"			},
  { "custom_200.frames",			"1"			},
  { "custom_200.EDITOR",			"RocksElements.pcx"	},
  { "custom_200.EDITOR.xpos",			"15"			},
  { "custom_200.EDITOR.ypos",			"13"			},

  { "custom_201",				"RocksElements.pcx"	},
  { "custom_201.xpos",				"7"			},
  { "custom_201.ypos",				"9"			},
  { "custom_201.frames",			"1"			},
  { "custom_201.EDITOR",			"RocksElements.pcx"	},
  { "custom_201.EDITOR.xpos",			"15"			},
  { "custom_201.EDITOR.ypos",			"13"			},

  { "custom_202",				"RocksElements.pcx"	},
  { "custom_202.xpos",				"7"			},
  { "custom_202.ypos",				"9"			},
  { "custom_202.frames",			"1"			},
  { "custom_202.EDITOR",			"RocksElements.pcx"	},
  { "custom_202.EDITOR.xpos",			"15"			},
  { "custom_202.EDITOR.ypos",			"13"			},

  { "custom_203",				"RocksElements.pcx"	},
  { "custom_203.xpos",				"7"			},
  { "custom_203.ypos",				"9"			},
  { "custom_203.frames",			"1"			},
  { "custom_203.EDITOR",			"RocksElements.pcx"	},
  { "custom_203.EDITOR.xpos",			"15"			},
  { "custom_203.EDITOR.ypos",			"13"			},

  { "custom_204",				"RocksElements.pcx"	},
  { "custom_204.xpos",				"7"			},
  { "custom_204.ypos",				"9"			},
  { "custom_204.frames",			"1"			},
  { "custom_204.EDITOR",			"RocksElements.pcx"	},
  { "custom_204.EDITOR.xpos",			"15"			},
  { "custom_204.EDITOR.ypos",			"13"			},

  { "custom_205",				"RocksElements.pcx"	},
  { "custom_205.xpos",				"7"			},
  { "custom_205.ypos",				"9"			},
  { "custom_205.frames",			"1"			},
  { "custom_205.EDITOR",			"RocksElements.pcx"	},
  { "custom_205.EDITOR.xpos",			"15"			},
  { "custom_205.EDITOR.ypos",			"13"			},

  { "custom_206",				"RocksElements.pcx"	},
  { "custom_206.xpos",				"7"			},
  { "custom_206.ypos",				"9"			},
  { "custom_206.frames",			"1"			},
  { "custom_206.EDITOR",			"RocksElements.pcx"	},
  { "custom_206.EDITOR.xpos",			"15"			},
  { "custom_206.EDITOR.ypos",			"13"			},

  { "custom_207",				"RocksElements.pcx"	},
  { "custom_207.xpos",				"7"			},
  { "custom_207.ypos",				"9"			},
  { "custom_207.frames",			"1"			},
  { "custom_207.EDITOR",			"RocksElements.pcx"	},
  { "custom_207.EDITOR.xpos",			"15"			},
  { "custom_207.EDITOR.ypos",			"13"			},

  { "custom_208",				"RocksElements.pcx"	},
  { "custom_208.xpos",				"7"			},
  { "custom_208.ypos",				"9"			},
  { "custom_208.frames",			"1"			},
  { "custom_208.EDITOR",			"RocksElements.pcx"	},
  { "custom_208.EDITOR.xpos",			"15"			},
  { "custom_208.EDITOR.ypos",			"13"			},

  { "custom_209",				"RocksElements.pcx"	},
  { "custom_209.xpos",				"7"			},
  { "custom_209.ypos",				"9"			},
  { "custom_209.frames",			"1"			},
  { "custom_209.EDITOR",			"RocksElements.pcx"	},
  { "custom_209.EDITOR.xpos",			"15"			},
  { "custom_209.EDITOR.ypos",			"13"			},

  { "custom_210",				"RocksElements.pcx"	},
  { "custom_210.xpos",				"7"			},
  { "custom_210.ypos",				"9"			},
  { "custom_210.frames",			"1"			},
  { "custom_210.EDITOR",			"RocksElements.pcx"	},
  { "custom_210.EDITOR.xpos",			"15"			},
  { "custom_210.EDITOR.ypos",			"13"			},

  { "custom_211",				"RocksElements.pcx"	},
  { "custom_211.xpos",				"7"			},
  { "custom_211.ypos",				"9"			},
  { "custom_211.frames",			"1"			},
  { "custom_211.EDITOR",			"RocksElements.pcx"	},
  { "custom_211.EDITOR.xpos",			"15"			},
  { "custom_211.EDITOR.ypos",			"13"			},

  { "custom_212",				"RocksElements.pcx"	},
  { "custom_212.xpos",				"7"			},
  { "custom_212.ypos",				"9"			},
  { "custom_212.frames",			"1"			},
  { "custom_212.EDITOR",			"RocksElements.pcx"	},
  { "custom_212.EDITOR.xpos",			"15"			},
  { "custom_212.EDITOR.ypos",			"13"			},

  { "custom_213",				"RocksElements.pcx"	},
  { "custom_213.xpos",				"7"			},
  { "custom_213.ypos",				"9"			},
  { "custom_213.frames",			"1"			},
  { "custom_213.EDITOR",			"RocksElements.pcx"	},
  { "custom_213.EDITOR.xpos",			"15"			},
  { "custom_213.EDITOR.ypos",			"13"			},

  { "custom_214",				"RocksElements.pcx"	},
  { "custom_214.xpos",				"7"			},
  { "custom_214.ypos",				"9"			},
  { "custom_214.frames",			"1"			},
  { "custom_214.EDITOR",			"RocksElements.pcx"	},
  { "custom_214.EDITOR.xpos",			"15"			},
  { "custom_214.EDITOR.ypos",			"13"			},

  { "custom_215",				"RocksElements.pcx"	},
  { "custom_215.xpos",				"7"			},
  { "custom_215.ypos",				"9"			},
  { "custom_215.frames",			"1"			},
  { "custom_215.EDITOR",			"RocksElements.pcx"	},
  { "custom_215.EDITOR.xpos",			"15"			},
  { "custom_215.EDITOR.ypos",			"13"			},

  { "custom_216",				"RocksElements.pcx"	},
  { "custom_216.xpos",				"7"			},
  { "custom_216.ypos",				"9"			},
  { "custom_216.frames",			"1"			},
  { "custom_216.EDITOR",			"RocksElements.pcx"	},
  { "custom_216.EDITOR.xpos",			"15"			},
  { "custom_216.EDITOR.ypos",			"13"			},

  { "custom_217",				"RocksElements.pcx"	},
  { "custom_217.xpos",				"7"			},
  { "custom_217.ypos",				"9"			},
  { "custom_217.frames",			"1"			},
  { "custom_217.EDITOR",			"RocksElements.pcx"	},
  { "custom_217.EDITOR.xpos",			"15"			},
  { "custom_217.EDITOR.ypos",			"13"			},

  { "custom_218",				"RocksElements.pcx"	},
  { "custom_218.xpos",				"7"			},
  { "custom_218.ypos",				"9"			},
  { "custom_218.frames",			"1"			},
  { "custom_218.EDITOR",			"RocksElements.pcx"	},
  { "custom_218.EDITOR.xpos",			"15"			},
  { "custom_218.EDITOR.ypos",			"13"			},

  { "custom_219",				"RocksElements.pcx"	},
  { "custom_219.xpos",				"7"			},
  { "custom_219.ypos",				"9"			},
  { "custom_219.frames",			"1"			},
  { "custom_219.EDITOR",			"RocksElements.pcx"	},
  { "custom_219.EDITOR.xpos",			"15"			},
  { "custom_219.EDITOR.ypos",			"13"			},

  { "custom_220",				"RocksElements.pcx"	},
  { "custom_220.xpos",				"7"			},
  { "custom_220.ypos",				"9"			},
  { "custom_220.frames",			"1"			},
  { "custom_220.EDITOR",			"RocksElements.pcx"	},
  { "custom_220.EDITOR.xpos",			"15"			},
  { "custom_220.EDITOR.ypos",			"13"			},

  { "custom_221",				"RocksElements.pcx"	},
  { "custom_221.xpos",				"7"			},
  { "custom_221.ypos",				"9"			},
  { "custom_221.frames",			"1"			},
  { "custom_221.EDITOR",			"RocksElements.pcx"	},
  { "custom_221.EDITOR.xpos",			"15"			},
  { "custom_221.EDITOR.ypos",			"13"			},

  { "custom_222",				"RocksElements.pcx"	},
  { "custom_222.xpos",				"7"			},
  { "custom_222.ypos",				"9"			},
  { "custom_222.frames",			"1"			},
  { "custom_222.EDITOR",			"RocksElements.pcx"	},
  { "custom_222.EDITOR.xpos",			"15"			},
  { "custom_222.EDITOR.ypos",			"13"			},

  { "custom_223",				"RocksElements.pcx"	},
  { "custom_223.xpos",				"7"			},
  { "custom_223.ypos",				"9"			},
  { "custom_223.frames",			"1"			},
  { "custom_223.EDITOR",			"RocksElements.pcx"	},
  { "custom_223.EDITOR.xpos",			"15"			},
  { "custom_223.EDITOR.ypos",			"13"			},

  { "custom_224",				"RocksElements.pcx"	},
  { "custom_224.xpos",				"7"			},
  { "custom_224.ypos",				"9"			},
  { "custom_224.frames",			"1"			},
  { "custom_224.EDITOR",			"RocksElements.pcx"	},
  { "custom_224.EDITOR.xpos",			"15"			},
  { "custom_224.EDITOR.ypos",			"13"			},

  { "custom_225",				"RocksElements.pcx"	},
  { "custom_225.xpos",				"7"			},
  { "custom_225.ypos",				"9"			},
  { "custom_225.frames",			"1"			},
  { "custom_225.EDITOR",			"RocksElements.pcx"	},
  { "custom_225.EDITOR.xpos",			"15"			},
  { "custom_225.EDITOR.ypos",			"13"			},

  { "custom_226",				"RocksElements.pcx"	},
  { "custom_226.xpos",				"7"			},
  { "custom_226.ypos",				"9"			},
  { "custom_226.frames",			"1"			},
  { "custom_226.EDITOR",			"RocksElements.pcx"	},
  { "custom_226.EDITOR.xpos",			"15"			},
  { "custom_226.EDITOR.ypos",			"13"			},

  { "custom_227",				"RocksElements.pcx"	},
  { "custom_227.xpos",				"7"			},
  { "custom_227.ypos",				"9"			},
  { "custom_227.frames",			"1"			},
  { "custom_227.EDITOR",			"RocksElements.pcx"	},
  { "custom_227.EDITOR.xpos",			"15"			},
  { "custom_227.EDITOR.ypos",			"13"			},

  { "custom_228",				"RocksElements.pcx"	},
  { "custom_228.xpos",				"7"			},
  { "custom_228.ypos",				"9"			},
  { "custom_228.frames",			"1"			},
  { "custom_228.EDITOR",			"RocksElements.pcx"	},
  { "custom_228.EDITOR.xpos",			"15"			},
  { "custom_228.EDITOR.ypos",			"13"			},

  { "custom_229",				"RocksElements.pcx"	},
  { "custom_229.xpos",				"7"			},
  { "custom_229.ypos",				"9"			},
  { "custom_229.frames",			"1"			},
  { "custom_229.EDITOR",			"RocksElements.pcx"	},
  { "custom_229.EDITOR.xpos",			"15"			},
  { "custom_229.EDITOR.ypos",			"13"			},

  { "custom_230",				"RocksElements.pcx"	},
  { "custom_230.xpos",				"7"			},
  { "custom_230.ypos",				"9"			},
  { "custom_230.frames",			"1"			},
  { "custom_230.EDITOR",			"RocksElements.pcx"	},
  { "custom_230.EDITOR.xpos",			"15"			},
  { "custom_230.EDITOR.ypos",			"13"			},

  { "custom_231",				"RocksElements.pcx"	},
  { "custom_231.xpos",				"7"			},
  { "custom_231.ypos",				"9"			},
  { "custom_231.frames",			"1"			},
  { "custom_231.EDITOR",			"RocksElements.pcx"	},
  { "custom_231.EDITOR.xpos",			"15"			},
  { "custom_231.EDITOR.ypos",			"13"			},

  { "custom_232",				"RocksElements.pcx"	},
  { "custom_232.xpos",				"7"			},
  { "custom_232.ypos",				"9"			},
  { "custom_232.frames",			"1"			},
  { "custom_232.EDITOR",			"RocksElements.pcx"	},
  { "custom_232.EDITOR.xpos",			"15"			},
  { "custom_232.EDITOR.ypos",			"13"			},

  { "custom_233",				"RocksElements.pcx"	},
  { "custom_233.xpos",				"7"			},
  { "custom_233.ypos",				"9"			},
  { "custom_233.frames",			"1"			},
  { "custom_233.EDITOR",			"RocksElements.pcx"	},
  { "custom_233.EDITOR.xpos",			"15"			},
  { "custom_233.EDITOR.ypos",			"13"			},

  { "custom_234",				"RocksElements.pcx"	},
  { "custom_234.xpos",				"7"			},
  { "custom_234.ypos",				"9"			},
  { "custom_234.frames",			"1"			},
  { "custom_234.EDITOR",			"RocksElements.pcx"	},
  { "custom_234.EDITOR.xpos",			"15"			},
  { "custom_234.EDITOR.ypos",			"13"			},

  { "custom_235",				"RocksElements.pcx"	},
  { "custom_235.xpos",				"7"			},
  { "custom_235.ypos",				"9"			},
  { "custom_235.frames",			"1"			},
  { "custom_235.EDITOR",			"RocksElements.pcx"	},
  { "custom_235.EDITOR.xpos",			"15"			},
  { "custom_235.EDITOR.ypos",			"13"			},

  { "custom_236",				"RocksElements.pcx"	},
  { "custom_236.xpos",				"7"			},
  { "custom_236.ypos",				"9"			},
  { "custom_236.frames",			"1"			},
  { "custom_236.EDITOR",			"RocksElements.pcx"	},
  { "custom_236.EDITOR.xpos",			"15"			},
  { "custom_236.EDITOR.ypos",			"13"			},

  { "custom_237",				"RocksElements.pcx"	},
  { "custom_237.xpos",				"7"			},
  { "custom_237.ypos",				"9"			},
  { "custom_237.frames",			"1"			},
  { "custom_237.EDITOR",			"RocksElements.pcx"	},
  { "custom_237.EDITOR.xpos",			"15"			},
  { "custom_237.EDITOR.ypos",			"13"			},

  { "custom_238",				"RocksElements.pcx"	},
  { "custom_238.xpos",				"7"			},
  { "custom_238.ypos",				"9"			},
  { "custom_238.frames",			"1"			},
  { "custom_238.EDITOR",			"RocksElements.pcx"	},
  { "custom_238.EDITOR.xpos",			"15"			},
  { "custom_238.EDITOR.ypos",			"13"			},

  { "custom_239",				"RocksElements.pcx"	},
  { "custom_239.xpos",				"7"			},
  { "custom_239.ypos",				"9"			},
  { "custom_239.frames",			"1"			},
  { "custom_239.EDITOR",			"RocksElements.pcx"	},
  { "custom_239.EDITOR.xpos",			"15"			},
  { "custom_239.EDITOR.ypos",			"13"			},

  { "custom_240",				"RocksElements.pcx"	},
  { "custom_240.xpos",				"7"			},
  { "custom_240.ypos",				"9"			},
  { "custom_240.frames",			"1"			},
  { "custom_240.EDITOR",			"RocksElements.pcx"	},
  { "custom_240.EDITOR.xpos",			"15"			},
  { "custom_240.EDITOR.ypos",			"13"			},

  { "custom_241",				"RocksElements.pcx"	},
  { "custom_241.xpos",				"7"			},
  { "custom_241.ypos",				"9"			},
  { "custom_241.frames",			"1"			},
  { "custom_241.EDITOR",			"RocksElements.pcx"	},
  { "custom_241.EDITOR.xpos",			"15"			},
  { "custom_241.EDITOR.ypos",			"13"			},

  { "custom_242",				"RocksElements.pcx"	},
  { "custom_242.xpos",				"7"			},
  { "custom_242.ypos",				"9"			},
  { "custom_242.frames",			"1"			},
  { "custom_242.EDITOR",			"RocksElements.pcx"	},
  { "custom_242.EDITOR.xpos",			"15"			},
  { "custom_242.EDITOR.ypos",			"13"			},

  { "custom_243",				"RocksElements.pcx"	},
  { "custom_243.xpos",				"7"			},
  { "custom_243.ypos",				"9"			},
  { "custom_243.frames",			"1"			},
  { "custom_243.EDITOR",			"RocksElements.pcx"	},
  { "custom_243.EDITOR.xpos",			"15"			},
  { "custom_243.EDITOR.ypos",			"13"			},

  { "custom_244",				"RocksElements.pcx"	},
  { "custom_244.xpos",				"7"			},
  { "custom_244.ypos",				"9"			},
  { "custom_244.frames",			"1"			},
  { "custom_244.EDITOR",			"RocksElements.pcx"	},
  { "custom_244.EDITOR.xpos",			"15"			},
  { "custom_244.EDITOR.ypos",			"13"			},

  { "custom_245",				"RocksElements.pcx"	},
  { "custom_245.xpos",				"7"			},
  { "custom_245.ypos",				"9"			},
  { "custom_245.frames",			"1"			},
  { "custom_245.EDITOR",			"RocksElements.pcx"	},
  { "custom_245.EDITOR.xpos",			"15"			},
  { "custom_245.EDITOR.ypos",			"13"			},

  { "custom_246",				"RocksElements.pcx"	},
  { "custom_246.xpos",				"7"			},
  { "custom_246.ypos",				"9"			},
  { "custom_246.frames",			"1"			},
  { "custom_246.EDITOR",			"RocksElements.pcx"	},
  { "custom_246.EDITOR.xpos",			"15"			},
  { "custom_246.EDITOR.ypos",			"13"			},

  { "custom_247",				"RocksElements.pcx"	},
  { "custom_247.xpos",				"7"			},
  { "custom_247.ypos",				"9"			},
  { "custom_247.frames",			"1"			},
  { "custom_247.EDITOR",			"RocksElements.pcx"	},
  { "custom_247.EDITOR.xpos",			"15"			},
  { "custom_247.EDITOR.ypos",			"13"			},

  { "custom_248",				"RocksElements.pcx"	},
  { "custom_248.xpos",				"7"			},
  { "custom_248.ypos",				"9"			},
  { "custom_248.frames",			"1"			},
  { "custom_248.EDITOR",			"RocksElements.pcx"	},
  { "custom_248.EDITOR.xpos",			"15"			},
  { "custom_248.EDITOR.ypos",			"13"			},

  { "custom_249",				"RocksElements.pcx"	},
  { "custom_249.xpos",				"7"			},
  { "custom_249.ypos",				"9"			},
  { "custom_249.frames",			"1"			},
  { "custom_249.EDITOR",			"RocksElements.pcx"	},
  { "custom_249.EDITOR.xpos",			"15"			},
  { "custom_249.EDITOR.ypos",			"13"			},

  { "custom_250",				"RocksElements.pcx"	},
  { "custom_250.xpos",				"7"			},
  { "custom_250.ypos",				"9"			},
  { "custom_250.frames",			"1"			},
  { "custom_250.EDITOR",			"RocksElements.pcx"	},
  { "custom_250.EDITOR.xpos",			"15"			},
  { "custom_250.EDITOR.ypos",			"13"			},

  { "custom_251",				"RocksElements.pcx"	},
  { "custom_251.xpos",				"7"			},
  { "custom_251.ypos",				"9"			},
  { "custom_251.frames",			"1"			},
  { "custom_251.EDITOR",			"RocksElements.pcx"	},
  { "custom_251.EDITOR.xpos",			"15"			},
  { "custom_251.EDITOR.ypos",			"13"			},

  { "custom_252",				"RocksElements.pcx"	},
  { "custom_252.xpos",				"7"			},
  { "custom_252.ypos",				"9"			},
  { "custom_252.frames",			"1"			},
  { "custom_252.EDITOR",			"RocksElements.pcx"	},
  { "custom_252.EDITOR.xpos",			"15"			},
  { "custom_252.EDITOR.ypos",			"13"			},

  { "custom_253",				"RocksElements.pcx"	},
  { "custom_253.xpos",				"7"			},
  { "custom_253.ypos",				"9"			},
  { "custom_253.frames",			"1"			},
  { "custom_253.EDITOR",			"RocksElements.pcx"	},
  { "custom_253.EDITOR.xpos",			"15"			},
  { "custom_253.EDITOR.ypos",			"13"			},

  { "custom_254",				"RocksElements.pcx"	},
  { "custom_254.xpos",				"7"			},
  { "custom_254.ypos",				"9"			},
  { "custom_254.frames",			"1"			},
  { "custom_254.EDITOR",			"RocksElements.pcx"	},
  { "custom_254.EDITOR.xpos",			"15"			},
  { "custom_254.EDITOR.ypos",			"13"			},

  { "custom_255",				"RocksElements.pcx"	},
  { "custom_255.xpos",				"7"			},
  { "custom_255.ypos",				"9"			},
  { "custom_255.frames",			"1"			},
  { "custom_255.EDITOR",			"RocksElements.pcx"	},
  { "custom_255.EDITOR.xpos",			"15"			},
  { "custom_255.EDITOR.ypos",			"13"			},

  { "custom_256",				"RocksElements.pcx"	},
  { "custom_256.xpos",				"7"			},
  { "custom_256.ypos",				"9"			},
  { "custom_256.frames",			"1"			},
  { "custom_256.EDITOR",			"RocksElements.pcx"	},
  { "custom_256.EDITOR.xpos",			"15"			},
  { "custom_256.EDITOR.ypos",			"13"			},


#endif	/* CONF_CUS_C */
