/***********************************************************
* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
*----------------------------------------------------------*
* (c) 1995-2006 Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* conf_chr.c                                               *
***********************************************************/

/* ----- this file was automatically generated -- do not edit by hand ----- */

#ifndef CONF_CHR_C
#define CONF_CHR_C

/* values for graphics configuration (character elements) */

  { "char_space",				"RocksFontEM.pcx"	},
  { "char_space.xpos",				"0"			},
  { "char_space.ypos",				"0"			},
  { "char_space.frames",			"1"			},
  { "char_space.EDITOR",			"RocksFontEM.pcx"	},
  { "char_space.EDITOR.xpos",			"7"			},
  { "char_space.EDITOR.ypos",			"4"			},
  { "char_space.EDITOR.frames",			"1"			},

  { "char_exclam",				"RocksFontEM.pcx"	},
  { "char_exclam.xpos",				"1"			},
  { "char_exclam.ypos",				"0"			},
  { "char_exclam.frames",			"1"			},

  { "char_quotedbl",				"RocksFontEM.pcx"	},
  { "char_quotedbl.xpos",			"2"			},
  { "char_quotedbl.ypos",			"0"			},
  { "char_quotedbl.frames",			"1"			},

  { "char_numbersign",				"RocksFontEM.pcx"	},
  { "char_numbersign.xpos",			"3"			},
  { "char_numbersign.ypos",			"0"			},
  { "char_numbersign.frames",			"1"			},

  { "char_dollar",				"RocksFontEM.pcx"	},
  { "char_dollar.xpos",				"4"			},
  { "char_dollar.ypos",				"0"			},
  { "char_dollar.frames",			"1"			},

  { "char_percent",				"RocksFontEM.pcx"	},
  { "char_percent.xpos",			"5"			},
  { "char_percent.ypos",			"0"			},
  { "char_percent.frames",			"1"			},

  { "char_ampersand",				"RocksFontEM.pcx"	},
  { "char_ampersand.xpos",			"6"			},
  { "char_ampersand.ypos",			"0"			},
  { "char_ampersand.frames",			"1"			},

  { "char_apostrophe",				"RocksFontEM.pcx"	},
  { "char_apostrophe.xpos",			"7"			},
  { "char_apostrophe.ypos",			"0"			},
  { "char_apostrophe.frames",			"1"			},

  { "char_parenleft",				"RocksFontEM.pcx"	},
  { "char_parenleft.xpos",			"8"			},
  { "char_parenleft.ypos",			"0"			},
  { "char_parenleft.frames",			"1"			},

  { "char_parenright",				"RocksFontEM.pcx"	},
  { "char_parenright.xpos",			"9"			},
  { "char_parenright.ypos",			"0"			},
  { "char_parenright.frames",			"1"			},

  { "char_asterisk",				"RocksFontEM.pcx"	},
  { "char_asterisk.xpos",			"10"			},
  { "char_asterisk.ypos",			"0"			},
  { "char_asterisk.frames",			"1"			},

  { "char_plus",				"RocksFontEM.pcx"	},
  { "char_plus.xpos",				"11"			},
  { "char_plus.ypos",				"0"			},
  { "char_plus.frames",				"1"			},

  { "char_comma",				"RocksFontEM.pcx"	},
  { "char_comma.xpos",				"12"			},
  { "char_comma.ypos",				"0"			},
  { "char_comma.frames",			"1"			},

  { "char_minus",				"RocksFontEM.pcx"	},
  { "char_minus.xpos",				"13"			},
  { "char_minus.ypos",				"0"			},
  { "char_minus.frames",			"1"			},

  { "char_period",				"RocksFontEM.pcx"	},
  { "char_period.xpos",				"14"			},
  { "char_period.ypos",				"0"			},
  { "char_period.frames",			"1"			},

  { "char_slash",				"RocksFontEM.pcx"	},
  { "char_slash.xpos",				"15"			},
  { "char_slash.ypos",				"0"			},
  { "char_slash.frames",			"1"			},

  { "char_0",					"RocksFontEM.pcx"	},
  { "char_0.xpos",				"0"			},
  { "char_0.ypos",				"1"			},
  { "char_0.frames",				"1"			},

  { "char_1",					"RocksFontEM.pcx"	},
  { "char_1.xpos",				"1"			},
  { "char_1.ypos",				"1"			},
  { "char_1.frames",				"1"			},

  { "char_2",					"RocksFontEM.pcx"	},
  { "char_2.xpos",				"2"			},
  { "char_2.ypos",				"1"			},
  { "char_2.frames",				"1"			},

  { "char_3",					"RocksFontEM.pcx"	},
  { "char_3.xpos",				"3"			},
  { "char_3.ypos",				"1"			},
  { "char_3.frames",				"1"			},

  { "char_4",					"RocksFontEM.pcx"	},
  { "char_4.xpos",				"4"			},
  { "char_4.ypos",				"1"			},
  { "char_4.frames",				"1"			},

  { "char_5",					"RocksFontEM.pcx"	},
  { "char_5.xpos",				"5"			},
  { "char_5.ypos",				"1"			},
  { "char_5.frames",				"1"			},

  { "char_6",					"RocksFontEM.pcx"	},
  { "char_6.xpos",				"6"			},
  { "char_6.ypos",				"1"			},
  { "char_6.frames",				"1"			},

  { "char_7",					"RocksFontEM.pcx"	},
  { "char_7.xpos",				"7"			},
  { "char_7.ypos",				"1"			},
  { "char_7.frames",				"1"			},

  { "char_8",					"RocksFontEM.pcx"	},
  { "char_8.xpos",				"8"			},
  { "char_8.ypos",				"1"			},
  { "char_8.frames",				"1"			},

  { "char_9",					"RocksFontEM.pcx"	},
  { "char_9.xpos",				"9"			},
  { "char_9.ypos",				"1"			},
  { "char_9.frames",				"1"			},

  { "char_colon",				"RocksFontEM.pcx"	},
  { "char_colon.xpos",				"10"			},
  { "char_colon.ypos",				"1"			},
  { "char_colon.frames",			"1"			},

  { "char_semicolon",				"RocksFontEM.pcx"	},
  { "char_semicolon.xpos",			"11"			},
  { "char_semicolon.ypos",			"1"			},
  { "char_semicolon.frames",			"1"			},

  { "char_less",				"RocksFontEM.pcx"	},
  { "char_less.xpos",				"12"			},
  { "char_less.ypos",				"1"			},
  { "char_less.frames",				"1"			},

  { "char_equal",				"RocksFontEM.pcx"	},
  { "char_equal.xpos",				"13"			},
  { "char_equal.ypos",				"1"			},
  { "char_equal.frames",			"1"			},

  { "char_greater",				"RocksFontEM.pcx"	},
  { "char_greater.xpos",			"14"			},
  { "char_greater.ypos",			"1"			},
  { "char_greater.frames",			"1"			},

  { "char_question",				"RocksFontEM.pcx"	},
  { "char_question.xpos",			"15"			},
  { "char_question.ypos",			"1"			},
  { "char_question.frames",			"1"			},

  { "char_at",					"RocksFontEM.pcx"	},
  { "char_at.xpos",				"0"			},
  { "char_at.ypos",				"2"			},
  { "char_at.frames",				"1"			},

  { "char_a",					"RocksFontEM.pcx"	},
  { "char_a.xpos",				"1"			},
  { "char_a.ypos",				"2"			},
  { "char_a.frames",				"1"			},

  { "char_b",					"RocksFontEM.pcx"	},
  { "char_b.xpos",				"2"			},
  { "char_b.ypos",				"2"			},
  { "char_b.frames",				"1"			},

  { "char_c",					"RocksFontEM.pcx"	},
  { "char_c.xpos",				"3"			},
  { "char_c.ypos",				"2"			},
  { "char_c.frames",				"1"			},

  { "char_d",					"RocksFontEM.pcx"	},
  { "char_d.xpos",				"4"			},
  { "char_d.ypos",				"2"			},
  { "char_d.frames",				"1"			},

  { "char_e",					"RocksFontEM.pcx"	},
  { "char_e.xpos",				"5"			},
  { "char_e.ypos",				"2"			},
  { "char_e.frames",				"1"			},

  { "char_f",					"RocksFontEM.pcx"	},
  { "char_f.xpos",				"6"			},
  { "char_f.ypos",				"2"			},
  { "char_f.frames",				"1"			},

  { "char_g",					"RocksFontEM.pcx"	},
  { "char_g.xpos",				"7"			},
  { "char_g.ypos",				"2"			},
  { "char_g.frames",				"1"			},

  { "char_h",					"RocksFontEM.pcx"	},
  { "char_h.xpos",				"8"			},
  { "char_h.ypos",				"2"			},
  { "char_h.frames",				"1"			},

  { "char_i",					"RocksFontEM.pcx"	},
  { "char_i.xpos",				"9"			},
  { "char_i.ypos",				"2"			},
  { "char_i.frames",				"1"			},

  { "char_j",					"RocksFontEM.pcx"	},
  { "char_j.xpos",				"10"			},
  { "char_j.ypos",				"2"			},
  { "char_j.frames",				"1"			},

  { "char_k",					"RocksFontEM.pcx"	},
  { "char_k.xpos",				"11"			},
  { "char_k.ypos",				"2"			},
  { "char_k.frames",				"1"			},

  { "char_l",					"RocksFontEM.pcx"	},
  { "char_l.xpos",				"12"			},
  { "char_l.ypos",				"2"			},
  { "char_l.frames",				"1"			},

  { "char_m",					"RocksFontEM.pcx"	},
  { "char_m.xpos",				"13"			},
  { "char_m.ypos",				"2"			},
  { "char_m.frames",				"1"			},

  { "char_n",					"RocksFontEM.pcx"	},
  { "char_n.xpos",				"14"			},
  { "char_n.ypos",				"2"			},
  { "char_n.frames",				"1"			},

  { "char_o",					"RocksFontEM.pcx"	},
  { "char_o.xpos",				"15"			},
  { "char_o.ypos",				"2"			},
  { "char_o.frames",				"1"			},

  { "char_p",					"RocksFontEM.pcx"	},
  { "char_p.xpos",				"0"			},
  { "char_p.ypos",				"3"			},
  { "char_p.frames",				"1"			},

  { "char_q",					"RocksFontEM.pcx"	},
  { "char_q.xpos",				"1"			},
  { "char_q.ypos",				"3"			},
  { "char_q.frames",				"1"			},

  { "char_r",					"RocksFontEM.pcx"	},
  { "char_r.xpos",				"2"			},
  { "char_r.ypos",				"3"			},
  { "char_r.frames",				"1"			},

  { "char_s",					"RocksFontEM.pcx"	},
  { "char_s.xpos",				"3"			},
  { "char_s.ypos",				"3"			},
  { "char_s.frames",				"1"			},

  { "char_t",					"RocksFontEM.pcx"	},
  { "char_t.xpos",				"4"			},
  { "char_t.ypos",				"3"			},
  { "char_t.frames",				"1"			},

  { "char_u",					"RocksFontEM.pcx"	},
  { "char_u.xpos",				"5"			},
  { "char_u.ypos",				"3"			},
  { "char_u.frames",				"1"			},

  { "char_v",					"RocksFontEM.pcx"	},
  { "char_v.xpos",				"6"			},
  { "char_v.ypos",				"3"			},
  { "char_v.frames",				"1"			},

  { "char_w",					"RocksFontEM.pcx"	},
  { "char_w.xpos",				"7"			},
  { "char_w.ypos",				"3"			},
  { "char_w.frames",				"1"			},

  { "char_x",					"RocksFontEM.pcx"	},
  { "char_x.xpos",				"8"			},
  { "char_x.ypos",				"3"			},
  { "char_x.frames",				"1"			},

  { "char_y",					"RocksFontEM.pcx"	},
  { "char_y.xpos",				"9"			},
  { "char_y.ypos",				"3"			},
  { "char_y.frames",				"1"			},

  { "char_z",					"RocksFontEM.pcx"	},
  { "char_z.xpos",				"10"			},
  { "char_z.ypos",				"3"			},
  { "char_z.frames",				"1"			},

  { "char_bracketleft",				"RocksFontEM.pcx"	},
  { "char_bracketleft.xpos",			"11"			},
  { "char_bracketleft.ypos",			"3"			},
  { "char_bracketleft.frames",			"1"			},

  { "char_backslash",				"RocksFontEM.pcx"	},
  { "char_backslash.xpos",			"12"			},
  { "char_backslash.ypos",			"3"			},
  { "char_backslash.frames",			"1"			},

  { "char_bracketright",			"RocksFontEM.pcx"	},
  { "char_bracketright.xpos",			"13"			},
  { "char_bracketright.ypos",			"3"			},
  { "char_bracketright.frames",			"1"			},

  { "char_asciicircum",				"RocksFontEM.pcx"	},
  { "char_asciicircum.xpos",			"14"			},
  { "char_asciicircum.ypos",			"3"			},
  { "char_asciicircum.frames",			"1"			},

  { "char_underscore",				"RocksFontEM.pcx"	},
  { "char_underscore.xpos",			"15"			},
  { "char_underscore.ypos",			"3"			},
  { "char_underscore.frames",			"1"			},

  { "char_copyright",				"RocksFontEM.pcx"	},
  { "char_copyright.xpos",			"0"			},
  { "char_copyright.ypos",			"4"			},
  { "char_copyright.frames",			"1"			},

  { "char_aumlaut",				"RocksFontEM.pcx"	},
  { "char_aumlaut.xpos",			"1"			},
  { "char_aumlaut.ypos",			"4"			},
  { "char_aumlaut.frames",			"1"			},

  { "char_oumlaut",				"RocksFontEM.pcx"	},
  { "char_oumlaut.xpos",			"2"			},
  { "char_oumlaut.ypos",			"4"			},
  { "char_oumlaut.frames",			"1"			},

  { "char_uumlaut",				"RocksFontEM.pcx"	},
  { "char_uumlaut.xpos",			"3"			},
  { "char_uumlaut.ypos",			"4"			},
  { "char_uumlaut.frames",			"1"			},

  { "char_degree",				"RocksFontEM.pcx"	},
  { "char_degree.xpos",				"4"			},
  { "char_degree.ypos",				"4"			},
  { "char_degree.frames",			"1"			},

  { "char_trademark",				"RocksFontEM.pcx"	},
  { "char_trademark.xpos",			"5"			},
  { "char_trademark.ypos",			"4"			},
  { "char_trademark.frames",			"1"			},

  { "char_cursor",				"RocksFontEM.pcx"	},
  { "char_cursor.xpos",				"6"			},
  { "char_cursor.ypos",				"4"			},
  { "char_cursor.frames",			"1"			},


#endif	/* CONF_CHR_C */
