/***********************************************************
* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
*----------------------------------------------------------*
* (c) 1995-2006 Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* conf_chr.h                                               *
***********************************************************/

/* ----- this file was automatically generated -- do not edit by hand ----- */

#ifndef CONF_CHR_H
#define CONF_CHR_H

/* values for elements configuration (character elements) */

#define EL_CHAR_SPACE			(EL_CHAR_ASCII0 + 32)
#define EL_CHAR_EXCLAM			(EL_CHAR_ASCII0 + 33)
#define EL_CHAR_QUOTEDBL		(EL_CHAR_ASCII0 + 34)
#define EL_CHAR_NUMBERSIGN		(EL_CHAR_ASCII0 + 35)
#define EL_CHAR_DOLLAR			(EL_CHAR_ASCII0 + 36)
#define EL_CHAR_PERCENT			(EL_CHAR_ASCII0 + 37)
#define EL_CHAR_AMPERSAND		(EL_CHAR_ASCII0 + 38)
#define EL_CHAR_APOSTROPHE		(EL_CHAR_ASCII0 + 39)
#define EL_CHAR_PARENLEFT		(EL_CHAR_ASCII0 + 40)
#define EL_CHAR_PARENRIGHT		(EL_CHAR_ASCII0 + 41)
#define EL_CHAR_ASTERISK		(EL_CHAR_ASCII0 + 42)
#define EL_CHAR_PLUS			(EL_CHAR_ASCII0 + 43)
#define EL_CHAR_COMMA			(EL_CHAR_ASCII0 + 44)
#define EL_CHAR_MINUS			(EL_CHAR_ASCII0 + 45)
#define EL_CHAR_PERIOD			(EL_CHAR_ASCII0 + 46)
#define EL_CHAR_SLASH			(EL_CHAR_ASCII0 + 47)
#define EL_CHAR_0			(EL_CHAR_ASCII0 + 48)
#define EL_CHAR_1			(EL_CHAR_ASCII0 + 49)
#define EL_CHAR_2			(EL_CHAR_ASCII0 + 50)
#define EL_CHAR_3			(EL_CHAR_ASCII0 + 51)
#define EL_CHAR_4			(EL_CHAR_ASCII0 + 52)
#define EL_CHAR_5			(EL_CHAR_ASCII0 + 53)
#define EL_CHAR_6			(EL_CHAR_ASCII0 + 54)
#define EL_CHAR_7			(EL_CHAR_ASCII0 + 55)
#define EL_CHAR_8			(EL_CHAR_ASCII0 + 56)
#define EL_CHAR_9			(EL_CHAR_ASCII0 + 57)
#define EL_CHAR_COLON			(EL_CHAR_ASCII0 + 58)
#define EL_CHAR_SEMICOLON		(EL_CHAR_ASCII0 + 59)
#define EL_CHAR_LESS			(EL_CHAR_ASCII0 + 60)
#define EL_CHAR_EQUAL			(EL_CHAR_ASCII0 + 61)
#define EL_CHAR_GREATER			(EL_CHAR_ASCII0 + 62)
#define EL_CHAR_QUESTION		(EL_CHAR_ASCII0 + 63)
#define EL_CHAR_AT			(EL_CHAR_ASCII0 + 64)
#define EL_CHAR_A			(EL_CHAR_ASCII0 + 65)
#define EL_CHAR_B			(EL_CHAR_ASCII0 + 66)
#define EL_CHAR_C			(EL_CHAR_ASCII0 + 67)
#define EL_CHAR_D			(EL_CHAR_ASCII0 + 68)
#define EL_CHAR_E			(EL_CHAR_ASCII0 + 69)
#define EL_CHAR_F			(EL_CHAR_ASCII0 + 70)
#define EL_CHAR_G			(EL_CHAR_ASCII0 + 71)
#define EL_CHAR_H			(EL_CHAR_ASCII0 + 72)
#define EL_CHAR_I			(EL_CHAR_ASCII0 + 73)
#define EL_CHAR_J			(EL_CHAR_ASCII0 + 74)
#define EL_CHAR_K			(EL_CHAR_ASCII0 + 75)
#define EL_CHAR_L			(EL_CHAR_ASCII0 + 76)
#define EL_CHAR_M			(EL_CHAR_ASCII0 + 77)
#define EL_CHAR_N			(EL_CHAR_ASCII0 + 78)
#define EL_CHAR_O			(EL_CHAR_ASCII0 + 79)
#define EL_CHAR_P			(EL_CHAR_ASCII0 + 80)
#define EL_CHAR_Q			(EL_CHAR_ASCII0 + 81)
#define EL_CHAR_R			(EL_CHAR_ASCII0 + 82)
#define EL_CHAR_S			(EL_CHAR_ASCII0 + 83)
#define EL_CHAR_T			(EL_CHAR_ASCII0 + 84)
#define EL_CHAR_U			(EL_CHAR_ASCII0 + 85)
#define EL_CHAR_V			(EL_CHAR_ASCII0 + 86)
#define EL_CHAR_W			(EL_CHAR_ASCII0 + 87)
#define EL_CHAR_X			(EL_CHAR_ASCII0 + 88)
#define EL_CHAR_Y			(EL_CHAR_ASCII0 + 89)
#define EL_CHAR_Z			(EL_CHAR_ASCII0 + 90)
#define EL_CHAR_BRACKETLEFT		(EL_CHAR_ASCII0 + 91)
#define EL_CHAR_BACKSLASH		(EL_CHAR_ASCII0 + 92)
#define EL_CHAR_BRACKETRIGHT		(EL_CHAR_ASCII0 + 93)
#define EL_CHAR_ASCIICIRCUM		(EL_CHAR_ASCII0 + 94)
#define EL_CHAR_UNDERSCORE		(EL_CHAR_ASCII0 + 95)
#define EL_CHAR_COPYRIGHT		(EL_CHAR_ASCII0 + 96)
#define EL_CHAR_AUMLAUT			(EL_CHAR_ASCII0 + 97)
#define EL_CHAR_OUMLAUT			(EL_CHAR_ASCII0 + 98)
#define EL_CHAR_UUMLAUT			(EL_CHAR_ASCII0 + 99)
#define EL_CHAR_DEGREE			(EL_CHAR_ASCII0 + 100)
#define EL_CHAR_TRADEMARK		(EL_CHAR_ASCII0 + 101)
#define EL_CHAR_CURSOR			(EL_CHAR_ASCII0 + 102)

#endif	/* CONF_CHR_C */
