/***********************************************************
* Artsoft Retro-Game Library                               *
*----------------------------------------------------------*
* (c) 1994-2006 Artsoft Entertainment                      *
*               Holger Schemel                             *
*               Detmolder Strasse 189                      *
*               33604 Bielefeld                            *
*               Germany                                    *
*               e-mail: info@artsoft.org                   *
*----------------------------------------------------------*
* libgame.h                                                *
***********************************************************/

#ifndef LIBGAME_H
#define LIBGAME_H

#define LIBGAME_VERSION_1_0_0

#include "platform.h"
#include "types.h"
#include "system.h"
#include "random.h"
#include "gadgets.h"
#include "text.h"
#include "sound.h"
#include "joystick.h"
#include "toons.h"
#include "image.h"
#include "pcx.h"
#include "setup.h"
#include "misc.h"

#endif /* LIBGAME_H */
